{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFCustomSavePoint.pas - TghCustomSavePoint class unit.                 }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFCustomSavePoint;  { Custom Save Point }

Interface

  Uses
    Classes, GHFObjList;

  Type
    TghCustomSavePoint = Class;

    { Custom Save Point Event type }
    TghCustomSavePointEvent = Procedure (Point :TghCustomSavePoint) Of
      Object;

    { Save point action type }
    TghSavePointAction = (ghsaNone, ghsaConfirm, ghsaRestore);

    TghCustomSavePoints = Class;

    { Custom Save Point class }
    TghCustomSavePoint = Class
      Protected
        { Fields }

        FConfirming :Boolean;
        FOwner :TghCustomSavePoints;
        FRestoring :Boolean;
        FSaved :Boolean;

        ///pendiente/estudiar
        //OwnedObjs :TghObjList;

        { Static methods }
        Procedure CheckConfirm;
        Function GetFinalizing :Boolean;
        Function GetGlobalIndex :Integer;
        Function GetIndex :Integer;

        { Virtual methods }
        Procedure InternalConfirm; Virtual;
        Procedure InternalDestroy; Virtual;
        Procedure InternalRelease; Virtual;
        Procedure InternalRestore; Virtual; Abstract;
        Procedure InternalSave; Virtual; Abstract;
      Public
        Constructor Create (AOwner :TghCustomSavePoints); Virtual;
        Destructor Destroy; Override;

        { Static methods }
        Procedure Confirm (Const Resave :Boolean = False);
        Procedure Finalize (Const AConfirm :Boolean);
        Procedure Restore (Const KeepSaved :Boolean = False);
        Procedure Save;

        { Virtual methods }
        Procedure Invalidate (
          DetailAction :TghSavePointAction = ghsaNone); Virtual;

        { Overridden methods }
        Procedure AfterConstruction; Override;

        { Properties }
        Property Confirming :Boolean Read FConfirming;
        Property Finalizing :Boolean Read GetFinalizing;
        Property GlobalIndex :Integer Read GetGlobalIndex;
        Property Index :Integer Read GetIndex;
        Property Owner :TghCustomSavePoints Read FOwner;
        Property Restoring :Boolean Read FRestoring;
        Property Saved :Boolean Read FSaved;
    End;

    { TghCustomSavePoint class type }
    TghCustomSavePointClass = Class Of TghCustomSavePoint;

    { Save Points List class }
    TghSavePointList = Class (TghObjList)
      Protected
        { Static methods }
        Function GetFirst :TghCustomSavePoint;
        Function GetItem (Const Index :Integer): TghCustomSavePoint;
        Function GetLast :TghCustomSavePoint;
        Procedure SetItem (Const Index :Integer;
          Const Value :TghCustomSavePoint);
      Public
        { Static methods }
        Procedure Confirm (Const StartIndex :Integer);
        Procedure Restore (Const StartIndex :Integer);
        Procedure Terminate (Const StartIndex :Integer;
          Const AConfirm :Boolean);

        { Properties }

        {$Warn Hiding_Member Off}
        Property First :TghCustomSavePoint Read GetFirst;
        {$Warn Hiding_Member On}

        Property Items [Const Index :Integer] :TghCustomSavePoint
          Read GetItem Write SetItem; Default;

        {$Warn Hiding_Member Off}
        Property Last :TghCustomSavePoint Read GetLast;
        {$Warn Hiding_Member On}
    End;

    { Custom Save Points class }
    TghCustomSavePoints = Class (TComponent)
      Protected
        List :TghSavePointList;

        { Static methods }
        Function GetCount :Integer;
        Function GetFirst :TghCustomSavePoint;
        Function GetGlobalCount :Integer;
        Function GetItem (Const Index :Integer): TghCustomSavePoint;
        Function GetLast :TghCustomSavePoint;
        Function Invalidate (Const StartIndex :Integer = 0;
          Const DetailAction :TghSavePointAction = ghsaNone) :Integer;
          Overload;
        Function Invalidate (Const DetailAction :TghSavePointAction)
          :Integer; Overload;
        Function InvalidateGlobal (Const StartIndex :Integer;
          Const DetailAction :TghSavePointAction = ghsaNone) :Integer;

        { Virtual methods }
        Procedure DoAfterConfirm (Point :TghCustomSavePoint); Virtual;
        Procedure DoAfterRestore (Point :TghCustomSavePoint); Virtual;
        Procedure DoAfterSave (Point :TghCustomSavePoint); Virtual;
        Procedure DoBeforeConfirm (Point :TghCustomSavePoint); Virtual;
        Procedure DoBeforeRestore (Point :TghCustomSavePoint); Virtual;
        Procedure DoBeforeSave (Point :TghCustomSavePoint); Virtual;
        Function NewPointClass :TghCustomSavePointClass; Virtual; Abstract;
      Public
        Constructor Create (AOwner :TComponent); Override;
        Destructor Destroy; Override;

        { Static methods }
        Function CheckDetail :Boolean;
        Function HasMaster :Boolean;
        Function IndexOf (Const Point :TghCustomSavePoint) :Integer;
        Function Save :TghCustomSavePoint;

        { Properties }
        Property Count :Integer Read GetCount;
        Property First :TghCustomSavePoint Read GetFirst;
        Property GlobalCount :Integer Read GetGlobalCount;
        Property Items [Const Index :Integer] :TghCustomSavePoint
          Read GetItem; Default;
        Property Last :TghCustomSavePoint Read GetLast;
    End;

  { Functions and procedures }

  Function ghSavePoints :TghSavePointList;

Implementation

  Uses
    SysUtils, GHFRTL;

  ThreadVar
    SavePoints :TghSavePointList;

  { TghCustomSavePoint }

  Constructor TghCustomSavePoint.Create (AOwner :TghCustomSavePoints);
  Begin
    FOwner := AOwner;
    Owner.List.Add (Self);

    If SavePoints = Nil Then
      SavePoints := TghSavePointList.Create (False);

    SavePoints.Add (Self);
  End;

  Destructor TghCustomSavePoint.Destroy;
  Begin
    CheckConfirm;
    InternalDestroy;
    SavePoints.Remove (Self);

    If SavePoints.Count = 0 Then
      FreeAndNil (SavePoints);

    Owner.List.Extract (Self);
  End;

  { Protected static methods }

  Procedure TghCustomSavePoint.CheckConfirm;
  Begin
    If Not Saved Or Finalizing Then
      Exit;

    FConfirming := True;

    Try
      Owner.DoBeforeConfirm (Self);
      InternalConfirm;
      Invalidate (ghsaConfirm);
      Owner.DoAfterConfirm (Self);
    Finally
      FConfirming := False;
    End;
  End;

  Function TghCustomSavePoint.GetFinalizing :Boolean;
  Begin
    Result := Confirming Or Restoring;
  End;

  Function TghCustomSavePoint.GetGlobalIndex :Integer;
  Begin
    Result := SavePoints.IndexOf (Self);
  End;

  Function TghCustomSavePoint.GetIndex :Integer;
  Begin
    Result := Owner.List.IndexOf (Self);
  End;

  { Protected virtual methods }

  Procedure TghCustomSavePoint.InternalConfirm;
  Begin
  End;

  Procedure TghCustomSavePoint.InternalDestroy;
  Begin
  End;

  Procedure TghCustomSavePoint.InternalRelease;
  Begin
  End;

  { Public static methods }

  Procedure TghCustomSavePoint.Confirm (Const Resave :Boolean = False);
  Begin
    If Finalizing Then
      Exit;
      
    CheckConfirm;

    If Resave Then
      Save
    Else
      Free;
  End;

  Procedure TghCustomSavePoint.Finalize (Const AConfirm :Boolean);
  Begin
    If AConfirm Then
      Confirm
    Else
      Restore;
  End;

  Procedure TghCustomSavePoint.Restore (Const KeepSaved :Boolean = False);
  Begin
    If Finalizing Then
      Exit;

    If Saved Then
    Begin
      FRestoring := True;

      Try
        Owner.DoBeforeRestore (Self);
        InternalRestore;
        Owner.Invalidate (Index + 1);  { Delete following save points }

        { Restore global detail save points }
        SavePoints.Restore (GlobalIndex + 1);

        Owner.DoAfterRestore (Self);
      Finally
        FRestoring := False;
      End;
    End;

    If KeepSaved Then
      If Saved Then
        Owner.Invalidate (Index + 1)
      Else
        Save
    Else
    Begin
      If Saved Then
        InternalRelease;

      Invalidate;
    End;
  End;

  Procedure TghCustomSavePoint.Save;
  Begin
    CheckConfirm;
    Owner.DoBeforeSave (Self);
    InternalSave;
    FSaved := True;
    Owner.DoAfterSave (Self);
  End;

  { Public virtual methods }

  Procedure TghCustomSavePoint.Invalidate (
    DetailAction :TghSavePointAction = ghsaNone);
  Begin
    If Saved Then
    Begin
      FSaved := False;
      Owner.Invalidate (Index + 1);  { Delete following save points }

      If DetailAction <> ghsaNone Then
        { Confirm or restore global detail save points }
        SavePoints.Terminate (GlobalIndex + 1,
          DetailAction = ghsaConfirm);
    End;

    If Not Finalizing Then
      Free;
  End;

  { Public overridden methods }

  Procedure TghCustomSavePoint.AfterConstruction;
  Begin
    Inherited AfterConstruction;
    Save;
  End;

  { TghSavePointList }

  { Protected static methods }

  Function TghSavePointList.GetFirst :TghCustomSavePoint;
  Begin
    Result := TghCustomSavePoint (Inherited First ());
  End;

  Function TghSavePointList.GetItem (Const Index :Integer):
    TghCustomSavePoint;
  Begin
    Result := TghCustomSavePoint (Inherited Items [Index]);
  End;

  Function TghSavePointList.GetLast :TghCustomSavePoint;
  Begin
    Result := TghCustomSavePoint (Inherited Last ());
  End;

  Procedure TghSavePointList.SetItem (Const Index :Integer;
    Const Value :TghCustomSavePoint);
  Begin
    Inherited Items [Index] := Value;
  End;

  { Public static methods }

  Procedure TghSavePointList.Confirm (Const StartIndex :Integer);
  Begin
    Terminate (StartIndex, True);
  End;

  Procedure TghSavePointList.Restore (Const StartIndex :Integer);
  Begin
    Terminate (StartIndex, False);
  End;

  Procedure TghSavePointList.Terminate (Const StartIndex :Integer;
    Const AConfirm :Boolean);
  Var
    Method :Pointer;
  Begin
    If AConfirm Then
      Method := @TghCustomSavePoint.Confirm
    Else
      Method := @TghCustomSavePoint.Restore;

    Call (StartIndex, Method, Pointer (False));
  End;

  { TghCustomSavePoints }

  Constructor TghCustomSavePoints.Create (AOwner :TComponent);
  Begin
    Inherited Create (AOwner);
    List := TghSavePointList.Create (True);
  End;

  Destructor TghCustomSavePoints.Destroy;
  Begin
    FreeAndNil (List);
    Inherited Destroy;
  End;

  { Protected static methods }

  Function TghCustomSavePoints.GetCount :Integer;
  Begin
    Result := List.Count;
  End;

  Function TghCustomSavePoints.GetFirst :TghCustomSavePoint;
  Begin
    If Count > 0 Then
      Result := List.First
    Else
      Result := Nil;
  End;

  Function TghCustomSavePoints.GetGlobalCount :Integer;
  Begin
    If SavePoints <> Nil Then
      Result := SavePoints.Count
    Else
      Result := 0;
  End;

  Function TghCustomSavePoints.GetItem (Const Index :Integer): TghCustomSavePoint;
  Begin
    If ghIsIndex (Index, Count) Then
      Result := List [Index]
    Else
      Result := Nil;
  End;

  Function TghCustomSavePoints.GetLast :TghCustomSavePoint;
  Begin
    If Count > 0 Then
      Result := List.Last
    Else
      Result := Nil;
  End;

  { Returns the global index of the first save point to be invalidated }
  Function TghCustomSavePoints.Invalidate (Const StartIndex :Integer = 0;
    Const DetailAction :TghSavePointAction = ghsaNone) :Integer;
  Begin
    If ghIsIndex (StartIndex, Count) Then
    Begin
      Result := List [StartIndex].GlobalIndex;
      List [StartIndex].Invalidate (DetailAction);
    End
    Else
      Result := -1;
  End;

  Function TghCustomSavePoints.Invalidate (
    Const DetailAction :TghSavePointAction) :Integer;
  Begin
    Result := Invalidate (0, DetailAction);
  End;

  Function TghCustomSavePoints.InvalidateGlobal (Const StartIndex :Integer;
    Const DetailAction :TghSavePointAction = ghsaNone) :Integer;
  Var
    I :Integer;
  Begin
    For I := 0 To Count - 1 Do
      If List [I].GlobalIndex >= StartIndex Then
      Begin
        Result := Invalidate (I, DetailAction);
        Exit;
      End;

    Result := -1;
  End;

  { Protected virtual methods }

  Procedure TghCustomSavePoints.DoAfterConfirm (Point :TghCustomSavePoint);
  Begin
  End;

  Procedure TghCustomSavePoints.DoAfterRestore (Point :TghCustomSavePoint);
  Begin
  End;

  Procedure TghCustomSavePoints.DoAfterSave (Point :TghCustomSavePoint);
  Begin
  End;

  Procedure TghCustomSavePoints.DoBeforeConfirm (
    Point :TghCustomSavePoint);
  Begin
  End;

  Procedure TghCustomSavePoints.DoBeforeRestore (
    Point :TghCustomSavePoint);
  Begin
  End;

  Procedure TghCustomSavePoints.DoBeforeSave (Point :TghCustomSavePoint);
  Begin
  End;

  { Public static methods }

  Function TghCustomSavePoints.CheckDetail :Boolean;
  Begin
    If (GlobalCount > 0) And (SavePoints.Last.Owner <> Self) Then
    Begin
      Save;
      Result := True;
    End
    Else
      Result := False;
  End;

  Function TghCustomSavePoints.HasMaster :Boolean;
  Begin
    Result := (Count > 0) And (First.GlobalIndex > 0);
  End;

  Function TghCustomSavePoints.IndexOf (Const Point :TghCustomSavePoint)
    :Integer;
  Begin
    Result := List.IndexOf (Point)
  End;

  Function TghCustomSavePoints.Save :TghCustomSavePoint;
  Begin
    Result := NewPointClass ().Create (Self);
  End;

  { Functions and procedures }

  Function ghSavePoints :TghSavePointList;
  Begin
    { NOTE: This function prevents the warning "Exported package threadvar
      cannot be used outside of this package". }
    Result := SavePoints;
  End;

End.

