{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFDataSource.pas - TghDataSource class unit.                           }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFDataSource;  { Data Source }

Interface

  Uses
    DB, GHFCustomDataSetEvents, GHFDataSetEventHolder, GHFVCL, Classes,
    GHFObjList;

  Type
    { Data Source class }
    TghDataSource = Class (TDataSource)
      Protected
        Chain :TghCustomDataSetEvents;  // TChain
        FDataSetCloned :Boolean;
        FDataSetEvents :TghDataSetEventHolder;
        FOnCloneDataSet :TghCloneDataSetEvent;

        { Static methods }
        Function CheckDataSet :TDataSet;
        Function CloneDataSet (Const Value :TDataSet) :TDataSet;
        Procedure SetDataSet (Const Value :TDataSet);
        Procedure SetDataSetCloned (Const Value :Boolean);

        { Virtual methods }
        Function InternalCloneDataSet (Value :TDataSet;
          AClass :TDataSetClass) :TDataSet; Virtual;
          
        { Overridden methods }
        Procedure Notification (AComponent :TComponent;
          Operation :TOperation); Override;
      Public
        Constructor Create (AOwner :TComponent); Override;
        Destructor Destroy; Override;

        { Properties }
        Property DataLinks;
      Published
        { Properties }
        Property DataSet Write SetDataSet;
        Property DataSetCloned :Boolean Read FDataSetCloned
          Write SetDataSetCloned Default False;
        Property DataSetEvents :TghDataSetEventHolder Read FDataSetEvents;

        { Event properties }
        Property OnCloneDataSet :TghCloneDataSetEvent
          Read FOnCloneDataSet Write FOnCloneDataSet;
    End;

    TghDataSourceClass = Class Of TghDataSource;

  Var
    GHDataSources :TghObjList;

Implementation

  Uses
    SysUtils, GHFRTL;

  Type
    TChain = Class (TghCustomDataSetEvents)
      Public
        FDataSet :TDataSet;
        List :TList;

        Constructor Create (Const ADataSet :TDataSet); Reintroduce;
        Destructor Destroy; Override;

        { Static methods }
        Procedure DoListEvents (Const Method :Pointer;
          Const Sender :TDataSet); Overload;
        Procedure DoListEvents (Const Method :Pointer;
          Const Sender :TDataSet; Const E :EDatabaseError;
          Var Action :TDataAction); Overload;

        { Overridden methods }
        Procedure DoAfterCancel (Sender :TDataSet); Override;
        Procedure DoAfterClose (Sender :TDataSet); Override;
        Procedure DoAfterDelete (Sender :TDataSet); Override;
        Procedure DoAfterEdit (Sender :TDataSet); Override;
        Procedure DoAfterInsert (Sender :TDataSet); Override;
        Procedure DoAfterOpen (Sender :TDataSet); Override;
        Procedure DoAfterPost (Sender :TDataSet); Override;
        Procedure DoAfterRefresh (Sender :TDataSet); Override;
        Procedure DoAfterScroll (Sender :TDataSet); Override;
        Procedure DoBeforeCancel (Sender :TDataSet); Override;
        Procedure DoBeforeClose (Sender :TDataSet); Override;
        Procedure DoBeforeDelete (Sender :TDataSet); Override;
        Procedure DoBeforeEdit (Sender :TDataSet); Override;
        Procedure DoBeforeInsert (Sender :TDataSet); Override;
        Procedure DoBeforeOpen (Sender :TDataSet); Override;
        Procedure DoBeforePost (Sender :TDataSet); Override;
        Procedure DoBeforeRefresh (Sender :TDataSet); Override;
        Procedure DoBeforeScroll (Sender :TDataSet); Override;
        Procedure DoOnCalcFields (Sender :TDataSet); Override;
        Procedure DoOnDeleteError (Sender :TDataSet; E :EDatabaseError;
          Var Action :TDataAction); Override;
        Procedure DoOnEditError (Sender :TDataSet; E :EDatabaseError;
          Var Action :TDataAction); Override;
        Procedure DoOnFilterRecord (Sender :TDataSet; Var Accept :Boolean);
          Override;
        Procedure DoOnNewRecord (Sender :TDataSet); Override;
        Procedure DoOnPostError (Sender :TDataSet; E :EDatabaseError;
          Var Action :TDataAction); Override;
        Function GetDataSet :TDataSet; Override;
        Procedure InternalSetDataSet (Value :TDataSet); Override;
        Procedure Notification (AComponent :TComponent;
          Operation :TOperation); Override;
    End;

  Var
    ChainList :TList;

  { TChain }

  Constructor TChain.Create (Const ADataSet :TDataSet);
  Begin
    Inherited Create (Nil);
    DataSet := ADataSet;
    DataSet.FreeNotification (Self);
    List := TList.Create;

    If ChainList = Nil Then
      ChainList := TList.Create;

    ChainList.Add (Self)
  End;

  Destructor TChain.Destroy;
  Begin
    ChainList.Remove (Self);

    If ChainList.Count = 0 Then
      FreeAndNil (ChainList);

    FreeAndNil (List);
    Inherited Destroy;
  End;

  Procedure TChain.DoListEvents (Const Method :Pointer;
    Const Sender :TDataSet);
  Var
    I :Integer;
  Begin
    For I := 0 To List.Count - 1 Do
      If TghDataSource (List [I]).Enabled Then
        ghCall (TghDataSource (List [I]).DataSetEvents, Method, Sender);
  End;

  Procedure TChain.DoListEvents (Const Method :Pointer;
    Const Sender :TDataSet; Const E :EDatabaseError;
    Var Action :TDataAction);
  Var
    I :Integer;
  Begin
    For I := 0 To List.Count - 1 Do
      If TghDataSource (List [I]).Enabled Then
        ghCall (TghDataSource (List [I]).DataSetEvents, Method, Sender,
          E, @Action);
  End;

  Procedure TChain.DoAfterCancel (Sender :TDataSet);
  Begin
    Inherited DoAfterCancel (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterCancel, Sender);
  End;

  Procedure TChain.DoAfterClose (Sender :TDataSet);
  Begin
    Inherited DoAfterClose (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterClose, Sender);
  End;

  Procedure TChain.DoAfterDelete (Sender :TDataSet);
  Begin
    Inherited DoAfterDelete (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterDelete, Sender);
  End;

  Procedure TChain.DoAfterEdit (Sender :TDataSet);
  Begin
    Inherited DoAfterEdit (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterEdit, Sender);
  End;

  Procedure TChain.DoAfterInsert (Sender :TDataSet);
  Begin
    Inherited DoAfterInsert (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterInsert, Sender);
  End;

  Procedure TChain.DoAfterOpen (Sender :TDataSet);
  Begin
    Inherited DoAfterOpen (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterOpen, Sender);
  End;

  Procedure TChain.DoAfterPost (Sender :TDataSet);
  Begin
    Inherited DoAfterPost (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterPost, Sender);
  End;

  Procedure TChain.DoAfterRefresh (Sender :TDataSet);
  Begin
    Inherited DoAfterRefresh (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterRefresh, Sender);
  End;

  Procedure TChain.DoAfterScroll (Sender :TDataSet);
  Begin
    Inherited DoAfterScroll (Sender);
    DoListEvents (@TghDataSetEventHolder.DoAfterScroll, Sender);
  End;

  Procedure TChain.DoBeforeCancel (Sender :TDataSet);
  Begin
    Inherited DoBeforeCancel (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforeCancel, Sender);
  End;

  Procedure TChain.DoBeforeClose (Sender :TDataSet);
  Begin
    Inherited DoBeforeClose (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforeClose, Sender);
  End;

  Procedure TChain.DoBeforeDelete (Sender :TDataSet);
  Begin
    Inherited DoBeforeDelete (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforeDelete, Sender);
  End;

  Procedure TChain.DoBeforeEdit (Sender :TDataSet);
  Begin
    Inherited DoBeforeEdit (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforeEdit, Sender);
  End;

  Procedure TChain.DoBeforeInsert (Sender :TDataSet);
  Begin
    Inherited DoBeforeInsert (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforeInsert, Sender);
  End;

  Procedure TChain.DoBeforeOpen (Sender :TDataSet);
  Begin
    Inherited DoBeforeOpen (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforeOpen, Sender);
  End;

  Procedure TChain.DoBeforePost (Sender :TDataSet);
  Begin
    Inherited DoBeforePost (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforePost, Sender);
  End;

  Procedure TChain.DoBeforeRefresh (Sender :TDataSet);
  Begin
    Inherited DoBeforeRefresh (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforeRefresh, Sender);
  End;

  Procedure TChain.DoBeforeScroll (Sender :TDataSet);
  Begin
    Inherited DoBeforeScroll (Sender);
    DoListEvents (@TghDataSetEventHolder.DoBeforeScroll, Sender);
  End;

  Procedure TChain.DoOnCalcFields (Sender :TDataSet);
  Begin
    Inherited DoOnCalcFields (Sender);
    DoListEvents (@TghDataSetEventHolder.DoOnCalcFields, Sender);
  End;

  Procedure TChain.DoOnDeleteError (Sender :TDataSet;
    E :EDatabaseError; Var Action :TDataAction);
  Begin
    Inherited DoOnDeleteError (Sender, E, Action);
    DoListEvents (@TghDataSetEventHolder.DoOnDeleteError, Sender, E,
      Action);
  End;

  Procedure TChain.DoOnEditError (Sender :TDataSet; E :EDatabaseError;
    Var Action :TDataAction);
  Begin
    Inherited DoOnEditError (Sender, E, Action);
    DoListEvents (@TghDataSetEventHolder.DoOnEditError, Sender, E, Action);
  End;

  Procedure TChain.DoOnFilterRecord (Sender :TDataSet;
    Var Accept :Boolean);
  Var
    I :Integer;
  Begin
    Inherited DoOnFilterRecord (Sender, Accept);

    For I := 0 To List.Count - 1 Do
      If TghDataSource (List [I]).Enabled Then
        TghDataSource (List [I]).DataSetEvents.DoOnFilterRecord (
          Sender, Accept);
  End;

  Procedure TChain.DoOnNewRecord (Sender :TDataSet);
  Begin
    Inherited DoOnNewRecord (Sender);
    DoListEvents (@TghDataSetEventHolder.DoOnNewRecord, Sender);
  End;

  Procedure TChain.DoOnPostError (Sender :TDataSet; E :EDatabaseError;
    Var Action :TDataAction);
  Begin
    Inherited DoOnPostError (Sender, E, Action);
    DoListEvents (@TghDataSetEventHolder.DoOnPostError, Sender, E, Action);
  End;

  Function TChain.GetDataSet :TDataSet;
  Begin
    Result := FDataSet;
  End;

  Procedure TChain.InternalSetDataSet (Value :TDataSet);
  Begin
    FDataSet := Value;
  End;

  Procedure TChain.Notification (AComponent :TComponent;
    Operation :TOperation);
  Begin
    Inherited Notification (AComponent, Operation);

    If (Operation = opRemove) And (AComponent = DataSet) Then
      Free;
  End;

  { TghDataSource }

  Constructor TghDataSource.Create (AOwner :TComponent);
  Begin
    Inherited Create (AOwner);
    FDataSetEvents := TghDataSetEventHolder.Create (Self);
    ghSetSubcomponent (DataSetEvents, 'DataSetEvents');

    If GHDataSources = Nil Then
      GHDataSources := TghObjList.Create (False);

    GHDataSources.Add (Self);
  End;

  Destructor TghDataSource.Destroy;
  Begin
    DataSet := Nil;
    Inherited Destroy;

    { NOTE: The inherited destructor could call this destructor indirectly
      for another object. } 
    If (GHDataSources <> Nil) And (GHDataSources.Count = 0) Then
      FreeAndNil (GHDataSources);
  End;

  { Protected static methods }

  {$Warn No_RetVal Off}
  Function TghDataSource.CheckDataSet :TDataSet;
  Begin
    If DataSet = Nil Then
      DatabaseError ('DataSet property is not assigned.', Self)
    Else
      Result := DataSet;
  End;
  {$Warn No_RetVal On}

  Function TghDataSource.CloneDataSet (Const Value :TDataSet) :TDataSet;
  Var
    AClass :TDataSetClass;
  Begin
    AClass := TDataSetClass (Value.ClassType);
    Result := Nil;

    If Assigned (OnCloneDataSet) Then
      OnCloneDataSet (Self, Value, AClass, Result);

    If (AClass <> Nil) And (Result = Nil) Then
      Result := InternalCloneDataSet (Value, AClass);
  End;

  Procedure TghDataSource.SetDataSet (Const Value :TDataSet);
  Var
    I :Integer;
  Begin
    If Value = DataSet Then
      Exit;

    If Not (csDesigning In ComponentState) Then
    Begin
      If Chain <> Nil Then
      Begin
        TChain (Chain).List.Remove (Self);

        If TChain (Chain).List.Count = 0 Then
          Chain.Free;

        Chain := Nil;
      End;

      If DataSetCloned Then
        DataSet.Free;  // Destroy current clone
    End;

    If (Value <> Nil) And DataSetCloned And
    Not (csDesigning In ComponentState) Then
    Begin
      Inherited DataSet := CloneDataSet (Value);
      FDataSetCloned := (DataSet <> Nil) And (DataSet.Owner = Self);
    End
    Else
      Inherited DataSet := Value;

    If (DataSet = Nil) Or (csDesigning In ComponentState) Then
      Exit;

    If Not DataSetCloned And (ChainList <> Nil) Then
      For I := 0 To ChainList.Count - 1 Do
        If TChain (ChainList [I]).DataSet = DataSet Then
        Begin
          Chain := ChainList [I];
          Break;
        End;

    If Chain = Nil Then
      Chain := TChain.Create (DataSet);

    Chain.FreeNotification (Self);
    TChain (Chain).List.Add (Self);
  End;

  Procedure TghDataSource.SetDataSetCloned (Const Value :Boolean);
  Var
    ADataSet :TDataSet;
  Begin
    If Value = DataSetCloned Then
      Exit;

    If (csDesigning In ComponentState) Or (DataSet = Nil) Then
      FDataSetCloned := Value
    Else
      If Value Then
      Begin
        ADataSet := DataSet;

        // We avoid the clone has invalid event handlers (current TChain)
        DataSet := Nil;

        DataSet := CloneDataSet (ADataSet);
        FDataSetCloned := (DataSet <> Nil) And (DataSet.Owner = Self);
      End
      Else
      Begin
        DataSet := Nil;
        FDataSetCloned := False;
      End;
  End;

  { Protected virtual methods }

  Function TghDataSource.InternalCloneDataSet (Value :TDataSet;
    AClass :TDataSetClass) :TDataSet;
  Begin
    Result := ghCloneDataSet (Value, AClass, Self);
  End;

  { Protected overridden methods }

  Procedure TghDataSource.Notification (AComponent :TComponent;
    Operation :TOperation);
  Begin
    Inherited Notification (AComponent, Operation);

    If (Operation = opRemove) And (AComponent = Chain) Then
      Chain := Nil;
  End;

End.

