{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFFirebirdSQLConnection.pas - TghFirebirdSQLConnection class unit.     }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFFirebirdSQLConnection;  { Firebird SQL Connection }

Interface

  Uses
    GHFSQLConnection, Classes, GHFVCL, GHFRTL;

  Type
    { Firebird SQL Connection class }
    TghFirebirdSQLConnection = Class (TghSQLConnection)
      Protected
        { Overridden methods }
        Function NextValueExpr :String; Override;
        Function SelectAuxTable :String; Override;
      Public
        Constructor Create (AOwner :TComponent); Override;

        { Overridden methods }
        Function ConstraintField (Const Constraint :String) :String;
          Override;
        Function ConstraintRefTable (Const Constraint :String) :String;
          Override;
        Function ConstraintTable (Const Constraint :String) :String;
          Override;
        Function FieldConstraint (Const Table, Field :String) :String;
          Override;
        Function FieldOfIndex (Const Index :String) :String; Override;
        Function PrimaryKeyIndex (Const Table :String) :String; Override;
        Function TableError (Const Msg :String) :TghTableError;
          Override;
        Function TableOfIndex (Const Index :String) :String; Override;
      Published
        { Properties }
        Property NameCase Default ghcaUpper;
    End;

Implementation

  Uses
    SysUtils, StrUtils;

  { TghFirebirdSQLConnection }

  Constructor TghFirebirdSQLConnection.Create (AOwner :TComponent);
  Begin
    Inherited Create (AOwner);
    NameCase := ghcaUpper;
  End;

  { Protected overridden methods }

  Function TghFirebirdSQLConnection.NextValueExpr :String;
  Begin
    Result := 'Gen_ID (%s, 1)';
  End;

  Function TghFirebirdSQLConnection.SelectAuxTable :String;
  Begin
    Result := 'RDB$DATABASE';
  End;

  { Public overridden methods }

  Function TghFirebirdSQLConnection.ConstraintField (
    Const Constraint :String) :String;
  Begin
    { Get first field of constraint index }
    Result := SelectTrim ('ISe.RDB$Field_Name From RDB$Index_Segments ' +
      'ISe Inner Join RDB$Relation_Constraints RC On RC.RDB$Index_Name ' +
      '= ISe.RDB$Index_Name Where (RC.RDB$Constraint_Name = ' +
      QuotedCasedName (Constraint) + ') And (ISe.RDB$Field_Position = 0)');
  End;

  Function TghFirebirdSQLConnection.ConstraintRefTable (
    Const Constraint :String) :String;
  Begin
    { Get reference table of constraint }
    Result := SelectTrim (
      'Select RelC.RDB$Relation_Name From RDB$Relation_Constraints RelC ' +
      'Inner Join RDB$Ref_Constraints RefC On RefC.RDB$Const_Name_UQ = ' +
      'RelC.RDB$Constraint_Name Where RefC.RDB$Constraint_Name = ' +
      QuotedCasedName (Constraint));
  End;

  Function TghFirebirdSQLConnection.ConstraintTable (
    Const Constraint :String) :String;
  Begin
    { Get table of constraint }
    Result := SelectTrim (
      'Select RDB$Relation_Name From RDB$Relation_Constraints Where ' +
      'RDB$Constraint_Name = ' + QuotedCasedName (Constraint));
  End;

  Function TghFirebirdSQLConnection.FieldConstraint (
    Const Table, Field :String) :String;
  Begin
    Result := SelectTrim (
      'First 1 RC.RDB$Constraint_Name From RDB$Relation_Constraints RC ' +
      'Inner Join RDB$Index_Segments ISe On ISe.RDB$Index_Name = ' +
      'RC.RDB$Index_Name Where (RC.RDB$Relation_Name = ' +
      QuotedCasedName (Table) + ') And (ISe.RDB$Field_Name = ' +
      QuotedCasedName (Field) + ') And (ISe.RDB$Field_Position = 0)');
  End;

  Function TghFirebirdSQLConnection.FieldOfIndex (Const Index :String)
    :String;
  Begin
    Result := SelectTrim ('RDB$Field_Name From RDB$Index_Segments Where ' +
      '(RDB$Index_Name = ' + QuotedCasedName (Index) +
      ') And (RDB$Field_Position = 0)');
  End;

  Function TghFirebirdSQLConnection.PrimaryKeyIndex (
    Const Table :String) :String;
  Begin              
    Result := SelectTrim ('Select RDB$Index_Name From ' +
      'RDB$Relation_Constraints Where (RDB$Relation_Name = ' +
      QuotedCasedName (Table) + ') And (RDB$Constraint_Type = ' +
      QuotedStr ('PRIMARY KEY') + ')');
  End;
  
  Function TghFirebirdSQLConnection.TableError (Const Msg :String)
    :TghTableError;
  Var
    Index :String;
  Begin
    { NOTE: This method is based on the semantics of the error messages.
      Override when it is necessary. }

    With Result Do
      { Required field }
      If (Pos ('validation error for column', Msg) > 0) And
      (RightStr (Msg, 20) = 'value "*** null ***"') Then
      Begin
        Code := gherNull;

        { Only field name can be extracted from this message }
        Table := '';
        Field := ghLeft (ghRight (Msg, 'column '), ',');
      End
      Else
        { Duplicate key in unique index }
        If Pos ('attempt to store duplicate value (visible to active ' +
        'transactions) in unique index', Msg) > 0 Then
        Begin
          Code := gherDuplicate;
          Index := ghDeleteRight (ghRight (Msg, 'index "'));
          Table := TableOfIndex (Index);
          Field := FieldOfIndex (Index);
        End
        Else
          { Invalid reference or necessary record for referential
            integrity }
          If Pos ('violation of FOREIGN KEY constraint', Msg) > 0 Then
          Begin
            Code := gherRef;
            Table := ghLeft (ghRight (Msg, 'table "'), '"');
            Field := ConstraintField (ghLeft (
              ghRight (Msg, 'constraint "'), '"'));
          End
          Else
          Begin
            { Record deleted by another user or program }
            If (Pos ('Record not found or changed by another user',
            Msg) > 0) Or (Pos ('cannot update erased record',
            Msg) > 0) Then
              Code := gherNotFound
            Else
              { Record locked by another user or program }
              If Pos ('lock conflict on no wait transaction', Msg) > 0
              Then
                Code := gherLock
              Else  { Other errors }
                Code := gherUnknown;

            Table := '';
            Field := '';
          End;
  End;

  Function TghFirebirdSQLConnection.TableOfIndex (Const Index :String)
    :String;
  Begin
    Result := SelectTrim ('RDB$Relation_Name From RDB$Indices Where ' +
      'RDB$Index_Name = ' + QuotedCasedName (Index));
  End;

End.
