{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFForm.pas - TghForm class unit.                                       }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFForm;  { Form }

Interface

  Uses
    Forms, Controls, Classes, DB, GHFVCL, GHFRTL;

  Type
    { Form class }
    TghForm = Class (TForm)
      Protected
        { Fields }
        AutoModalResult :TModalResult;
        FInOnCreate :Boolean;
        FOpenParams :PVariant;
        FReader :TReader;

        { Static methods }
        Function GetOpenParam (Const Index :Integer) :Variant;
        Function GetOpenParamCount :Integer;
        Class Function InternalExecute (Const Params :PVariant;
          Const AAutoModalResult :TModalResult = mrNone) :TModalResult;
        Class Function InternalOpen (Const Params :PVariant) :TghForm;
        Class Function PrepareOpen (Const Params :PVariant = Nil) :TghForm;

        { Virtual methods }
        Procedure CloseDataSource (DataSource :TDataSource); Virtual;
        Function DoExecute :TModalResult; Virtual;
        Procedure DoOpen; Virtual;
        Procedure GetControlStates (Control :TControl;
          Var AVisible, AEnabled :Boolean); Virtual;
        Procedure OpenDataSource (DataSource :TDataSource); Virtual;
        Procedure UpdateControl (Control :TControl); Virtual;

        { Overridden methods }
        Procedure DoClose (Var Action :TCloseAction); Override;
        Procedure DoCreate; Override;
        Procedure ReadState (Reader :TReader); Override;

        { Message methods }
        Procedure msCloseModal (Var Msg :TghmsCloseModal);
          Message ghmsCloseModal;
      Public
        Constructor Create; Reintroduce; Overload; Virtual;

        { Static methods }
        Class Function Execute (Const Params :Variant;
          Const AAutoModalResult :TModalResult = mrNone) :TModalResult;
          Overload;
        Class Function Execute (Const Params :Array Of Const;
          Const AAutoModalResult :TModalResult = mrNone) :TModalResult;
          Overload;
        Class Function Execute (Const Param :Pointer;
          Const AAutoModalResult :TModalResult = mrNone) :TModalResult;
          Overload;
        Class Function Execute :TModalResult; Overload;
        Function FirstControl :TWinControl;
        Function FocusFirst :Boolean;
        Function HasFocus (Const Control :TWinControl = Nil) :Boolean;
        Class Function Instance :TghForm;
        Class Function Open (Const Params :Variant) :TghForm; Overload;
        Class Function Open (Const Params :Array Of Const) :TghForm;
          Overload;
        Class Function Open (Const Param :Pointer) :TghForm; Overload;
        Class Function Open :TghForm; Overload;
        Procedure RefocusControl;
        Function RefocusFirst :Boolean;
        Function ReleasePending :Boolean;
        Procedure UpdateControls (Const Parent :TWinControl;
          Const IncludeParent :Boolean = False); Overload;
        Procedure UpdateControls (Const IncludeForm :Boolean = False);
          Overload;
        Procedure UpdateControls (Const Parents :Array Of TWinControl;
          Const IncludeParents :Boolean = False); Overload;

        { Virtual methods }
        Procedure CloseDataSources; Virtual;
        Procedure OpenDataSources; Virtual;

        { Properties }
        Property InOnCreate :Boolean Read FInOnCreate;
        Property OpenParams [Const Index :Integer] :Variant
          Read GetOpenParam;
        Property OpenParamCount :Integer Read GetOpenParamCount;
        Property Reader :TReader Read FReader;
    End;

    { TghForm class type }
    TghFormClass = Class Of TghForm;

Implementation

  Uses
    Variants, Windows, StdCtrls, GHFObjListEx, GHFObjList;

  { TghForm }

  Constructor TghForm.Create;
  Begin
    Create (Application);
  End;

  { Protected static methods }

  Function TghForm.GetOpenParam (Const Index :Integer) :Variant;
  Begin
    If Index < 0 Then
      ghRaiseIndex (Index)
    Else
      If Index >= OpenParamCount Then
        Result := Unassigned  // Default value of "optional" parameter
      Else
        Result := ghValue (FOpenParams^, Index);
  End;

  Function TghForm.GetOpenParamCount :Integer;
  Begin
    If FOpenParams = Nil Then
      Result := 0
    Else
      Result := ghLength (FOpenParams^);
  End;

  Class Function TghForm.InternalExecute (Const Params :PVariant;
    Const AAutoModalResult :TModalResult = mrNone) :TModalResult;
  Begin
    With PrepareOpen (Params) Do
      Try
        If AAutoModalResult <> mrNone Then
        Begin
          AutoModalResult := AAutoModalResult;
          PostMessage (Handle, ghmsCloseModal, AutoModalResult, 0);
        End;

        Result := DoExecute;
      Finally
        FOpenParams := Nil;
        AutoModalResult := mrNone;
      End;
  End;

  Class Function TghForm.InternalOpen (Const Params :PVariant) :TghForm;
  Begin
    Result := PrepareOpen (Params);

    Try
      Result.DoOpen;
    Finally
      Result.FOpenParams := Nil;
    End;
  End;

  Class Function TghForm.PrepareOpen (Const Params :PVariant = Nil)
    :TghForm;
  Begin
    Result := Instance;

    If (Result = Nil) Or (fsModal In Result.FormState) Or
    Result.ReleasePending Then
    Begin
      Result := TghForm (NewInstance);
      Result.FOpenParams := Params;
      Result.Create;
    End
    Else
      Result.FOpenParams := Params;
  End;

  { Protected virtual methods }

  Procedure TghForm.CloseDataSource (DataSource :TDataSource);
  Begin
    If DataSource.DataSet <> Nil Then
      DataSource.DataSet.Close;
  End;

  Function TghForm.DoExecute :TModalResult;
  Begin
    Hide;
    Result := ShowModal;
  End;

  Procedure TghForm.DoOpen;
  Begin
    Show;
  End;

  Procedure TghForm.GetControlStates (Control :TControl;
    Var AVisible, AEnabled :Boolean);
  Begin
  End;

  Procedure TghForm.OpenDataSource (DataSource :TDataSource);
  Begin
    If DataSource.DataSet <> Nil Then
      DataSource.DataSet.Open;
  End;

  Procedure TghForm.UpdateControl (Control :TControl);
  Type
    TGetControlStatesMethod = Procedure (Control :TControl;
      Var AVisible, AEnabled :Boolean) Of Object;
  Var
    AEnabled, AVisible :Boolean;
    Method :TMethod;
  Begin
    TGetControlStatesMethod (Method) := GetControlStates;

    // We check if the GetControlStates method was overridden
    If Method.Code = @TghForm.GetControlStates Then
      Exit;

    AVisible := Control.Visible;
    AEnabled := Control.Enabled;
    GetControlStates (Control, AVisible, AEnabled);
    Control.Visible := AVisible;
    Control.Enabled := AEnabled;
  End;
  
  { Protected overridden methods }

  Procedure TghForm.DoClose (Var Action :TCloseAction);
  Begin
    If Action = caHide Then
      Action := caFree;

    Inherited DoClose (Action);
  End;

  Procedure TghForm.DoCreate;
  Begin
    FInOnCreate := True;

    Try
      Inherited DoCreate;
    Finally
      FInOnCreate := False;
    End;
  End;

  Procedure TghForm.ReadState (Reader :TReader);
  Begin
    FReader := Reader;

    Try
      Inherited ReadState (Reader);
    Finally
      FReader := Nil;
    End;
  End;
  
  { Protected message methods }

  Procedure TghForm.msCloseModal (Var Msg :TghmsCloseModal);
    // Call back routine }
    Procedure CheckButton (Const Button :TButton;
      Const ModalResult :Variant; Var Continue :Boolean);
    Begin
      If Button.Enabled And (Button.ModalResult = ModalResult) Then
      Begin
        Button.Click;
        Continue := False;
      End;
    End;
  Begin
    If ghEnumComponents (Self, TButton, True, @CheckButton,
    Msg.ModalResult) = Nil Then
      ModalResult := Msg.ModalResult;
  End;

  { Public static methods }

  Class Function TghForm.Execute (Const Params :Variant;
    Const AAutoModalResult :TModalResult = mrNone) :TModalResult;
  Begin
    Result := InternalExecute (@Params, AAutoModalResult);
  End;

  Class Function TghForm.Execute (Const Params :Array Of Const;
    Const AAutoModalResult :TModalResult = mrNone) :TModalResult;
  Begin
    Result := Execute (ghVar (Params), AAutoModalResult);
  End;

  Class Function TghForm.Execute (Const Param :Pointer;
    Const AAutoModalResult :TModalResult = mrNone) :TModalResult;
  Begin
    Result := Execute (ghVar (Param), AAutoModalResult);
  End;

  Class Function TghForm.Execute :TModalResult;
  Begin
    Result := InternalExecute (Nil);
  End;

  Function TghForm.FirstControl :TWinControl;
  Begin
    Result := FindNextControl (Nil, True, True, False);
  End;

  Function TghForm.FocusFirst :Boolean;
  Begin
    ActiveControl := FirstControl;
    Result := ActiveControl <> Nil;
  End;

  Function TghForm.HasFocus (Const Control :TWinControl = Nil) :Boolean;
  Begin
    Result := (Screen.ActiveForm = Self) And
      ((Control = Nil) Or Control.ContainsControl (ActiveControl));
  End;

  Class Function TghForm.Instance :TghForm;
  Begin
    Result := TghForm (ghComponent (Application, Self));
  End;

  Class Function TghForm.Open (Const Params :Variant) :TghForm;
  Begin
    Result := InternalOpen (@Params);
  End;

  Class Function TghForm.Open (Const Params :Array Of Const) :TghForm;
  Begin
    Result := Open (ghVar (Params));
  End;

  Class Function TghForm.Open (Const Param :Pointer) :TghForm;
  Begin
    Result := Open (ghVar (Param));
  End;
  
  Class Function TghForm.Open :TghForm;
  Begin
    Result := InternalOpen (Nil);
  End;

  Procedure TghForm.RefocusControl;
  Var
    Control :TWinControl;
  Begin
    If ActiveControl = Nil Then
      Exit;
      
    Control := ActiveControl;
    ActiveControl := Nil;

    If Control.CanFocus Then
      ActiveControl := Control;
  End;

  Function TghForm.RefocusFirst :Boolean;
  Begin
    ActiveControl := Nil;
    Result := FocusFirst;
  End;

  Function TghForm.ReleasePending :Boolean;
  Begin
    Result := ghMsgPending (Handle, cm_Release);
  End;

  Procedure TghForm.UpdateControls (Const Parent :TWinControl;
    Const IncludeParent :Boolean = False);

    // Call back routine
    Procedure Update (Const Control :TControl; Const ASelf :Variant;
      Var Continue :Boolean);
    Begin
      TghForm (ghPtr (ASelf)).UpdateControl (Control);
    End;
  Begin
    With TghObjListEx.Create (False) Do
      Try
        If IncludeParent Then
          Add (Parent);

        AddControls (Parent, True);
        Enum (@Update, ghVar (Self));
      Finally
        Free;
      End;
  End;

  Procedure TghForm.UpdateControls (
    Const IncludeForm :Boolean = False);
  Begin
    UpdateControls (Self, IncludeForm);
  End;

  Procedure TghForm.UpdateControls (Const Parents :Array Of TWinControl;
    Const IncludeParents :Boolean = False);
  Var
    I :Integer;
  Begin
    For I := 0 To High (Parents) Do
      UpdateControls (Parents [I], IncludeParents);
  End;


  { Public virtual methods }

  Procedure TghForm.CloseDataSources;
    // Call back routine
    Procedure CheckClose (Const DataSource :TDataSource;
      Const ASelf :Variant; Var Continue :Boolean);
    Begin
      If DataSource.State <> dsInactive Then
        TghForm (ghPtr (ASelf)).CloseDataSource (DataSource);
    End;
  Begin
    With TghObjList.Create (False) Do
      Try
        AddComponents (Self, TDataSource);
        Enum (@CheckClose, ghVar (Self));
      Finally
        Free;
      End;
  End;

  Procedure TghForm.OpenDataSources;
    // Call back routine
    Procedure CheckOpen (Const DataSource :TDataSource;
      Const ASelf :Variant; Var Continue :Boolean);
    Begin
      If DataSource.State = dsInactive Then
        TghForm (ghPtr (ASelf)).OpenDataSource (DataSource);
    End;
  Begin
    With TghObjList.Create (False) Do
      Try
        AddComponents (Self, TDataSource);
        Enum (@CheckOpen, ghVar (Self));
      Finally
        Free;
      End;
  End;

End.

