{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFRTL.pas - Unit of RTL related elements.                              }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFRTL;  { RTL }

Interface

  Uses
    Classes, SysUtils, Types, Windows, TypInfo, Messages, DateUtils;

  Const
    { Button Styles }
    ghbsTypeMask = $F;  // Covers from bs_PushButton to bs_OwnerDraw

    { ASCII Control Codes }
    ghccNUL =  #0;  // NUL
    ghccSOH =  #1;  // SOH
    ghccSTX =  #2;  // STX
    ghccETX =  #3;  // ETX
    ghccEOT =  #4;  // EOT
    ghccENQ =  #5;  // ENQ
    ghccACK =  #6;  // ACK
    ghccBEL =  #7;  // BEL
    ghccBS  =  #8;  // BS
    ghccHT  =  #9;  // HT
    ghccLF  = #10;  // LF
    ghccVT  = #11;  // VT
    ghccFF  = #12;  // FF
    ghccCR  = #13;  // CR
    ghccSO  = #14;  // SO
    ghccSI  = #15;  // SI
    ghccDLE = #16;  // DLE
    ghccDC1 = #17;  // DC1
    ghccDC2 = #18;  // DC2
    ghccDC3 = #19;  // DC3
    ghccDC4 = #20;  // DC4
    ghccNAK = #21;  // NAK
    ghccSYN = #22;  // SYN
    ghccETB = #23;  // ETB
    ghccCAN = #24;  // CAN
    ghccEM  = #25;  // EM
    ghccSUB = #26;  // SUB
    ghccESC = #27;  // ESC
    ghccFS  = #28;  // FS
    ghccGS  = #29;  // GS
    ghccRS  = #30;  // RS
    ghccUS  = #31;  // US
    ghccCRLF = ghccCR + ghccLF;  // CR, LF
    ghccTwoCRLF = ghccCRLF + ghccCRLF;  // CR, LF, CR, LF

    { Maximums }
    ghmxCardinal   = High (Cardinal);
    ghmxChar       = High (Char);
    ghmxDate       = 2958465;  // 31/12/9999
    ghmxTime       = 0.999999988425926;  // 23:59:59.999
    ghmxDateTime   = ghmxDate + ghmxTime;
    ghmxInt64      = High (Int64);
    ghmxPointer    = Pointer ($FFFFFFFF);
    ghmxReal48     = 1.7e38;
    ghmxShortInt   = High (ShortInt);
    ghmxSmallInt   = High (SmallInt);
    ghmxTimeMinute = 1 - OneMinute;  // 23:59
    ghmxYear       = 9999;
    ghmxWideChar   = High (WideChar);

    { Characters }
    ghchAll = [#0..ghmxChar];

    { Diacritics }
    ghdiAll = ['', '', '`', '^', '~', ''];

    { Digits }
    ghdgBin = ['0'..'1'];  // Binary
    ghdgDecimal = ['0'..'9'];  // Decimal
    ghdgHex = ghdgDecimal + ['A'..'F'];  // Hexadecimal
    ghdgOctal = ['0'..'7'];  // Octal
    ghdgRoman = ['I', 'V', 'X', 'L', 'C', 'D', 'M'];  // Roman

    { Error codes }

    { Application Base, Windows reserves the 29th bit for
      application-defined error codes. }
    gherAppBase = $20000000;

    { Generic errors.  Range from gherAppBase + $400 up to gherAppBase +
      $5FF (512 values reserved by GH Freebrary). }
    gherMin       = gherAppBase + $400;  // Minimum
    gherUnknown   = gherMin + 0;
    gherNull      = gherMin + 1;
    gherEmpty     = gherMin + 2;
    gherInvalid   = gherMin + 3;
    gherDuplicate = gherMin + 4;
    gherRef       = gherMin + 5;  // Reference
    gherNotFound  = gherMin + 6;
    gherLock      = gherMin + 7;
    gherMax       = gherMin + $1FF;  // Maximum

    { Formats }
    ghfmISODate = 'yyyy-mm-dd';
    ghfmISOTime = 'hh:nn:ss';
    // ISO Date and Time
    ghfmISODateTime = ghfmISODate + '"T"' + ghfmISOTime;
    // Standard Date and Time
    ghfmStdDateTime = ghfmISODate + ' ' + ghfmISOTime;

    { Icon Names }
    ghinPrefix = 'GHF_ICON';
    ghinInfo = ghinPrefix + 'INFO';
    ghinQuestion = ghinPrefix + 'QUESTION';
    ghinStop = ghinPrefix + 'STOP';
    ghinWarning = ghinPrefix + 'WARNING';

    { Messages }

    { Range from wm_User + $400 up to wm_User + $5FF (512 values reserved
      by GH Freebrary) }
    ghmsMin = wm_User + $400;  // Minimum
    ghmsCloseModal = ghmsMin;
    ghmsMax = ghmsMin + $1FF;  // Maximum

    { Minimums }
    ghmnInt      = Low (Integer);
    ghmnInt64    = Low (Int64);
    ghmnReal48   = 2.9e-39;
    ghmnShortInt = Low (ShortInt);
    ghmnSmallInt = Low (SmallInt);

    { Version Strings }
    ghvsComments         = 'Comments';
    ghvsCompanyName      = 'CompanyName';
    ghvsFileDescription  = 'FileDescription';
    ghvsFileVersion      = 'FileVersion';
    ghvsInternalName     = 'InternalName';
    ghvsLegalCopyright   = 'LegalCopyright';
    ghvsLegalTrademarks  = 'LegalTrademarks';
    ghvsOriginalFilename = 'OriginalFilename';
    ghvsPrivateBuild     = 'PrivateBuild';
    ghvsProductName      = 'ProductName';
    ghvsProductVersion   = 'ProductVersion';
    ghvsSpecialBuild     = 'SpecialBuild';

  Type
    { Pointer types }
    PClass = ^TClass;
    PObject = ^TObject;

    { Before Continue Event type }
    TghBeforeContinueEvent = Procedure (Sender :TObject;
      Var AContinue :Boolean) Of Object;

    { Boolean Method type }
    TghBoolMethod = Function :Boolean Of Object;

    { Character Case type }
    TghChrCase = (ghcaDefault, ghcaUndefined = ghcaDefault, ghcaLower,
      ghcaUpper);

    { Date Parts type }
    TghDateParts = Packed Record
      Day :Byte;
      Month :Byte;
      Year :Word;
    End;

    { Diacritic Type type }
    TghDiacriticType = (ghdtNone, ghdtAcute {  }, ghdtDiaeresis {  },
      ghdtGrave { ` }, ghdtCircumflex { ^ }, ghdtTilde { ~ },
      ghdtCedilla {  });

    { Enumeration Procedure type.  NOTE: Preferably use "Const" or "Var"
      for Variant arguments, so these will not be copied (TghEnumProc is a
      generic prototype). }
    TghEnumProc = Procedure (Item :Pointer; Const Params :Variant;
      Var AResult :Boolean);

    { Do Enumeration Method type }
    TghDoEnumMethod = Procedure (Proc :TghEnumProc; Const Params :Variant)
      Of Object;

    { Four Words type }
    PghFourWords = ^TghFourWords;
    TghFourWords = Packed Record
      Case Byte Of
        0 : (Value1, Value2, Value3 :Word;
             Case Byte Of
               0 : (Value4 :Word);
               1 : (HighValue :Word));
        1 : (Values :Array [0..3] Of Word);
        2 : (LowValue :Word);
        3 : (IntValue1, IntValue2 :Integer);
        4 : (Int64Value :Int64);
    End;

    { Gender type.  Neuter/Undefined, Masculine/Male, Feminine/Female. }
    TghGender = (ghgnNeuter, ghgnUndefined = ghgnNeuter, ghgnMasculine,
      ghgnMale = ghgnMasculine, ghgnFeminine, ghgnFemale = ghgnFeminine);

    { Method type }
    TghMethod = Procedure Of Object;

    { Method Information type }
    PghMethodInfo = ^TghMethodInfo;
    TghMethodInfo = Packed Record
      Size :Word;
      Address :Pointer;
      Name :ShortString;  // Name :String [Length (Name)];
      // Return information (when $MethodInfo is On)
      // Parameter information (when $MethodInfo is On)
      // Parameter information (when $MethodInfo is On)
      // ...
    End;

    { Method Information Array type }
    TghMethodInfoArr = Array Of PghMethodInfo;

    { Method Table type }
    PghMethodTable = ^TghMethodTable;
    TghMethodTable = Packed Record
      Count :Word;
      First :TghMethodInfo;
      // TghMethodInfo
      // ...
    End;

    {*******************************************************************}
    { Midpoint Rounding type.                                           }
    {                                                                   }
    { Unbiased (bankers):      -2.5 = -2  -1.5 = -2  1.5 = 2  2.5 = 2  }
    { Toward zero:              -2.5 = -2  -1.5 = -1  1.5 = 1  2.5 = 2  }
    { Toward infinity:          -2.5 = -3  -1.5 = -2  1.5 = 2  2.5 = 3  }
    { Toward positive infinity: -2.5 = -2  -1.5 = -1  1.5 = 2  2.5 = 3  }
    { Toward negative infinity: -2.5 = -3  -1.5 = -2  1.5 = 1  2.5 = 2  }
    {*******************************************************************}
    TghMidpointRounding = (ghmrUnbiased, ghmrZero, ghmrInfinity,
      ghmrPositive, ghmrNegative);

    { Parameter Method Type }
    TghParamMethod = Procedure (Param :Pointer) Of Object;

    { Parameter Pointer Method Type }
    TghParamPtrMethod = Function (Param :Pointer) :Pointer Of Object;

    { Pointer and Dummy Wide String type }
    TghPtrDummyWideStr = Packed Record
       Ptr :Pointer;
       Dummy :WideString;
    End;

    { Platform String type }
    TghPlatStr = Type TghPtrDummyWideStr;

    { PPropInfo Array type }
    TghPPropInfoArr = Array Of PPropInfo;

    { Pointer and Integer type }
    TghPtrInt = Packed Record
      Ptr :Pointer;
      Int :Integer;
    End;

    { Pointer Method type }
    TghPtrMethod = Function :Pointer Of Object;

    { String Compare type }
    TghStrCompare = (ghscOrdinal, ghscOrdinalAnyCase, ghscLocale,
      ghscLocaleAnyCase, ghscSimple, ghscSimpleAnyCase);

    { String Parameter Integer Function type }
    TghStrParamIntFunc = Function (Const Value :String) :Integer;

    { String Parameter String Function type }
    TghStrParamStrFunc = Function (Const Value :String) :String;

    { Ternary type }
    PghTernary = ^TghTernary;
    TghTernary = (ghtrDefault, ghtrUndefined = ghtrDefault, ghtrFalse,
      ghtrTrue);

    { Three Parameters Pointer Method Type }
    TghThreeParamsPtrMethod = Function (Param1, Param2, Param3 :Pointer)
      :Pointer Of Object;

    { Two Characters type }
    TghTwoChrs = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :Char);
        1 : (Values :Array [0..1] Of Char);
        2 : (LowValue, HighValue :Char);
    End;

    { Two Dates type }
    TghTwoDates = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :TDateTime);
        1 : (Values :Array [0..1] Of TDateTime);
        2 : (LowValue, HighValue :TDateTime);
    End;

    { Two Floats type }
    TghTwoFloats = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :Double);
        1 : (Values :Array [0..1] Of Double);
        2 : (LowValue, HighValue :Double);
    End;

    { Two Integers type }
    TghTwoInts = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :Integer);
        1 : (Values :Array [0..1] Of Integer);
        2 : (LowValue, HighValue :Integer);
        3 : (Int64Value :Int64);
    End;

    { Two Parameters Pointer Method Type }
    TghTwoParamsPtrMethod = Function (Param1, Param2 :Pointer) :Pointer Of
      Object;

    { Two Strings type }
    TghTwoStrs = Packed Record
      Value1 :String;
      Value2 :String;
    End;

    { Two Words type }
    TghTwoWords = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :Word);
        1 : (Values :Array [0..1] Of Word);
        2 : (LowValue, HighValue :Word);
        3 : (IntValue :Integer);
    End;

    { TghTwoWords Array type }
    TghTwoWordsArr = Array Of TghTwoWords;

    { User type }
    TghUser = Packed Record
      ID :Integer;
      Name :String;
      Password :String;
      Role :String;
      Level :Byte;
    End;

    { Variant Method type }
    TghVarMethod = Function :Variant Of Object;

    { Variant Parameter Method type }
    TghVarParamMethod = Procedure (Const Param :Variant) Of Object;

  Const
    { Diacritic Array }
    ghdiArr :Array [ghdtAcute..ghdtCedilla] Of Char =
      ('', '', '`', '^', '~', '');

    { Letters }
    ghltDiacritics :Array [TghDiacriticType, 0..7, ghcaLower..ghcaUpper] Of
      Char = {     a           e           i           o           u
                    y           n           c                             }
             ((('a', 'A'), ('e', 'E'), ('i', 'I'), ('o', 'O'), ('u', 'U'),
               ('y', 'Y'), ('n', 'N'), ('c', 'C')),  // ghdtNone

              (('', ''), ('', ''), ('', ''), ('', ''), ('', ''),
               ('', ''), (#0, #0), (#0, #0)),  // ghdtAcute

              (('', ''), ('', ''), ('', ''), ('', ''), ('', ''),
               ('', ''), (#0, #0), (#0, #0)),  // ghdtDiaeresis

              (('', ''), ('', ''), ('', ''), ('', ''), ('', ''),
               (#0, #0), (#0, #0), (#0, #0)),  // ghdtGrave

              (('', ''), ('', ''), ('', ''), ('', ''), ('', ''),
               (#0, #0), (#0, #0), (#0, #0)),  // ghdtCircumflex

              (('', ''), (#0, #0), (#0, #0), ('', ''), (#0, #0),
               (#0, #0), ('', ''), (#0, #0)),  // ghdtTilde

              ((#0, #0), (#0, #0), (#0, #0), (#0, #0), (#0, #0),
               (#0, #0), (#0, #0), ('', '')));  // ghdtCedilla

    { TghStrCompare sets }
    ghscAll = [Low (TghStrCompare)..High (TghStrCompare)];
    ghscAnyCase = [ghscOrdinalAnyCase, ghscLocaleAnyCase,
      ghscSimpleAnyCase];
    ghscCases = [ghscOrdinal, ghscLocale, ghscSimple];
    ghscOrdinals = [ghscOrdinal, ghscOrdinalAnyCase];
    ghscLocales = [ghscLocale, ghscLocaleAnyCase];
    ghscSimples = [ghscSimple, ghscSimpleAnyCase];

  Var
    { Dummy variables }
    GHDUCardinal :Cardinal;
    GHDUDateTime :TDateTime;
    GHDUExtended :Extended;
    GHDUInt :Integer;
    GHDUInt64 :Int64;
    GHDUPChar :PChar;
    GHDUPtr :Pointer;

    { Formats }
    GHFMSQLDate :String = ghfmISODate;
    GHFMSQLTime :String = ghfmISOTime;
    GHFMSQLDateTime :String = ghfmStdDateTime;

    { Midpoint Rounding }
    GHMidpointRounding :TghMidpointRounding;

    { Users }
    GHURCurrent :TghUser;
    GHURPrev :TghUser;

  { Functions and procedures }

  { Accent }
  Function ghAccent (Var Str :String; Const Index :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :Boolean;

  { Accent Safe }
  Function ghAccentSafe (Var Str :String; Const Index :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :Boolean;

  { Accented }
  Function ghAccented (Const Value :String; Const Index :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :String;

  { Accented End }
  Function ghAccentedEnd (Const Value :String; Const Pos :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :String;

  { Accent End }
  Function ghAccentEnd (Var Str :String; Const Pos :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :Boolean;

  { Allocate Pointer }
  Function ghAllocPtr (Const Value :Pointer = Nil) :Pointer;

  { Assign }
  Procedure ghAssign (Var AVar :Variant; Const Index :Integer;
    Const Value :Variant);

  { Base Class Name }
  Function ghBaseClassName (Const Window :THandle) :String;

  { Before Continue }
  Function ghBeforeContinue (Const Event; Const Sender :TObject;
    Const Method :TghMethod = Nil) :Boolean; Overload;

  Function ghBeforeContinue (Const Event; Const Sender :TObject;
    Const Method :TghBoolMethod) :Boolean; Overload;

  { Before Continue After }
  Procedure ghBeforeContinueAfter (Const BeforeEvent;
    Const Sender :TObject; Const Method :TghMethod; Const AfterEvent);

  { Bit Bytes }
  Function ghBitBytes (Const Count :Integer) :Integer;

  { Bit On? }
  Function ghBitOn (Const Value, Bit :Integer) :Boolean; Overload;

  { Bit On? }
  Function ghBitOn (Const Buffer :Pointer; Const Bit :Integer) :Boolean;
    Overload;

  { Boolean }
  Function ghBool (Const Value :String) :Boolean; Overload;

  { Boolean }
  Function ghBool (Const Value :Variant) :Boolean; Overload;

  { Boolean }
  Function ghBool (Const Value :TghTernary) :Boolean; Overload;

  { Button Type }
  Function ghButtonType (Const Window :THandle) :Integer;

  { Call }
  Function ghCall (Const Routine :TghEnumProc; Const Item :Pointer;
    Const Params :Variant; Const DefaultResult :Boolean = False) :Boolean;
    Overload;

  { Call }
  Function ghCall (Const Obj :TObject; Const MethodAddress :Pointer)
    :Pointer; Overload;

  { Call }
  Function ghCall (Const Obj :TObject; Const MethodAddress, Param :Pointer)
    :Pointer; Overload;

  { Call }
  Function ghCall (Const Obj :TObject;
    Const MethodAddress, Param1, Param2 :Pointer) :Pointer; Overload;

  { Call }
  Function ghCall (Const Obj :TObject; Const MethodAddress, Param1, Param2,
    Param3 :Pointer) :Pointer; Overload;

  { Call Nested Enumeration Procedure }
  Function ghCallNestedEnumProc (Const ProcParams :Variant;
    Const Item :Pointer; Const DefaultResult :Boolean = False) :Boolean;

  { Case }
  Function ghCase (Const Value :String) :TghChrCase;

  { Cased }
  Function ghCased (Const Value :String; Const Upper :Boolean) :String;
    Overload;

  { Cased }
  Function ghCased (Const Value :String; Const ACase :TghChrCase) :String;
    Overload;

  { Cased }
  Function ghCased (Const Value, Sample :String) :String; Overload;

  { Check Bit }
  Function ghCheckBit (Const Value, Mask, OnMask :Integer) :Integer;

  { Check Bits }
  Function ghCheckBits (Const Value, Mask, OnMask :Integer) :Integer;

  { Check Flag Button }
  Function ghCheckFlagButton (Const Value, Flag :Integer) :Integer;

  { Check Flag Button Icon }
  Function ghCheckFlagButtonIcon (
    Const Value, ButtonFlag, IconFlag :Integer) :Integer;

  { Check Flag Icon }
  Function ghCheckFlagIcon (Const Value, Flag :Integer) :Integer;

  { Check Index }
  Function ghCheckIndex (Const Value, Count :Integer) :Integer;

  { Check Last Error }
  Procedure ghCheckLastError;

  { Character  }
  Function ghChr (Const Value :Char) :WideChar; Overload;

  { Character  }
  Function ghChr (Const Value :WideChar) :Char; Overload;

  { Character  }
  Function ghChr (Const Value :String; Const Index :Integer = 1) :Char;
    Overload;

  { Character  }
  Function ghChr (Const Value :WideString; Const Index :Integer = 1)
    :WideChar; Overload;

  { Character  }
  Function ghChr (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1) :Char; Overload;

  { Character  }
  Function ghChr (Const Value, Chrs :String; Const StartIndex :Integer = 1)
    :Char; Overload;

  { Character, End }
  Function ghChrEnd (Const Value :String; Const Pos :Integer = 1) :Char;

  { Characters  }
  Function ghChrs (Const Value :String) :TSysCharSet; Overload;

  { Characters  }
  Function ghChrs (Const Values :Array Of String) :TSysCharSet; Overload;

  { Character of Strings }
  Function ghChrStrs (Const Value :String; Const Chrs :Array Of String;
    Const StartIndex :Integer = 1) :Char;

  { Class }
  Function ghClass (Const Name :String) :Pointer;

  { Class Button? }
  Function ghClassButton (Const Window :THandle) :Boolean;

  { Class Name }
  Function ghClassName (Const Window :THandle) :String;

  { Clone }
  Function ghClone (Const Obj :TObject; Const AClass :TClass;
    Const Owner :TComponent = Nil; Const ExcludedProps :String = '')
    :TObject; Overload;

  { Clone }
  Function ghClone (Const Obj :TObject; Const Owner :TComponent = Nil;
    Const ExcludedProps :String = '') :TObject; Overload;

  { Clone }
  Function ghClone (Const Obj :TObject; Const ExcludedProps :String)
    :TObject; Overload;

  { Comma }
  Function ghComma (Const Value1, Value2 :String;
    Const IncludeEmpties :Boolean = False) :String;

  { Comma Integers }
  Function ghCommaInts (Const Values :Array Of Integer) :String;

  { Commas }
  Function ghCommas (Const Values :Array Of String;
    Const IncludeEmpties :Boolean = False) :String; Overload;

  { Commas }
  Function ghCommas (Const List :TStrings;
    Const IncludeEmpties :Boolean = False) :String; Overload;

  { Comma SQL Values }
  Function ghCommaSQLValues (Const Values :Variant) :String;

  { Compare }
  Function ghCompare (Const Value1, Value2 :String;
    Const CompareType :TghStrCompare) :Integer;

  { Comparar Simple }
  Function ghCompareSimple (Value1, Value2 :String;
    Const AnyCase :Boolean = False) :Integer;

  { Component }
  Function ghComponent (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Recurse :Boolean = False)
    :TComponent;

  { Composite }
  Function ghComposite (Const Value :Char;
    Const Diacritic :TghDiacriticType) :Char;

  { Composition }
  Function ghComposition (Const Value :Char; Const Typing :Boolean = False)
    :String;

  { Concatenate }
  Function ghConcat (Const Value1, Value2 :String;
    Const Separator :String = ' ') :String; Overload;

  { Concatenate }
  Function ghConcat (Const Value1, Value2, Separator :String;
    Const IncludeEmpties :Boolean) :String; Overload;

  { Concatenate }
  Function ghConcat (Value1, Value2 :String; Const Separator :Char;
    Const IncludeEmpties :Boolean; Const CheckAffixes :Boolean) :String;
    Overload;

  { Concatenate }
  Function ghConcat (Value1, Value2 :String; Const Separator :String;
    Const IncludeEmpties :Boolean; Const CheckAffixes :Boolean) :String;
    Overload;

  { Concatenate }
  Function ghConcat (Const Values :Array Of String;
    Const Separator :String = ' '; Const IncludeEmpties :Boolean = False;
    Const CheckAffixes :Boolean = False) :String; Overload;

  { Concatenate }
  Function ghConcat (Const List :TStrings; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False;
    Const CheckAffixes :Boolean = False) :String; Overload;

  { Concatenate Integers }
  Function ghConcatInts (Const Values :Array Of Integer;
    Const Separator :String = ' ') :String; Overload;

  { Concatenate Integers }
  Function ghConcatInts (Const Values :Array Of Integer;
    Const MinLength :Integer; Const PadChr :Char = '0';
    Const Separator :String = ' ') :String; Overload;

  { Concatenate Integers }
  Function ghConcatInts (Const Values :Array Of Integer;
    Const Format :String; Const Separator :String) :String; Overload;

  { Concatenate Paths }
  Function ghConcatPaths (Const Path1, Path2 :String;
    Const APathDelim :Char = PathDelim) :String;

  { Copy }
  Function ghCopy (Const Value :String; Const StartIndex :Integer) :String;
    Overload;

  { Copy }
  Function ghCopy (Const Value :String;
    Const StartIndex, EndIndex :Integer) :String; Overload;

  { Copy Memory }
  Function ghCopyMem (Const Buffer :Pointer; Const Bytes :Integer;
    Size :Integer = MaxInt) :Pointer;

  { Copy Property }
  Function ghCopyProp (Const Source, Dest :TObject; Const Prop :PPropInfo)
    :Boolean;

  { Copy Properties }
  Procedure ghCopyProps (Const Source, Dest :TObject;
    Const Excludeds :String = '');

  { Count }
  Function ghCount (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1; EndIndex :Integer = MaxInt) :Integer;
    Overload;

  { Count }
  Function ghCount (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1; EndIndex :Integer = MaxInt) :Integer;
    Overload;

  { Count String }
  Function ghCountStr (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;

  { Count To }
  Function ghCountTo (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer) :Integer; Overload;

  { Count To }
  Function ghCountTo (Const Value :String; Const Chrs :TSysCharSet;
    Const EndIndex :Integer) :Integer; Overload;

  { Date }
  Function ghDate (Const Value :TghDateParts) :TDateTime;

  { Date }
  Function ghDateParts (Const Value :TDateTime) :TghDateParts; Overload;

  { Date }
  Function ghDateParts :TghDateParts; Overload;

  { Decimal }
  Function ghDecimal (Const Value :Extended; Const Precision :Integer = 18;
    Const Decimals :Integer = 9999) :TFloatRec;

  { Decimals }
  Function ghDecimals (Const Value :Extended) :Integer;

  { Default Icon Module }
  Function ghDefaultIconModule (Const Icon :PChar) :THandle;

  { Delete }
  Function ghDelete (Const Value :String; Const Index :Integer = 1;
    Const Chrs :Integer = 1) :String; Overload;

  { Delete }
  Function ghDelete (Const Value :String; Const Chr :Char) :String;
    Overload;

  { Delete }
  Function ghDelete (Const Value :String; Const Chrs :TSysCharSet) :String;
    Overload;

  { Delete Left }
  Function ghDeleteLeft (Const Value :String; Const Chr :Char) :String;
    Overload;

  { Delete Left }
  Function ghDeleteLeft (Const Value :String; Const Chrs :TSysCharSet)
    :String; Overload;

  { Delete Right }
  Function ghDeleteRight (Const Value :String; Const Chrs :Integer = 1)
    :String; Overload;

  { Delete Right }
  Function ghDeleteRight (Const Value :String; Const Chr :Char) :String;
    Overload;

  { Delete Right }
  Function ghDeleteRight (Const Value :String; Const Chrs :TSysCharSet)
    :String; Overload;

  { Dialog Code }
  Function ghDialogCode (Const Window :THandle) :Integer;

  { Dialog Code Button? }
  Function ghDialogCodeButton (Const Code :Integer) :Boolean;

  { Dialog Code Tab? }
  Function ghDialogCodeTab (Const Code :Integer) :Boolean;

  { Directory }
  Function ghDir (Const Path :String; Const UpLevels :Integer = 1;
    Const APathDelim :Char = PathDelim; Const ConsiderRoot :Boolean = True)
    :String; Overload;

  { Directory }
  Function ghDir (Const Path :String; Const UpLevels :Integer;
    Const ConsiderRoot :Boolean) :String; Overload;

  { Directory }
  Function ghDir (Const Path :String; Const APathDelim :Char;
    Const ConsiderRoot :Boolean = True) :String; Overload;

  { Directory }
  Function ghDir (Const Path :String; Const ConsiderRoot :Boolean) :String;
    Overload;

  { Directory Path }
  Function ghDirPath (Const Path :String; Const UpLevels :Integer;
    Const SubPath :String; Const APathDelim :Char = PathDelim) :String;
    Overload;

  { Directory Path }
  Function ghDirPath (Const Path, SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;

  { Directory Path }
  Function ghDirPath (Const UpLevels :Integer; Const SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;

  { Directory Path }
  Function ghDirPath (Const SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;

  { Div Ceil }
  Function ghDivCeil (Const Dividend, Divisor :Integer) :Integer;

  { Div and Mod }
  Function ghDivMod (Const Dividend, Divisor :Integer) :TghTwoInts;
    Overload;

  { Div and Mod }
  Function ghDivMod (Const Dividend, Divisor :Extended;
    Const Decimals :Integer = 0) :TghTwoFloats; Overload;

  { Div and Mod Safe }
  Function ghDivModSafe (Const Dividend, Divisor :Integer) :TghTwoInts;

  { Div Safe }
  Function ghDivSafe (Const Dividend, Divisor :Integer) :Integer;

  { Ends? }
  Function ghEnds (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;

  { Ends? }
  Function ghEnds (Const Value :String; Const Chrs :TSysCharSet;
    ChrCount :Integer; Const SubStr :String;
    Const SubStrCompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
    
  { Ends? }
  Function ghEnds (Const Value :String; Const Chrs :TSysCharSet;
    Const SubStr :String;
    Const SubStrCompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
    
  { Ends With? }
  Function ghEndsWith (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    
  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Recurse :Boolean;
    Const Routine :TghEnumProc; Const Params :Variant) :TComponent;
    Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Routine :TghEnumProc;
    Const Params :Variant) :TComponent; Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Recurse :Boolean; Const Routine :TghEnumProc;
    Const Params :Variant) :TComponent; Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Routine :TghEnumProc; Const Params :Variant) :TComponent;
    Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Recurse :Boolean;
    Const Routine :TghEnumProc) :TComponent; Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Routine :TghEnumProc) :TComponent;
    Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Recurse :Boolean; Const Routine :TghEnumProc) :TComponent;
    Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Routine :TghEnumProc) :TComponent; Overload;

  { Enumerate Properties }
  Function ghEnumProps (Const Props :TghPPropInfoArr;
    Const Kinds :TTypeKinds; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;

  { Enumerate Properties }
  Function ghEnumProps (Const Obj :TObject; Const Kinds :TTypeKinds;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;

  { Enumerate Properties }
  Function ghEnumProps (Const Obj :TObject; Const Kinds :TTypeKinds;
    Const Routine :TghEnumProc = Nil) :Integer; Overload;

  { Enumerate Properties Object }
  Function ghEnumPropsObj (Const Obj :TObject; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;

  { Enumerate Properties Object }
  Function ghEnumPropsObj (Const Obj :TObject; Const Routine :TghEnumProc)
    :Integer; Overload;

  { Enumerate Properties Object }
  Function ghEnumPropsObj (Const Props :TghPPropInfoArr;
    Const AClass :TClass; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;

  { Enumerate Properties Object }
  Function ghEnumPropsObj (Const Obj :TObject; Const AClass :TClass;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;

  { Equals? }
  Function ghEquals (Const Value1, Value2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;

  { Equals? }
  Function ghEquals (Const Value1 :String; Const StartIndex :Integer;
    Const Value2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
    
  { Equals? }
  Function ghEquals (Const Value1, Value2 :Variant) :Boolean; Overload;

  { Executable }
  Function ghExe :String;

  { Executable Directory }
  Function ghExeDir (Const UpLevels :Integer = 1;
    Const APathDelim :Char = PathDelim) :String;

  { Executable Name }
  Function ghExeName :String;

  { Execute }
  Function ghExecute (Const Path :String; Const Params :String = '')
    :Boolean;

  { Extension }
  Function ghExt (Const Value :String) :String;

  { Extension Executable }
  Function ghExtExe (Const Ext :String = '') :String;

  { Extension Executable Name }
  Function ghExtExeName (Const Ext :String = '') :String;

  { Extension Path }
  Function ghExtPath (Const Path :String; Const Ext :String = '') :String;

  { Flag Button }
  Function ghFlagButton (Const Value, Flag :Integer) :Integer;

  { Flag Icon }
  Function ghFlagIcon (Const Value, Flag :Integer) :Integer;

  { Find Icon }
  Function ghFindIcon (Const Module :THandle; Const NameID :Integer;
    Const Size :PCardinal = Nil) :THandle; Overload;

  { Find Icon }
  Function ghFindIcon (Const Module :THandle; Const Name :String;
    Const Size :PCardinal = Nil) :THandle; Overload;

  { Find Icon }
  Function ghFindIcon (Const NameID :Integer; Const Size :PCardinal = Nil)
    :THandle; Overload;

  { Find Icon }
  Function ghFindIcon (Const Name :String; Const Size :PCardinal = Nil)
    :THandle; Overload;

  { Find Resource }
  Function ghFindResource (Const Module :THandle; Const NameID :Integer;
    Const AType :PChar; Const Size :PCardinal = Nil) :THandle; Overload;

  { Find Resource }
  Function ghFindResource (Const Module :THandle; Const Name :String;
    Const AType :PChar; Const Size :PCardinal = Nil) :THandle; Overload;

  { Find Resource }
  Function ghFindResource (Const Name :String; Const AType :PChar;
    Const Size :PCardinal = Nil) :THandle; Overload;

  { Float }
  Function ghFloat (Const Value :String) :Extended; Overload;

  { Float }
  Function ghFloat (Const Value :Variant) :Extended; Overload;

  { Format Optional }
  Function ghFormatOptional (Const AFormat, Param :String) :String;

  { Fraction }
  Function ghFrac (Const Value :Extended; Const Decimals :Integer)
    :Extended; Overload;

  { Fraction }
  Function ghFrac (Const Value :Extended; Const Decimal :Boolean;
    Const Decimals :Integer = MaxInt) :Extended; Overload;

  { Fraction Decimal }
  Function ghFracDecimal (Const Value :Extended;
    Decimals :Integer = MaxInt) :Extended;

  { Fraction Integer }
  Function ghFracInt (Const Value :Extended; Const Decimals :Integer)
    :Integer;

  { Fraction Round }
  Function ghFracRound (Const Value :Extended; Const Epsilon :Extended = 0)
    :Integer;

  { Fraction Shift }
  Function ghFracShift (Const Value :Extended; Const Decimals :Integer)
    :Extended;

  { Fraction Zero }
  Function ghFracZero (Const Value :Extended;
    Const Decimal :Boolean = False; Const Epsilon :Extended = 0;
    Const Decimals :Integer = MaxInt) :Extended; Overload;

  { Fraction Zero }
  Function ghFracZero (Const Value :Extended; Const Decimal :Boolean;
    Const Decimals :Integer) :Extended; Overload;

  { Fraction Zero }
  Function ghFracZero (Const Value :Extended; Const Epsilon :Extended;
    Const Decimals :Integer = MaxInt) :Extended; Overload;

  { Fraction Zero }
  Function ghFracZero (Const Value :Extended; Const Decimals :Integer)
    :Extended; Overload;

  { Free and set Nil }
  Procedure ghFreeNil (Var Ptr);

  { Free Object Pointer }
  Procedure ghFreeObjPtr (Const Ptr);

  { Free Object Pointer and set Nil }
  Procedure ghFreeObjPtrNil (Var Ptr);

  { Free Pointer Pointer }
  Procedure ghFreePtrPtr (Const Ptr);

  { Free Pointer Pointer and set Nil }
  Procedure ghFreePtrPtrNil (Var Ptr);

  { From }
  Function ghFrom (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer = MaxInt) :String; Overload;

  { From }
  Function ghFrom (Const Value, SubStr :String;
    Const EndIndex :Integer = MaxInt;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { From }
  Function ghFrom (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;

  { Gender }
  Function ghGender (Const Letter :Char) :TghGender;

  { Gender and Number String }
  Function ghGenderNumberStr (Const Values :Array Of String;
    Const Gender :TghGender; Const Plural :Boolean = False) :String;

  { Get Dialog Code Button? }
  Function ghGetDialogCodeButton (Const Window :THandle) :Boolean;

  { Greater? }
  Function ghGreater (Const Value1, Value2 :Extended;
    Const Epsilon :Extended = 0) :Boolean;

  { GUID }
  Function ghGUID :TGUID;

  { GUID String }
  Function ghGUIDStr :String;

  { Has Bit? }
  Function ghHasBit (Const Value, Mask :Integer) :Boolean;

  { Has Bits? }
  Function ghHasBits (Const Value, Mask :Integer) :Boolean; Overload;

  { Has Bits? }
  Function ghHasBits (Const Value :Integer;
    Const Masks :Array Of Integer) :Boolean; Overload;

  { Has Button Style? }
  Function ghHasButtonStyle (Const Window :THandle) :Boolean;

  { Has Value? }
  Function ghHasValue (Const Value :Variant) :Boolean;

  { High }
  Function ghHigh (Const Value :Variant; Const Dim :Integer = 1) :Integer;

  { Icon Flag }
  Function ghIconFlag (Const Mark :Char) :Integer;

  { Icon Standard? }
  Function ghIconStd (Const Icon :PChar) :Boolean;

  { In Bits? }
  Function ghInBits (Const Value, Mask :Integer) :Boolean;

  { Increment }
  Function ghInc (Var Int :Integer; Const Inc :Integer = 1) :Integer;
    Overload;

  { Increment }
  Function ghInc (Const Value :String; Const Inc :Integer = 1) :String;
    Overload;

  { Increment }
  Function ghInc (Const Value :Variant; Const Inc :Integer = 1) :Variant;
    Overload;

  { Increment Circular }
  Function ghIncCircular (Const Value, MinValue, MaxValue :Integer;
    Const Inc :Integer = 1) :Integer;

  { Increment Positive }
  Function ghIncPositive (Const Value :Integer; Const Inc :Integer = 1)
    :Integer;

  { Increment Units }
  Function ghIncUnits (Const Groups, Units :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9;
    Const AInc :Integer = 1) :TghTwoInts; Overload;

  { Increment Units }
  Function ghIncUnits (Const Value :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9;
    Const AInc :Integer = 1) :TghTwoInts; Overload;

  { Index }
  Function ghIndex (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer; Overload;

  { Index }
  Function ghIndex (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1) :Integer; Overload;

  { Index }
  Function ghIndex (Const Value, Chrs :String;
    Const StartIndex :Integer = 1) :Integer; Overload;

  { Index }
  Function ghIndex (Const List :TStrings; Const Str :String;
    Const StartIndex :Integer = 0;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Index }
  Function ghIndex (Const List :TStrings; Const Str :String;
    Const CompareType :TghStrCompare) :Integer; Overload;

  { Index A Last }
  Function ghIndexALast (Const Value :String; Const Chr :Char;
    Occurrence :Integer; Const StartIndex :Integer = 1) :Integer;

  { Index Last }
  Function ghIndexLast (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer;

  { Index Left }
  Function ghIndexLeft (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer;

  { Index Left Last }
  Function ghIndexLeftLast (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer;

  { Index Maximum }
  Function ghIndexMax (Const Value, Count :Integer) :Integer;

  { Index Minimum }
  Function ghIndexMin (Const Value :Integer) :Integer;

  { Inherits From? }
  Function ghInheritsFrom (AClass :TClass; Const Name :String) :Boolean;

  { In Range? }
  Function ghInRange (Const Value, MinValue, MaxValue :Cardinal) :Boolean;
    Overload;

  { In Range? }
  Function ghInRange (Const Value, MinValue, MaxValue :Pointer) :Boolean;
    Overload;

  { Integer }
  Function ghInt (Const Value :String) :Integer; Overload;

  { Integer }
  Function ghInt (Const Value :String; Const StartIndex :Integer) :Integer;
    Overload;

  { Integer }
  Function ghInt (Const Value :String; Const StartIndex, Chrs :Integer)
    :Integer; Overload;
    
  { Integer }
  Function ghInt (Const Value :Variant) :Integer; Overload;

  { Integer Even? }
  Function ghIntEven (Const Value :Extended) :Boolean;

  { Integer Strings }
  Function ghIntStrs (Const Values :Array Of Integer) :TStringDynArray;
    Overload;

  { Integer Strings }
  Function ghIntStrs (Const Values :Array Of Integer;
    Const MinLength :Integer; Const PadChr :Char = '0') :TStringDynArray;
    Overload;

  { Integer Strings }
  Function ghIntStrs (Const Values :Array Of Integer;
    Const AFormat :String) :TStringDynArray; Overload;

  { Is Boolean? }
  Function ghIsBool (Const Value :Variant; Const Bool :Boolean) :Boolean;

  { Is Characters? }
  Function ghIsChrs (Const Value :String; Const Chrs :TSysCharSet)
    :Boolean;

  { Is Date? }
  Function ghIsDate (Const Value :String) :Boolean;

  { Is Diacritic? }
  Function ghIsDiacritic (Const Value :Char) :Boolean;

  { Is Float? }
  Function ghIsFloat (Const Value :String) :Boolean; Overload;

  { Is Float? }
  Function ghIsFloat (Const Value :Variant) :Boolean; Overload;

  { Is Hexadecimal? }
  Function ghIsHex (Const Value :Char) :Boolean; Overload;

  { Is Hexadecimal? }
  Function ghIsHex (Const Value :String) :Boolean; Overload;

  { Is Icon Flag? }
  Function ghIsIconFlag (Const Flag :Integer) :Boolean;

  { Is Index? }
  Function ghIsIndex (Const Value, Count :Integer) :Boolean; Overload;

  { Is Index? }
  Function ghIsIndex (Const Value :Integer; Const Str :String) :Boolean;
    Overload;

  { Is Index? }
  Function ghIsIndex (Const Value :Integer; Const Str :WideString)
    :Boolean; Overload;

  { Is Integer? }
  Function ghIsInt (Const Value :Extended; Const Epsilon :Extended = 0)
    :Boolean; Overload;

  { Is Integer? }
  Function ghIsInt (Const Value :String) :Boolean; Overload;

  { Is Lower? }
  Function ghIsLower (Const Value :String) :Boolean;

  { ISO Date }
  Function ghISODate (Const Value :TDateTime) :String; Overload;

  { ISO Date }
  Function ghISODate :String; Overload;

  { ISO Date and Time }
  Function ghISODateTime (Const Value :TDateTime) :String; Overload;

  { ISO Date and Time }
  Function ghISODateTime :String; Overload;

  { Is Or Nil? }
  Function ghIsOrNil (Const Obj :TObject; Const AClass :TClass) :Boolean;

  { ISO Time }
  Function ghISOTime (Const Value :TDateTime) :String; Overload;

  { ISO Time }
  Function ghISOTime :String; Overload;

  { Is Pointer? }
  Function ghIsPtr (Const Value :String) :Boolean;

  { Is Upper? }
  Function ghIsUpper (Const Value :String) :Boolean;

  { Is Valid? }
  Function ghIsValid (Const Value :Single) :Boolean; Overload;

  { Is Valid? }
  Function ghIsValid (Const Value :Double) :Boolean; Overload;

  { Is Valid? }
  Function ghIsValid (Const Value :Extended) :Boolean; Overload;

  { Is Zero Or? }
  Function ghIsZeroOr (Const Value, Other :Extended;
    Const Epsilon :Extended = 0) :Boolean;

  { Key Character }
  Function ghKeyChr (Const Key :Integer) :Char;

  { Last Error Message }
  Function ghLastErrorMsg :String;

  { LastMinute }
  Function ghLastMinute (Const Value :TDateTime) :TDateTime; Overload;

  { LastMinute }
  Function ghLastMinute :TDateTime; Overload;

  { Left }
  Function ghLeft (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;

  { Left }
  Function ghLeft (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Left }
  Function ghLeft (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;

  { Left Last }
  Function ghLeftLast (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;

  { Left Last }
  Function ghLeftLast (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Left Last }
  Function ghLeftLast (Const Value, SubStr1, SubStr2 :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Left Of }
  Function ghLeftOf (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;

  { Left Of }
  Function ghLeftOf (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String; Overload;

  { Left Of }
  Function ghLeftOf (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;

  { Left Of A Last }
  Function ghLeftOfALast (Const Value :String; Const Chr :Char;
    Const Occurrence :Integer; Const StartIndex :Integer = 1) :String;

  { Left Of Last }
  Function ghLeftOfLast (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String;

  { Left and Right }
  Function ghLeftRight (Const Value :String; Const Chr :Char) :TghTwoStrs;
    Overload;

  { Left and Right }
  Function ghLeftRight (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :TghTwoStrs;
    Overload;

  { Length }
  Function ghLength (Const Value :Variant; Const Dim :Integer = 1)
    :Integer;

  { Length of All }
  Function ghLengthAll (Const Value :Variant) :Integer;

  { Length of Dimension }
  Function ghLengthDim (Const Value :Variant; Const Dim :Integer) :Integer;

  { Less? }
  Function ghLess (Const Value1, Value2 :Extended;
    Const Epsilon :Extended = 0) :Boolean;

  { Load Resource }
  Function ghLoadResource (Const Module :THandle; Const NameID :Integer;
    Const AType :PChar; Const Size :PCardinal = Nil) :THandle; Overload;

  { Load Resource }
  Function ghLoadResource (Const Module :THandle; Const Name :String;
    Const AType :PChar; Const Size :PCardinal = Nil) :THandle; Overload;

  { Lock Resource }
  Function ghLockResource (Const Module :THandle; Const NameID :Integer;
    Const AType :PChar; Const Size :PCardinal = Nil) :Pointer; Overload;

  { Lock Resource }
  Function ghLockResource (Const Module :THandle; Const Name :String;
    Const AType :PChar; Const Size :PCardinal = Nil) :Pointer; Overload;

  { Low }
  Function ghLow (Const Value :Variant; Const Dim :Integer = 1) :Integer;

  { Lower Case }
  Function ghLower (Const Value :Char) :Char; Overload;

  { Lower Case }
  Function ghLower (Const Value :String; Const Index :Integer = 1) :Char;
    Overload;

  { Lowered }
  Function ghLowered (Const Value :String; Const Index :Integer = 1)
    :String;

  { Maximum }
  Function ghMax (Const Value1, Value2 :TghTwoInts) :TghTwoInts;

  { Maximum Split Integer }
  Function ghMaxSplitInt (Const Value :String;
    Const Separator :String = ' '; Const IncludeEmpties :Boolean = False)
    :Integer;

  { Method Informations }
  Function ghMethodInfos (AClass :TClass) :TghMethodInfoArr;

  { Method Record }
  Function ghMethodRec (Const Obj, Address :Pointer) :TMethod;

  { Method Table }
  Function ghMethodTable (Const AClass :TClass) :PghMethodTable;

  { Minimum }
  Function ghMin (Const Value1, Value2 :TghTwoInts) :TghTwoInts;

  { Minimun Cardinal }
  Function ghMinCardinal (Const Value1, Value2 :Cardinal) :Cardinal;

  { Minimum and Maximum }
  Function ghMinMax (Const Value1, Value2 :Integer) :TghTwoInts;

  { Minimum Zero }
  Function ghMinZero (Const Value :Integer) :Integer;

  { Module Path }
  Function ghModulePath (Const Module :THandle) :String; Overload;

  { Module Path }
  Function ghModulePath :String; Overload;

  { More than Null? }
  Function ghMoreNull (Const Value :Variant) :Boolean;

  { Message Icon Flag }
  Function ghMsgIconFlag (Const Value :TSysCharSet) :Integer; Overload;

  { Message Icon Flag }
  Function ghMsgIconFlag (Const Value :String) :Integer; Overload;

  { Message Icon Flag of Strings }
  Function ghMsgIconFlagStrs (Const Values :Array Of String) :Integer;
    Overload;

  { Message Pending? }
  Function ghMsgPending (Const Window :THandle;
    Const MinID, MaxID :Integer) :Boolean; Overload;

  { Message Pending? }
  Function ghMsgPending (Const Window :THandle = 0; Const ID :Integer = 0)
    :Boolean; Overload;

  { Not Empty }
  Function ghNotEmpty (Const Value :Variant) :Variant;

  { Not Prefixed }
  Function ghNotPrefixed (Const Value :String; Const Prefix :Char) :String;
    Overload;

  { Not Prefixed }
  Function ghNotPrefixed (Const Value, Prefix :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Not Suffixed }
  Function ghNotSuffixed (Const Value :String; Const Suffix :Char) :String;
    Overload;

  { Not Prefixed }
  Function ghNotSuffixed (Const Value, Suffix :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Offset }
  Function ghOffset (Const Ptr :Pointer; Const Increment :Integer)
    :Pointer;

  { OLE Variant }
  Function ghOLEVar (Const Value :String) :OLEVariant; Overload;

  { OLE Variant }
  Function ghOLEVar (Const Value :TVarRec) :OLEVariant; Overload;

  { One }
  Function ghOne (Const Value :Boolean) :Integer;

  { Owns? }
  Function ghOwns (Const Owner :TComponent; Owned :TComponent) :Boolean;

  { Params }
  Function ghParams (Const ATrim :Boolean = False) :String;

  { Path Name }
  Function ghPathName (Const Path :String) :String;

  { Peek Message }
  Function ghPeekMsg (Out Msg :TMsg; Const Window :THandle;
    Const MinID, MaxID :Integer; Const Remove :Boolean = False) :Boolean;
    Overload;

  { Peek Message }
  Function ghPeekMsg (Out Msg :TMsg; Const Window :THandle = 0;
    Const ID :Integer = 0; Const Remove :Boolean = False) :Boolean;
    Overload;

  { Percent }
  Function ghPercent (Const Value, Percentage :Extended) :Extended;
    Overload;

  { Percent }
  Function ghPercent (Const Value, Percentage :Extended;
    Const Decimals :Integer) :Extended; Overload;

  { Percent 100 }
  Function ghPercent100 (Const Value, Percentage :Extended) :Extended;
    Overload;

  { Percent 100 }
  Function ghPercent100 (Const Value, Percentage :Extended;
    Const Decimals :Integer) :Extended; Overload;

  { Percentage }
  Function ghPercentage (Const Value, Portion :Extended) :Extended;
    Overload;

  { Percentage }
  Function ghPercentage (Const Value, Portion :Extended;
    Const Decimals :Integer) :Extended; Overload;

  { Platform String }
  Function ghPlatStr (Const Value :String) :TghPlatStr;

  { Position }
  Function ghPos (Value, SubStr :String; Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position }
  Function ghPos (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :Integer; Overload;

  { Position of Integer }
  Function ghPosInt (Const Values :Array Of Integer; Const Int :Integer;
    Const StartPos :Integer = 0) :Integer;

  { Position Last }
  Function ghPosLast (Value, SubStr :String; Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position Last }
  Function ghPosLast (Value, SubStr1, SubStr2 :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position Left }
  Function ghPosLeft (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;

  { Position Left Last }
  Function ghPosLeftLast (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position Left Last }
  Function ghPosLeftLast (Const Value, SubStr1, SubStr2 :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position of Pointer }
  Function ghPosPtr (Const Values :Array Of Pointer; Const Ptr :Pointer;
    Const StartPos :Integer = 0) :Integer;

  { Position Right }
  Function ghPosRight (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position Right }
  Function ghPosRight (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :Integer; Overload;

  { Position of String }
  Function ghPosStr (Const Values :Array Of String; Const Str :String;
    Const StartPos :Integer = 0;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position of String }
  Function ghPosStr (Const Values :Array Of String; Const Str :String;
    Const CompareType :TghStrCompare) :Integer; Overload;

  { Prefixed }
  Function ghPrefixed (Const Value :String; Const Prefix :Char;
    Const AllowEmpty :Boolean = False) :String; Overload;

  { Prefixed }
  Function ghPrefixed (Const Value, Prefix :String;
    Const AllowEmpty :Boolean = False;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Prepare Position }
  Function ghPreparePos (Var Str1, Str2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;

  { Property Boolean }
  Function ghPropBool (Const Obj :TObject; Const Name :String) :Boolean;

  { Property Object }
  Function ghPropObj (Const Obj :TObject; Const Name :String;
    Const AClass :TClass = Nil) :TObject; Overload;

  { Property Object }
  Function ghPropObj (Const Obj :TObject; Const AClass :TClass = Nil)
    :TObject; Overload;

  { Properties }
  Function ghProps (Const AClass :TClass) :TghPPropInfoArr; Overload;

  { Properties }
  Function ghProps (Const Obj :TObject) :TghPPropInfoArr; Overload;

  { Property String }
  Function ghPropStr (Const Obj :TObject; Const Name :String) :String;
  
  { Property Value }
  Function ghPropValue (Const Obj :TObject; Const Name :String) :Variant;

  { Pointer }
  Function ghPtr (Const Value :String) :Pointer; Overload;

  { Pointer }
  Function ghPtr (Const Value :Variant) :Pointer; Overload;

  { Quoted ISO Date }
  Function ghQuotedISODate (Const Value :TDateTime) :String; Overload;

  { Quoted ISO Date }
  Function ghQuotedISODate :String; Overload;

  { Quoted ISO Date and Time }
  Function ghQuotedISODateTime (Const Value :TDateTime) :String; Overload;

  { Quoted ISO Date and Time }
  Function ghQuotedISODateTime :String; Overload;

  { Quoted ISO Time }
  Function ghQuotedISOTime (Const Value :TDateTime) :String; Overload;

  { Quoted ISO Time }
  Function ghQuotedISOTime :String; Overload;

  { Quoted SQL Date }
  Function ghQuotedSQLDate (Const Value :TDateTime) :String; Overload;

  { Quoted SQL Date }
  Function ghQuotedSQLDate :String; Overload;

  { Quoted SQL Date and Time }
  Function ghQuotedSQLDateTime (Const Value :TDateTime) :String; Overload;

  { Quoted SQL Date and Time }
  Function ghQuotedSQLDateTime :String; Overload;

  { Quoted SQL Time }
  Function ghQuotedSQLTime (Const Value :TDateTime) :String; Overload;

  { Quoted SQL Time }
  Function ghQuotedSQLTime :String; Overload;

  { Raise }
  Procedure ghRaise (Const Error :String; Const Callers :Integer = 1);
    Overload;

  { Raise }
  Procedure ghRaise (Const Error :String; Const Params :Array Of Const;
    Const Callers :Integer = 1); Overload;

  { Raise Index }
  Procedure ghRaiseIndex (Const Value :Integer;
    Const Callers :Integer = 1);

  { Raise Last Error }
  Procedure ghRaiseLastError (
    Const AFormat :String = '%s (GetLastError %d).';
    Const Callers :Integer = 1);

  { Refer If }
  Function ghReferIf (Var Str :String; Const Value :String) :PString;

  { Replace }
  Function ghReplace (Const Value :String; Const Chr, NewChr :Char)
    :String; Overload;

  { Replace }
  Function ghReplace (Const Value :String; Const Chrs :TSysCharSet;
    Const NewChr :Char) :String; Overload;

  { Reread Short String }
  Procedure ghRereadShortStr (Const Reader :TReader;
    Var ARightStr :String);

  { Return Address }
  Function ghReturnAddress (Callers :Integer = 1) :Pointer;

  { Reverse }
  Function ghReverse (Const List :TList) :TList;

  { Right }
  Function ghRight (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer = MaxInt) :String; Overload;

  { Right }
  Function ghRight (Const Value, SubStr :String;
    Const EndIndex :Integer = MaxInt;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String; Overload;

  { Right }
  Function ghRight (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;

  { Right Characters }
  Function ghRightChrs (Const Value :String; Const Chrs :TSysCharSet;
    Const MaxLength :Integer = MaxInt) :String;

  { Right Digits }
  Function ghRightDigits (Const Value :String;
    Const MaxLength :Integer = MaxInt) :String;

  { Right Integer }
  Function ghRightInt (Const Value :String;
    Const MaxLength :Integer = MaxInt) :Integer;

  { Right and Left }
  Function ghRightLeft (Const Value :String; Const Chr1, Chr2 :Char)
    :String; Overload;

  { Right and Left }
  Function ghRightLeft (Const Value, SubStr1, SubStr2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Right and Left Of }
  Function ghRightLeftOf (Const Value :String; Const Chr1, Chr2 :Char)
    :String; Overload;

  { Right and Left Of }
  Function ghRightLeftOf (Const Value, SubStr1, SubStr2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Right Of }
  Function ghRightOf (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer = MaxInt) :String; Overload;

  { Right Of }
  Function ghRightOf (Const Value, SubStr :String;
    Const EndIndex :Integer = MaxInt;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Right Of }
  Function ghRightOf (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;

  { Round }
  Function ghRound (Const Value :Extended; Const Decimals :Integer = 0;
    Const Epsilon :Extended = 0) :Extended;

  { Round Integer }
  Function ghRoundInt (Const Value :Extended; Const Epsilon :Extended = 0)
    :Int64;

  { Round to Multiple }
  Function ghRoundMultiple (Const Value, Divisor :Extended;
    Const Epsilon :Extended = 0) :Extended;

  { Search Path }
  Function ghSearchPath (Const Path :String;
    Const RaiseError :Boolean = False) :String;

  { Send Message }
  Function ghSendMsg (Const Window :THandle; Const ID :Integer) :Integer;

  { Set Bit }
  Function ghSetBit (Const Buffer :Pointer; Const Bit :Integer;
    Const Value :Boolean = True) :Boolean;

  { Set Character }
  Function ghSetChr (Var Str :String; Const Index :Integer;
    Const Value :Char) :Boolean;

  { Set Character Safe }
  Function ghSetChrSafe (Var Str :String; Const Index :Integer;
    Const Value :Char) :Boolean;

  { Set Icon }
  Procedure ghSetIcon (Var Params :TMsgBoxParams; Const Icon :PChar;
    Const Module :THandle = 0);

  { Set Integer }
  Function ghSetInt (Const Ref :Pointer; Const Value :Integer = 0)
    :Boolean;

  { Set Lower }
  Function ghSetLower (Var Str :String; Const Index :Integer = 1)
    :Boolean;

  { Set Lower Safe }
  Function ghSetLowerSafe (Var Str :String; Const Index :Integer = 1)
    :Boolean;

  { Set Pointer }
  Function ghSetPtr (Const Ref :Pointer; Const Value :Pointer = Nil)
    :Boolean;

  { Set Subcomponent }
  Procedure ghSetSubcomponent (Const Component :TComponent;
    Const Name :String);

  { Set Upper }
  Function ghSetUpper (Var Str :String; Const Index :Integer = 1)
    :Boolean;

  { Set Upper Safe }
  Function ghSetUpperSafe (Var Str :String; Const Index :Integer = 1)
    :Boolean;

  { Simple }
  Function ghSimple (Const Value :Char) :Char;

  { Simple String }
  Function ghSimpleStr (Const Value :String;
    Const DiacriticIndex :Integer) :String; Overload;

  { Simple String }
  Function ghSimpleStr (Const Value :String) :String; Overload;

  { Simple String Upper case }
  Function ghSimpleStrUpper (Const Value :String) :String;

  { Simple Upper case }
  Function ghSimpleUpper (Const Value :Char) :Char;

  { Simplify }
  Function ghSimplify (Var Str :String; Const Index :Integer) :Boolean;

  { Simplify Safe }
  Function ghSimplifySafe (Var Str :String; Const Index :Integer) :Boolean;

  { Split }
  Function ghSplit (Const Value :String; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False) :TStringDynArray;

  { Split Integers }
  Function ghSplitInts (Const Value :String; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False) :TIntegerDynArray;

  { Split Last }
  Function ghSplitLast (Const Value :String; Const Separator :Char = ' ';
    Const IncludeEmpties :Boolean = False) :String;
    
  { SQL Date }
  Function ghSQLDate (Const Value :TDateTime) :String; Overload;

  { SQL Date }
  Function ghSQLDate :String; Overload;

  { SQL Date and Time }
  Function ghSQLDateTime (Const Value :TDateTime) :String; Overload;

  { SQL Date and Time }
  Function ghSQLDateTime :String; Overload;

  { SQL Time }
  Function ghSQLTime (Const Value :TDateTime) :String; Overload;

  { SQL Time }
  Function ghSQLTime :String; Overload;

  { SQL Value }
  Function ghSQLValue (Const Value :Variant) :String;

  { SQL Values }
  Function ghSQLValues (Const Values :Variant) :TStringDynArray;

  { Starts? }
  Function ghStarts (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;

  { Starts With? }
  Function ghStartsWith (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;

  { Standard Date and Time }
  Function ghStdDateTime (Const Value :TDateTime) :String; Overload;

  { Standard Date and Time }
  Function ghStdDateTime :String; Overload;

  { String }
  Function ghStr (Const Value, MinLength :Integer;
    Const PadChr :Char = '0') :String; Overload;

  { String }
  Function ghStr (Const Value :Pointer) :String; Overload;

  { Strings }
  Function ghStrs (Const Values :Array Of String) :TStringDynArray;
    Overload;

  { Strings }
  Function ghStrs (Const List :TStrings) :TStringDynArray; Overload;

  { Style }
  Function ghStyle (Const Window :THandle) :Integer;

  { Suffixed }
  Function ghSuffixed (Const Value :String; Const Suffix :Char;
    Const AllowEmpty :Boolean = False) :String; Overload;

  { Suffixed }
  Function ghSuffixed (Const Value, Suffix :String;
    Const AllowEmpty :Boolean = False;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Swap Characters }
  Function ghSwapChrs (Const Value :TghTwoChrs) :TghTwoChrs;

  { Swap Words }
  Function ghSwapWords (Value :Integer) :Integer;

  { Switch }
  Function ghSwitch (Const Value, OffMask, OnMask :Integer) :Integer;

  { Temporary Directory }
  Function ghTempDir :String;

  { Temporary Path }
  Function ghTempPath (Const NamePrefix :String) :String;

  { Ternary }
  Function ghTernary (Const Value :Boolean) :TghTernary;

  { The Executable? }
  Function ghTheExe (Const Path :String) :Boolean;

  { The Module? }
  Function ghTheModule (Const Path :String) :Boolean;

  { To }
  Function ghTo (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;

  { To }
  Function ghTo (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { To }
  Function ghTo (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;

  { Try Path Sub-Directory }
  Function ghTryPathSubDir (Const Path, SubDir :String) :String;

  { Try Sub-Directory }
  Function ghTrySubDir (Const Dir, SubDir :String) :String;

  { Turn Off }
  Function ghTurnOff (Const Value, Mask :Integer) :Integer;

  { Two Ints }
  Function ghTwoInts (Const Value1, Value2 :Integer) :TghTwoInts; Overload;

  { Two Ints }
  Function ghTwoInts (Const Values :Variant; Const StartIndex :Integer = 0)
    :TghTwoInts; Overload;

  { Type }
  Function ghType (Const Value :Char; Const Info :Integer = ct_CType1)
    :Word;

  { Unit Difference }
  Function ghUnitDiff (Const Groups1, Units1, Groups2, Units2 :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;

  { Unit Difference }
  Function ghUnitDiff (Const Value1, Value2 :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;

  { Units }
  Function ghUnits (Const Groups, Units :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;

  { Units }
  Function ghUnits (Const Value :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;

  { Upper Case }
  Function ghUpper (Const Value :Char) :Char; Overload;

  { Upper Case }
  Function ghUpper (Const Value :String; Const Index :Integer = 1) :Char;
    Overload;

  { Upper First }
  Function ghUppered (Const Value :String; Const Index :Integer = 1)
    :String;

  { Valid? }
  Function ghValid (Const Value :String; Const Chrs :TSysCharSet) :Boolean;

  { Value }
  Function ghValue (Const AVar :Variant; Const Pos :Integer) :Variant;
    Overload;

  { Variant }
  Function ghVar (Const Value :String) :Variant; Overload;

  { Variant }
  Function ghVar (Const Value :Pointer) :Variant; Overload;

  { Variant }
  Function ghVar (Const Value :TVarRec) :Variant; Overload;

  { Variant }
  Function ghVar (Const Values :Array Of Const) :Variant; Overload;

  { Variant Array }
  Function ghVarArr (Const Size :Integer;
    Const AType :TVarType = varVariant;
    Const AllowLessThanTwo :Boolean = True) :Variant; Overload;

  { Variant Array }
  Function ghVarArr (Const Size :Integer;
    Const AllowLessThanTwo :Boolean) :Variant; Overload;

  { Want All Keys? }
  Function ghWantAllKeys (Const Window :THandle) :Boolean;

  { Want Tab? }
  Function ghWantTab (Const Window :THandle) :Boolean;

  { Zero }
  Function ghZero (Const Buffer :Pointer; Const Bytes :Integer) :Integer;

Implementation

  Uses
    StrUtils, Variants, Math, RTLConsts, ShellAPI;

  Var
    Exe :String;

  { Functions and procedures }

  Function ghAccent (Var Str :String; Const Index :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :Boolean;
  Begin
    Result := ghSetChr (Str, Index, ghComposite (Str [Index], Diacritic));
  End;

  Function ghAccentSafe (Var Str :String; Const Index :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :Boolean;
  Begin
    Result := ghIsIndex (Index, Str) And ghAccent (Str, Index, Diacritic);
  End;

  Function ghAccented (Const Value :String; Const Index :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :String;
  Begin
    Result := Value;
    ghAccentSafe (Result, Index, Diacritic);
  End;

  Function ghAccentedEnd (Const Value :String; Const Pos :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :String;
  Begin
    Result := ghAccented (Value, Length (Value) - Pred (Pos), Diacritic);
  End;

  Function ghAccentEnd (Var Str :String; Const Pos :Integer = 1;
    Const Diacritic :TghDiacriticType = ghdtAcute) :Boolean;
  Begin
    Result := ghAccent (Str, Length (Str) - Pred (Pos), Diacritic);
  End;

  Function ghAllocPtr (Const Value :Pointer = Nil) :Pointer;
  Begin
    GetMem (Result, SizeOf (Pointer));
    PPointer (Result)^ := Value;
  End;

  Procedure ghAssign (Var AVar :Variant; Const Index :Integer;
    Const Value :Variant);
  Begin
    If VarIsArray (AVar) Then
      AVar [Index] := Value
    Else
      AVar := Value;
  End;

  Function ghBaseClassName (Const Window :THandle) :String;
  Var
    Name :Array [0..255] Of Char;
  Begin
    { NOTE: RealGetWindowClass does not work with themes (possible Windows
      bug related to window procedures). }
    SetString (Result, Name,
      RealGetWindowClass (Window, Name, Length (Name)));
  End;

  Function ghBeforeContinue (Const Event; Const Sender :TObject;
    Const Method :TghMethod = Nil) :Boolean; Overload;
  Begin
    Result := True;

    If Assigned (TghBeforeContinueEvent (Event)) Then
      TghBeforeContinueEvent (Event) (Sender, Result);

    If Result And Assigned (Method) Then
      Method;
  End;

  Function ghBeforeContinue (Const Event; Const Sender :TObject;
    Const Method :TghBoolMethod) :Boolean; Overload;
  Begin
    Result := ghBeforeContinue (Event, Sender);

    If Result And Assigned (Method) Then
      Result := Method ();
  End;

  Procedure ghBeforeContinueAfter (Const BeforeEvent;
    Const Sender :TObject; Const Method :TghMethod; Const AfterEvent);
  Begin
    ghBeforeContinue (BeforeEvent, Sender, Method);

    If Assigned (TNotifyEvent (AfterEvent)) Then
      TNotifyEvent (AfterEvent) (Sender);
  End;

  Function ghBitBytes (Const Count :Integer) :Integer;
  Begin
    Result := ghDivCeil (Count, 8);
  End;

  Function ghBitOn (Const Value, Bit :Integer) :Boolean; Overload;
  Begin
    Result := Value And (1 Shl Bit) <> 0;
  End;

  Function ghBitOn (Const Buffer :Pointer; Const Bit :Integer) :Boolean;
    Overload;
  Begin
    With ghDivMod (Bit, 8) Do
      Result := ghBitOn (PByteArray (Buffer) [HighValue], LowValue);
  End;

  Function ghBool (Const Value :String) :Boolean; Overload;
  Begin
    Result := StrToBoolDef (Value, False);
  End;

  Function ghBool (Const Value :Variant) :Boolean; Overload;
  Begin
    If VarIsNull (Value) Then
      Result := False
    Else
      If VarIsStr (Value) Then
        Result := ghBool (String (Value))
      Else
        Result := Value;
  End;

  Function ghBool (Const Value :TghTernary) :Boolean; Overload;
  Begin
    Result := Value = ghtrTrue;
  End;

  Function ghButtonType (Const Window :THandle) :Integer;
  Begin
    Result := ghStyle (Window) And ghbsTypeMask;
  End;

  Function ghCall (Const Routine :TghEnumProc; Const Item :Pointer;
    Const Params :Variant; Const DefaultResult :Boolean = False) :Boolean;
    Overload;
  Begin
    If Assigned (Routine) Then
    Begin
      Result := True;
      Routine (Item, Params, Result);
    End
    Else
      Result := DefaultResult;
  End;

  Function ghCall (Const Obj :TObject; Const MethodAddress :Pointer)
    :Pointer; Overload;
  Begin
    Result := TghPtrMethod (ghMethodRec (Obj, MethodAddress)) ();
  End;

  Function ghCall (Const Obj :TObject; Const MethodAddress, Param :Pointer)
    :Pointer; Overload;
  Begin
    Result := TghParamPtrMethod (ghMethodRec (Obj, MethodAddress)) (Param);
  End;

  Function ghCall (Const Obj :TObject;
    Const MethodAddress, Param1, Param2 :Pointer) :Pointer; Overload;
  Begin
    Result := TghTwoParamsPtrMethod (ghMethodRec (Obj, MethodAddress)) (
      Param1, Param2);
  End;

  Function ghCall (Const Obj :TObject; Const MethodAddress, Param1, Param2,
    Param3 :Pointer) :Pointer; Overload;
  Begin
    Result := TghThreeParamsPtrMethod (ghMethodRec (Obj, MethodAddress)) (
      Param1, Param2, Param3);
  End;

  Function ghCallNestedEnumProc (Const ProcParams :Variant;
    Const Item :Pointer; Const DefaultResult :Boolean = False) :Boolean;
  Begin
    Result := ghCall (TghEnumProc (ghPtr (ProcParams [0])), Item,
      PVariant (ghPtr (ProcParams [1]))^, DefaultResult);
  End;

  Function ghCase (Const Value :String) :TghChrCase;
  Begin
    If ghIsUpper (Value) Then
      Result := ghcaUpper
    Else
      Result := TghChrCase (ghIsLower (Value));
  End;

  Function ghCased (Const Value :String; Const Upper :Boolean) :String;
    Overload;
  Begin
    If Upper Then
      Result := ANSIUpperCase (Value)
    Else
      Result := ANSILowerCase (Value);
  End;

  Function ghCased (Const Value :String; Const ACase :TghChrCase) :String;
    Overload;
  Begin
    If ACase = ghcaDefault Then
      Result := Value
    Else
      Result := ghCased (Value, ACase = ghcaUpper);
  End;

  Function ghCased (Const Value, Sample :String) :String; Overload;
  Begin
    Result := ghCased (Value, ghCase (Sample));
  End;

  Function ghCheckBit (Const Value, Mask, OnMask :Integer) :Integer;
  Begin
    If ghHasBit (Value, Mask) Then
      Result := Value
    Else
      Result := Value Or OnMask;
  End;

  Function ghCheckBits (Const Value, Mask, OnMask :Integer) :Integer;
  Begin
    If ghHasBits (Value, Mask) Then
      Result := Value
    Else
      Result := Value Or OnMask;
  End;

  Function ghCheckFlagButton (Const Value, Flag :Integer) :Integer;
  Begin
    Result := ghCheckBit (Value, mb_TypeMask, Flag);
  End;

  Function ghCheckFlagButtonIcon (
    Const Value, ButtonFlag, IconFlag :Integer) :Integer;
  Begin
    Result := ghCheckFlagIcon (ghCheckFlagButton (Value, ButtonFlag),
      IconFlag);
  End;

  Function ghCheckFlagIcon (Const Value, Flag :Integer) :Integer;
  Begin
    Result := ghCheckBit (Value, mb_IconMask, Flag);
  End;

  {$Warn No_RetVal Off}
  Function ghCheckIndex (Const Value, Count :Integer) :Integer;
  Begin
    If ghIsIndex (Value, Count) Then
      Result := Value
    Else
      ghRaiseIndex (Value);
  End;
  {$Warn No_RetVal On}

  Procedure ghCheckLastError;
  Begin
    If GetLastError <> 0 Then
      ghRaiseLastError;
  End;

  Function ghChr (Const Value :Char) :WideChar; Overload;
  Begin
    Result := WideString (Value) [1];
  End;

  Function ghChr (Const Value :WideChar) :Char; Overload;
  Begin
    Result := String (Value) [1];
  End;

  Function ghChr (Const Value :String; Const Index :Integer = 1) :Char;
    Overload;
  Begin
    If ghIsIndex (Index, Value) Then
      Result := Value [Index]
    Else
      Result := #0;
  End;

  Function ghChr (Const Value :WideString; Const Index :Integer = 1)
    :WideChar; Overload;
  Begin
    If ghIsIndex (Index, Value) Then
      Result := Value [Index]
    Else
      Result := #0;
  End;

  Function ghChr (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1) :Char; Overload;
  Begin
    Result := ghChr (Value, ghIndex (Value, Chrs, StartIndex));
  End;

  Function ghChr (Const Value, Chrs :String; Const StartIndex :Integer = 1)
    :Char; Overload;
  Begin
    Result := ghChr (Value, ghChrs (Chrs), StartIndex);
  End;

  Function ghChrEnd (Const Value :String; Const Pos :Integer = 1) :Char;
  Begin
    Result := ghChr (Value, Length (Value) - Pred (Pos));
  End;

  Function ghChrs (Const Value :String) :TSysCharSet; Overload;
  Var
    I :Integer;
  Begin
    Result := [];

    For I := 1 To Length (Value) Do
      Include (Result, Value [I]);
  End;

  Function ghChrs (Const Values :Array Of String) :TSysCharSet; Overload;
  Var
    I :Integer;
  Begin
    Result := [];

    For I := 0 To High (Values) Do
      Result := Result + ghChrs (Values [I]);
  End;

  Function ghChrStrs (Const Value :String; Const Chrs :Array Of String;
    Const StartIndex :Integer = 1) :Char;
  Begin
    Result := ghChr (Value, ghChrs (Chrs), StartIndex);
  End;

  Function ghClass (Const Name :String) :Pointer;
  Begin
    Result := GetClass (Name);
  End;

  Function ghClassButton (Const Window :THandle) :Boolean;
  Begin
    // NOTE: See note at the ghBaseClassName function.
    Result := (ghBaseClassName (Window) = 'Button') And
      ghHasButtonStyle (Window);
  End;

  Function ghClassName (Const Window :THandle) :String;
  Var
    Name :Array [0..255] Of Char;
  Begin
    SetString (Result, Name, GetClassName (Window, Name, Length (Name)));
  End;

  Function ghClone (Const Obj :TObject; Const AClass :TClass;
    Const Owner :TComponent = Nil; Const ExcludedProps :String = '')
    :TObject; Overload;
  Begin
    If Obj Is TComponent Then
      Result := TComponentClass (AClass).Create (Owner)
    Else
      Result := AClass.Create;

    Try
      ghCopyProps (Obj, Result, ExcludedProps);
    Except
      Result.Free;
      Raise;
    End;
  End;

  Function ghClone (Const Obj :TObject; Const Owner :TComponent = Nil;
    Const ExcludedProps :String = '') :TObject; Overload;
  Begin
    Result := ghClone (Obj, Obj.ClassType, Owner, ExcludedProps);
  End;

  Function ghClone (Const Obj :TObject; Const ExcludedProps :String)
    :TObject; Overload;
  Begin
    Result := ghClone (Obj, Obj.ClassType, Nil, ExcludedProps);
  End;

  Function ghComma (Const Value1, Value2 :String;
    Const IncludeEmpties :Boolean = False) :String;
  Begin
    Result := ghConcat (Value1, Value2, ', ', IncludeEmpties);
  End;

  Function ghCommaInts (Const Values :Array Of Integer) :String;
  Begin
    Result := ghConcatInts (Values, ', ');
  End;

  Function ghCommas (Const Values :Array Of String;
    Const IncludeEmpties :Boolean = False) :String; Overload;
  Begin
    Result := ghConcat (Values, ', ', IncludeEmpties);
  End;

  Function ghCommas (Const List :TStrings;
    Const IncludeEmpties :Boolean = False) :String; Overload;
  Begin
    Result := ghCommas (ghStrs (List), IncludeEmpties);
  End;

  Function ghCommaSQLValues (Const Values :Variant) :String;
  Begin
    Result := ghCommas (ghSQLValues (Values), True);
  End;

  {$Warn No_RetVal Off}
  Function ghCompare (Const Value1, Value2 :String;
    Const CompareType :TghStrCompare) :Integer;
  Begin
    Case CompareType Of
      ghscOrdinal :
        Result := CompareStr (Value1, Value2);
      ghscOrdinalAnyCase :
        Result := CompareText (Value1, Value2);
      ghscLocale :
        Result := ANSICompareStr (Value1, Value2);
      ghscLocaleAnyCase :
        Result := ANSICompareText (Value1, Value2);
      ghscSimple :
        Result := ghCompareSimple (Value1, Value2);
      ghscSimpleAnyCase :
        Result := ghCompareSimple (Value1, Value2, True);
    End;
  End;
  {$Warn No_RetVal On}

  Function ghCompareSimple (Value1, Value2 :String;
    Const AnyCase :Boolean = False) :Integer;
  Begin
    If Length (Value1) = Length (Value2) Then
    Begin
      Value1 := ghSimpleStr (Value1);
      Value2 := ghSimpleStr (Value2);
    End;

    If AnyCase Then
      Result := ANSICompareText (Value1, Value2)
    Else
      Result := ANSICompareStr (Value1, Value2);
  End;

  Function ghComponent (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Recurse :Boolean = False)
    :TComponent;
  Var
    I :Integer;
  Begin
    Result := Nil;
    
    For I := 0 To Owner.ComponentCount - 1 Do
    Begin
      If Owner.Components [I] Is AClass Then
        Result := Owner.Components [I]
      Else
        If Recurse Then
          Result := ghComponent (Owner.Components [I], AClass, True);

      If Result <> Nil Then
        Exit;
    End;
  End;

  Function ghComposite (Const Value :Char;
    Const Diacritic :TghDiacriticType) :Char;
  Var
    I :TghDiacriticType;
    J :Integer;
    K :TghChrCase;
  Begin
    For I := Low (TghDiacriticType) To High (TghDiacriticType) Do
      For J := Low (ghltDiacritics [ghdtNone]) To
      High (ghltDiacritics [ghdtNone]) Do
        For K := ghcaLower To ghcaUpper Do
          If ghltDiacritics [I, J, K] = Value Then
          Begin
            // Same letter with the specified diacritic
            Result := ghltDiacritics [Diacritic, J, K];

            If Result = #0 Then
              Result := Value;

            Exit;
          End;

    Result := Value;
  End;

  Function ghComposition (Const Value :Char; Const Typing :Boolean = False)
    :String;
  Var
    AResult :TghTwoChrs;
    Chrs :Integer;
  Begin
    // NOTE: In Spanish, the letter  is not composite.
    If (Value In ['', '']) And (SysLocale.PriLangID = lang_Spanish) Then
      Result := Value
    Else
    Begin
      Chrs := FoldString (map_Composite, @Value, 1, @AResult, 2);

      If Typing Then  // For keyboard entries
        If Chrs = 2 Then
          // First the diacritic and then the letter
          AResult := ghSwapChrs (AResult)
        Else  // Chrs = 1
          If ghIsDiacritic (Value) Then  // It is only an accent mark
          Begin
            AResult.Value2 := ' ';  //  + space -> 
            Chrs := 2;
          End;

      SetString (Result, AResult.Values, Chrs);
    End;
  End;

  Function ghConcat (Const Value1, Value2 :String;
    Const Separator :String = ' ') :String; Overload;
  Begin
    If (Value1 <> '') And (Value2 <> '') Then
      Result := Value1 + Separator + Value2
    Else
      Result := Value1 + Value2;
  End;

  Function ghConcat (Const Value1, Value2, Separator :String;
    Const IncludeEmpties :Boolean) :String; Overload;
  Begin
    If IncludeEmpties Then
      Result := Value1 + Separator + Value2
    Else
      Result := ghConcat (Value1, Value2, Separator);
  End;

  Function ghConcat (Value1, Value2 :String; Const Separator :Char;
    Const IncludeEmpties :Boolean; Const CheckAffixes :Boolean) :String;
    Overload;
  Begin
    If CheckAffixes Then
    Begin
      If ghChrEnd (Value1) = Separator Then
        Value1 := ghDeleteRight (Value1);

      If ghChr (Value2) = Separator Then
        Value2 := ghCopy (Value2, 2)
    End;

    Result := ghConcat (Value1, Value2, Separator, IncludeEmpties);
  End;

  Function ghConcat (Value1, Value2 :String; Const Separator :String;
    Const IncludeEmpties :Boolean; Const CheckAffixes :Boolean) :String;
    Overload;
  Begin
    If CheckAffixes Then
    Begin
      If ANSIEndsStr (Separator, Value1) Then
        Value1 := ghDeleteRight (Value1, Length (Separator));

      If ANSIStartsStr (Separator, Value2) Then
        Value2 := ghCopy (Value2, Length (Separator) + 1);
    End;

    Result := ghConcat (Value1, Value2, Separator, IncludeEmpties);
  End;

  Function ghConcat (Const Values :Array Of String;
    Const Separator :String = ' '; Const IncludeEmpties :Boolean = False;
    Const CheckAffixes :Boolean = False) :String; Overload;
  Var
    I :Integer;
  Begin
    If Length (Values) > 0 Then
    Begin
      Result := Values [0];

      For I := 1 To High (Values) Do
        If (I > 1) And IncludeEmpties And CheckAffixes And
        ((Values [I - 1] = '') Or (Values [I - 1] = Separator)) Then
          { ['a', '', 'c'], ',', True, True -> 'a,' + ',c' = 'a,,c'
            ['a', ',', ',c'], ',', True, True -> 'a,' + ',c' = 'a,,c' }
          Result := Result + ghConcat ('', Values [I], Separator, True,
            True)
        Else
          Result := ghConcat (Result, Values [I], Separator,
            IncludeEmpties, CheckAffixes);
    End
    Else
      Result := '';
  End;

  Function ghConcat (Const List :TStrings; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False;
    Const CheckAffixes :Boolean = False) :String; Overload;
  Begin
    Result := ghConcat (ghStrs (List), Separator, IncludeEmpties,
      CheckAffixes);
  End;

  Function ghConcatInts (Const Values :Array Of Integer;
    Const Separator :String = ' ') :String; Overload;
  Begin
    Result := ghConcat (ghIntStrs (Values), Separator);
  End;

  Function ghConcatInts (Const Values :Array Of Integer;
    Const MinLength :Integer; Const PadChr :Char = '0';
    Const Separator :String = ' ') :String; Overload;
  Begin
    Result := ghConcat (ghIntStrs (Values, MinLength, PadChr), Separator);
  End;

  Function ghConcatInts (Const Values :Array Of Integer;
    Const Format :String; Const Separator :String) :String; Overload;
  Begin
    Result := ghConcat (ghIntStrs (Values, Format), Separator);
  End;

  Function ghConcatPaths (Const Path1, Path2 :String;
    Const APathDelim :Char = PathDelim) :String;
  Begin
    Result := ghConcat (Path1, Path2, APathDelim, False, True);
  End;

  Function ghCopy (Const Value :String; Const StartIndex :Integer) :String;
    Overload;
  Begin
    Result := Copy (Value, StartIndex, MaxInt);
  End;

  Function ghCopy (Const Value :String;
    Const StartIndex, EndIndex :Integer) :String; Overload;
  Begin
    Result := Copy (Value, StartIndex,
      EndIndex - ghMinZero (StartIndex - 1));
  End;

  Function ghCopyMem (Const Buffer :Pointer; Const Bytes :Integer;
    Size :Integer = MaxInt) :Pointer; Overload;
  Begin
    If Buffer <> Nil Then
    Begin
      If Size = MaxInt Then
        Size := Bytes;

      GetMem (Result, Size);

      Try
        Move (Buffer^, Result^, Bytes);
      Except
        FreeMem (Result);
        Raise;
      End;
    End
    Else
      Result := Nil;
  End;

  Function ghCopyProp (Const Source, Dest :TObject; Const Prop :PPropInfo)
    :Boolean;
  Begin
    If Prop.SetProc <> Nil  Then
    Begin
      Case Prop.PropType^.Kind Of
        { Object }
        tkClass : SetObjectProp (Dest, Prop, GetObjectProp (Source, Prop));
        { Event }
        tkMethod : SetMethodProp (Dest, Prop, GetMethodProp (Source, Prop))
        Else
          SetPropValue (Dest, Prop.Name, GetPropValue (Source, Prop.Name,
            False));
      End;

      Result := True;
    End
    Else
      Result := False;
  End;

  Procedure ghCopyProps (Const Source, Dest :TObject;
    Const Excludeds :String = '');
  Var
    ExcludedNames :TStringDynArray;
    Events :Boolean;
    I :Integer;
    Props :TghPPropInfoArr;
  Begin
    Props := ghProps (Dest);
    ExcludedNames := ghSplit (Excludeds, ';');

    // Substring '(events)' = not copy event properties
    Events := ghPosStr (ExcludedNames, '(events)') = -1;

    For I := 0 To High (Props) Do
      If (Events Or (Props [I].PropType^.Kind <> tkMethod)) And
      (ghPosStr (ExcludedNames, Props [I].Name) = -1) Then
        ghCopyProp (Source, Dest, Props [I]);
  End;
  
  Function ghCount (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1; EndIndex :Integer = MaxInt) :Integer;
    Overload;
  Var
    I :Integer;
  Begin
    If EndIndex = MaxInt Then
      EndIndex := Length (Value);

    Result := 0;

    For I := StartIndex To EndIndex Do
      If Value [I] = Chr Then
        Inc (Result);
  End;

  Function ghCount (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1; EndIndex :Integer = MaxInt) :Integer;
    Overload;
  Var
    I :Integer;
  Begin
    If EndIndex = MaxInt Then
      EndIndex := Length (Value);

    Result := 0;

    For I := StartIndex To EndIndex Do
      If Value [I] In Chrs Then
        Inc (Result);
  End;

  Function ghCountStr (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
  Var
    I :Integer;
  Begin
    Result := 0;
    I := 1;

    While ghIsIndex (I, Value) Do
    Begin
      I := ghPos (Value, SubStr, I, CompareType);

      If I > 0 Then
      Begin
        Inc (Result);
        Inc (I, Length (SubStr));
      End;
    End;
  End;

  Function ghCountTo (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer) :Integer; Overload;
  Begin
    Result := ghCount (Value, Chr, 1, EndIndex);
  End;

  Function ghCountTo (Const Value :String; Const Chrs :TSysCharSet;
    Const EndIndex :Integer) :Integer; Overload;
  Begin
    Result := ghCount (Value, Chrs, 1, EndIndex);
  End;

  Function ghDate (Const Value :TghDateParts) :TDateTime;
  Begin
    Result := EncodeDate (Value.Year, Value.Month, Value.Day);
  End;

  Function ghDateParts (Const Value :TDateTime) :TghDateParts; Overload;
  Var
    ADay, AMonth :Word;
  Begin
    DecodeDate (Value, Result.Year, AMonth, ADay);
    Result.Month := AMonth;
    Result.Day := ADay;
  End;

  Function ghDateParts :TghDateParts; Overload;
  Begin
    Result := ghDateParts (Date);
  End;

  Function ghDecimal (Const Value :Extended; Const Precision :Integer = 18;
    Const Decimals :Integer = 9999) :TFloatRec;
  Begin
    FloatToDecimal (Result, Value, fvExtended, Precision, Decimals);
  End;

  Function ghDecimals (Const Value :Extended) :Integer;
  Begin
    If Frac (Value) <> 0 Then
      With ghDecimal (Value) Do
        { When Frac (Value) <> 0 and Exponent < 18, StrLen (Digits) is
          greater than Exponent.  StrLen (Digits) - Exponent =
          "represented" decimals in Value. }
        If Exponent < 18 Then
          Result := Integer (StrLen (Digits)) - Exponent
        Else
          Result := 0  // Value has no "represented" decimals
    Else
      Result := 0;
  End;

  Function ghDefaultIconModule (Const Icon :PChar) :THandle;
  Begin
    If ghIconStd (Icon) Then
      Result := 0
    Else
      Result := HInstance;
  End;

  Function ghDelete (Const Value :String; Const Index :Integer = 1;
    Const Chrs :Integer = 1) :String;
  Begin
    Result := Value;
    Delete (Result, Index, Chrs);
  End;

  Function ghDelete (Const Value :String; Const Chr :Char) :String;
    Overload;
  Var
    Index :Integer;
  Begin
    Result := Value;
    Index := 1;

    While Index <= Length (Result) Do
      If Result [Index] = Chr Then
        Result := ghDelete (Result, Index)
      Else
        Inc (Index);
  End;

  Function ghDelete (Const Value :String; Const Chrs :TSysCharSet) :String;
    Overload;
  Var
    Index :Integer;
  Begin
    Result := Value;
    Index := 1;

    While Index <= Length (Result) Do
      If Result [Index] In Chrs Then
        Result := ghDelete (Result, Index)
      Else
        Inc (Index);
  End;

  Function ghDeleteLeft (Const Value :String; Const Chr :Char) :String;
    Overload;
  Var
    I :Integer;
  Begin
    I := 1;

    While (I <= Length (Value)) And (Value [I] = Chr) Do
      Inc (I);

    Result := ghCopy (Value, I);
  End;

  Function ghDeleteLeft (Const Value :String; Const Chrs :TSysCharSet)
    :String; Overload;
  Var
    I :Integer;
  Begin
    I := 1;

    While (I <= Length (Value)) And (Value [I] In Chrs) Do
      Inc (I);

    Result := ghCopy (Value, I);
  End;

  Function ghDeleteRight (Const Value :String; Const Chrs :Integer = 1)
    :String; Overload;
  Begin
    Result := LeftStr (Value, Length (Value) - Chrs);
  End;        

  Function ghDeleteRight (Const Value :String; Const Chr :Char) :String;
    Overload;
  Var
    I :Integer;
  Begin
    I := Length (Value);

    While (I > 0) And (Value [I] = Chr) Do
      Dec (I);

    Result := LeftStr (Value, I);
  End;

  Function ghDeleteRight (Const Value :String; Const Chrs :TSysCharSet)
    :String; Overload;
  Var
    I :Integer;
  Begin
    I := Length (Value);

    While (I > 0) And (Value [I] In Chrs) Do
      Dec (I);

    Result := LeftStr (Value, I);
  End;

  Function ghDialogCode (Const Window :THandle) :Integer;
  Begin
    Result := ghSendMsg (Window, wm_GetDlgCode);
  End;                           

  Function ghDialogCodeButton (Const Code :Integer) :Boolean;
  Begin
    Result := ghHasBit (Code, dlgc_DefPushButton Or dlgc_UnDefPushButton);
  End;

  Function ghDialogCodeTab (Const Code :Integer) :Boolean;
  Begin
    Result := ghHasBit (Code, dlgc_WantAllKeys Or dlgc_WantTab);
  End;

  Function ghDir (Const Path :String; Const UpLevels :Integer = 1;
    Const APathDelim :Char = PathDelim; Const ConsiderRoot :Boolean = True)
    :String; Overload;
  Var
    DriveLength, I :Integer;
    EndsPathDelim :Boolean Absolute I;
  Begin
    Result := '';

    If Path = '' Then
      Exit;

    EndsPathDelim := ghChrEnd (Path) = APathDelim;

    If (UpLevels < 1) And Not EndsPathDelim Then
      Result := Path
    Else
    Begin
      { We consider drive/UNC roots only if ConsiderRoot is True and
        APathDelim has the value of '\' }
      If ConsiderRoot And (APathDelim = PathDelim) Then
        DriveLength := Length (ExtractFileDrive (Path))
      Else
        DriveLength := 0;

      If UpLevels > 0 Then
        I := ghIndexALast (Path, APathDelim,
          UpLevels + Ord (EndsPathDelim), DriveLength + 1)
      Else
        I := Length (Path);

      If I > 0 Then
        Result := LeftStr (Path, I - Ord ((DriveLength <> 2) Or
          (I > DriveLength + 1) Or (Path [DriveLength] <> DriveDelim)));
    End;
  End;

  Function ghDir (Const Path :String; Const UpLevels :Integer;
    Const ConsiderRoot :Boolean) :String; Overload;
  Begin
    Result := ghDir (Path, UpLevels, PathDelim, ConsiderRoot);
  End;

  Function ghDir (Const Path :String; Const APathDelim :Char;
    Const ConsiderRoot :Boolean = True) :String; Overload;
  Begin
    Result := ghDir (Path, 1, APathDelim, ConsiderRoot);
  End;

  Function ghDir (Const Path :String; Const ConsiderRoot :Boolean) :String;
    Overload;
  Begin
    Result := ghDir (Path, 1, ConsiderRoot);
  End;

  Function ghDirPath (Const Path :String; Const UpLevels :Integer;
    Const SubPath :String; Const APathDelim :Char = PathDelim) :String;
    Overload;
  Begin
    Result := ghConcatPaths (ghDir (Path, UpLevels, APathDelim), SubPath,
      APathDelim);
  End;

  Function ghDirPath (Const Path, SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;
  Begin
    Result := ghDirPath (Path, 1, SubPath, APathDelim);
  End;

  Function ghDirPath (Const UpLevels :Integer; Const SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;
  Begin
    Result := ghDirPath (ghExe, UpLevels, SubPath, APathDelim);
  End;

  Function ghDirPath (Const SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;
  Begin
    Result := ghDirPath (1, SubPath, APathDelim);
  End;

  Function ghDivCeil (Const Dividend, Divisor :Integer) :Integer;
  Begin
    Result := (Dividend + (Divisor - 1)) Div Divisor;
  End;

  Function ghDivMod (Const Dividend, Divisor :Integer) :TghTwoInts;
    Overload;
  Begin
    Result.HighValue := Dividend Div Divisor;
    Result.LowValue := Dividend Mod Divisor;
  End;

  Function ghDivMod (Const Dividend, Divisor :Extended;
    Const Decimals :Integer = 0) :TghTwoFloats; Overload;
  Begin
    { ghDivMod (10.24, 3, 2) -> High = 3.41, Low = 0.01
      (3.41 * 3) + 0.01 = 10.24 }

    // 10.24 / 3 = 3.41333... -> 3.41
    Result.HighValue := ghRound (Dividend / Divisor, Decimals);

    // 10.24 - (3.41 * 3) = 10.24 - 10.23 = 0.01
    Result.LowValue := Dividend - (Result.HighValue * Divisor);
  End;

  Function ghDivModSafe (Const Dividend, Divisor :Integer) :TghTwoInts;
  Begin
    If Divisor = 0 Then
      Result.Int64Value := 0
    Else
      Result := ghDivMod (Dividend, Divisor);
  End;

  Function ghDivSafe (Const Dividend, Divisor :Integer) :Integer;
  Begin
    If Divisor = 0 Then
      Result := 0
    Else
      Result := Dividend Div Divisor;
  End;

  Function ghEnds (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
  Begin
    Result := (Value <> '') And (SubStr <> '') And
      ghEquals (RightStr (Value, Length (SubStr)), SubStr, CompareType);
  End;

  Function ghEnds (Const Value :String; Const Chrs :TSysCharSet;
    ChrCount :Integer; Const SubStr :String;
    Const SubStrCompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
  Var
    I :Integer;  // "From" Chrs index
    J :Integer Absolute ChrCount;  // "To" Chrs index
  Begin
    Result := False;

    If (Value = '') Or (SubStr = '') Then
      Exit;

    I := (Length (Value) - Length (SubStr)) - Max (ChrCount - 1, -1);

    If I < 1 Then  // SubStr > Value Or not enough characters for ChrCount
      Exit
    Else
    Begin
      Inc (J, I - 1);

      While I <= J Do
        If Value [I] In Chrs Then
          Inc (I)
        Else
          Exit;
    End;

    Result := ghEquals (Value, I, SubStr, SubStrCompareType);
  End;

  Function ghEnds (Const Value :String; Const Chrs :TSysCharSet;
    Const SubStr :String;
    Const SubStrCompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
  Begin
    Result := ghEnds (Value, Chrs, 1, SubStr, SubStrCompareType);
  End;

  Function ghEndsWith (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
  Begin
    Result := (Length (Value) > Length (SubStr)) And
      ghEnds (Value, SubStr, CompareType);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Recurse :Boolean;
    Const Routine :TghEnumProc; Const Params :Variant) :TComponent;
    Overload;
  Var
    I :Integer;
  Begin
    { NOTE: Use the TghObjectList's enumeration instead of this function
      if Routine can alter the Components/ComponentCount properties. }

    For I := 0 To Owner.ComponentCount - 1 Do
    Begin
      Result := Owner.Components [I];

      If ((AClass <> Nil) And Not (Result Is AClass)) Or
      ghCall (Routine, Result, Params) Then
        If Recurse Then
          Result := ghEnumComponents (Owner.Components [I], AClass, True,
            Routine, Params)
        Else
          Result := Nil;

      If Result <> Nil Then
        Exit;
    End;

    Result := Nil;
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Routine :TghEnumProc;
    Const Params :Variant) :TComponent; Overload;
  Begin
    Result := ghEnumComponents (Owner, AClass, False, Routine, Params);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Recurse :Boolean; Const Routine :TghEnumProc;
    Const Params :Variant) :TComponent; Overload;
  Begin
    Result := ghEnumComponents (Owner, TComponentClass (Nil), Recurse,
      Routine, Params);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Routine :TghEnumProc; Const Params :Variant) :TComponent;
    Overload;
  Begin
    Result := ghEnumComponents (Owner, TComponentClass (Nil), False,
      Routine, Params);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Recurse :Boolean;
    Const Routine :TghEnumProc) :TComponent; Overload;
  Begin
    Result := ghEnumComponents (Owner, AClass, Recurse, Routine,
      Unassigned);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const AClass :TComponentClass; Const Routine :TghEnumProc) :TComponent;
    Overload;
  Begin
    Result := ghEnumComponents (Owner, AClass, False, Routine, Unassigned);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Recurse :Boolean; Const Routine :TghEnumProc) :TComponent;
    Overload;
  Begin
    Result := ghEnumComponents (Owner, TComponentClass (Nil), Recurse,
      Routine, Unassigned);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Routine :TghEnumProc) :TComponent; Overload;
  Begin
    Result := ghEnumComponents (Owner, TComponentClass (Nil), False,
      Routine, Unassigned);
  End;

  Function ghEnumProps (Const Props :TghPPropInfoArr;
    Const Kinds :TTypeKinds; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;
  Begin
    For Result := 0 To High (Props) Do
      If (Props [Result].PropType^.Kind In Kinds) And Not ghCall (Routine,
      Props [Result], Params) Then
        Exit;

    Result := -1;
  End;

  Function ghEnumProps (Const Obj :TObject; Const Kinds :TTypeKinds;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;
  Begin
    Result := ghEnumProps (ghProps (Obj), Kinds, Routine, Params);
  End;

  Function ghEnumProps (Const Obj :TObject; Const Kinds :TTypeKinds;
    Const Routine :TghEnumProc = Nil) :Integer; Overload;
  Begin
    Result := ghEnumProps (Obj, Kinds, Routine, Null);
  End;

  Function ghEnumPropsObj (Const Obj :TObject; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;
  Begin
    Result := ghEnumProps (Obj, [tkClass], Routine, Params);
  End;

  Function ghEnumPropsObj (Const Obj :TObject; Const Routine :TghEnumProc)
    :Integer; Overload;
  Begin
    Result := ghEnumPropsObj (Obj, Routine, Null);
  End;

  Function ghEnumPropsObj (Const Props :TghPPropInfoArr;
    Const AClass :TClass; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;
  Begin
    For Result := 0 To High (Props) Do
      If (Props [Result].PropType^.Kind = tkClass) And ((AClass = Nil) Or
      GetTypeData (Props [Result].PropType^).ClassType.InheritsFrom (
      AClass)) And Not ghCall (Routine, Props [Result], Params) Then
        Exit;

    Result := -1;
  End;

  Function ghEnumPropsObj (Const Obj :TObject; Const AClass :TClass;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;
  Begin
    Result := ghEnumPropsObj (ghProps (Obj), AClass, Routine, Params);
  End;

  Function ghEquals (Const Value1, Value2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
  Begin
    Result := ghCompare (Value1, Value2, CompareType) = 0;
  End;

  Function ghEquals (Const Value1 :String; Const StartIndex :Integer;
    Const Value2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
  Begin
    Result := ghEquals (ghCopy (Value1, StartIndex), Value2, CompareType);
  End;

  Function ghEquals (Const Value1, Value2 :Variant) :Boolean; Overload;
  Begin
    Result := (VarType (Value1) = VarType (Value2)) And (Value1 = Value2);
  End;

  Function ghExe :String;
  Begin
    If Exe = '' Then
      Exe := ParamStr (0);

    Result := Exe;
  End;

  Function ghExeDir (Const UpLevels :Integer = 1;
    Const APathDelim :Char = PathDelim) :String;
  Begin
    Result := ghDir (ghExe, UpLevels, APathDelim);
  End;

  Function ghExeName :String;
  Begin
    Result := ExtractFileName (ghExe);
  End;

  Function ghExecute (Const Path :String; Const Params :String = '')
    :Boolean;
  Begin
    Result := ShellExecute (0, 'open', PChar (Path), PChar (Params), Nil,
      sw_Show) > 32;
  End;

  Function ghExt (Const Value :String) :String;
  Begin
    Result := ghPrefixed (Value, '.');
  End;

  Function ghExtExe (Const Ext :String = '') :String;
  Begin
    Result := ghExtPath (ghExe, Ext);
  End;

  Function ghExtExeName (Const Ext :String = '') :String;
  Begin
    Result := ghExtPath (ghExeName, Ext);
  End;

  Function ghExtPath (Const Path :String; Const Ext :String = '') :String;
  Begin
    Result := ChangeFileExt (Path, ghExt (Ext));
  End;

  Function ghFlagButton (Const Value, Flag :Integer) :Integer;
  Begin
    Result := ghSwitch (Value, mb_TypeMask, Flag);
  End;

  Function ghFlagIcon (Const Value, Flag :Integer) :Integer;
  Begin
    Result := ghSwitch (Value, mb_IconMask, Flag);
  End;

  Function ghFindIcon (Const Module :THandle; Const NameID :Integer;
    Const Size :PCardinal = Nil) :THandle; Overload;
  Begin
    Result := ghFindResource (Module, NameID, rt_Group_Icon, Size);
  End;

  Function ghFindIcon (Const Module :THandle; Const Name :String;
    Const Size :PCardinal = Nil) :THandle; Overload;
  Begin
    Result := ghFindIcon (Module, Integer (Name), Size);
  End;

  Function ghFindIcon (Const NameID :Integer; Const Size :PCardinal = Nil)
    :THandle; Overload;
  Begin
    Result := ghFindIcon (HInstance, NameID, Size);
  End;

  Function ghFindIcon (Const Name :String; Const Size :PCardinal = Nil)
    :THandle; Overload;
  Begin
    Result := ghFindIcon (HInstance, Name, Size);
  End;

  Function ghFindResource (Const Module :THandle; Const NameID :Integer;
    Const AType :PChar; Const Size :PCardinal = Nil) :THandle; Overload;
  Begin
    Result := FindResource (Module, MakeIntResource (NameID), AType);

    If (Result <> 0) And (Size <> Nil) Then
    Begin
      Size^ := SizeOfResource (Module, Result);

      If Size^ = 0 Then
        ghRaiseLastError;
    End;
  End;

  Function ghFindResource (Const Module :THandle; Const Name :String;
    Const AType :PChar; Const Size :PCardinal = Nil) :THandle; Overload;
  Begin
    Result := ghFindResource (Module, Integer (Name), AType, Size);
  End;

  Function ghFindResource (Const Name :String; Const AType :PChar;
    Const Size :PCardinal = Nil) :THandle; Overload;
  Begin
    Result := ghFindResource (HInstance, Name, AType, Size);
  End;

  Function ghFloat (Const Value :String) :Extended; Overload;
  Begin
    Result := StrToFloatDef (Value, 0);
  End;

  Function ghFloat (Const Value :Variant) :Extended; Overload;
  Begin
    If VarIsNull (Value) Then
      Result := 0
    Else
      If VarIsStr (Value) Then
        Result := ghFloat (String (Value))
      Else
        Result := Value;
  End;

  Function ghFormatOptional (Const AFormat, Param :String) :String;
  Begin
    Result := Format (AFormat, [ghIncPositive (Length (Param)), Param]);
  End;

  Function ghFrac (Const Value :Extended; Const Decimals :Integer)
    :Extended; Overload;
  Begin
    Result := Frac (Value);

    If Decimals < MaxInt Then
      Result := ghRound (Result, Decimals);
  End;

  Function ghFrac (Const Value :Extended; Const Decimal :Boolean;
    Const Decimals :Integer = MaxInt) :Extended; Overload;
  Begin
    If Decimal Then
      Result := ghFracDecimal (Value, Decimals)
    Else
      Result := ghFrac (Value, Decimals);
  End;

  Function ghFracDecimal (Const Value :Extended;
    Decimals :Integer = MaxInt) :Extended;
  Var
    Buffer :PChar Absolute Result;
    Desc :TFloatRec;  // Decimal descriptor
  Begin
    { Frac (-123.0045) -> -0.0044999...
      ghFracDecimal (-123.0045) -> -0.0045 }

    Result := Frac (Value);

    If Result = 0 Then
      Exit;

    { If Value is not a fraction but has fractional part, then we try to
      get it based on its decimal representation }
    If Abs (Value) > 1 Then
    Begin
      Desc := ghDecimal (Value);

      { When Frac (Value) <> 0 and Exponent < 18, StrLen (Digits) is
        greater than Exponent.  StrLen (Digits) - Exponent = "represented"
        decimals in Value.  If Exponent >= 18 then Value has no
        "represented" decimals. }
      If Desc.Exponent < 18 Then
      Begin
        If (Decimals < MaxInt) And (Integer (StrLen (Desc.Digits)) -
        Desc.Exponent <= Decimals) Then
          Decimals := MaxInt;

        { -123.0045, Digits = ['1', '2', '3', '0', '0', '4', '5', #0...] ->
          ['1', '-', '.', '0', '0', '4', '5', #0...], Buffer = '-.0045'.
          NOTE: This operation can overwrite the Desc's Exponent and
          Negative fields. ---------------------------------------------- }

        Buffer := PChar (@Desc.Digits) + (Desc.Exponent - 2);

        If Desc.Negative Then
          Buffer [0] := '-'
        Else
          Inc (Buffer);

        Buffer [Byte (Buffer [0] = '-')] := DecimalSeparator;
        //-----------------------------------------------------------------

        TextToFloat (Buffer, Result, fvExtended);  // '-.0045' -> -0.0045
      End;
    End;

    If Decimals < MaxInt Then
      Result := ghRound (Result, Decimals);
  End;

  Function ghFracInt (Const Value :Extended; Const Decimals :Integer)
    :Integer;
  Begin
    // ghFracInt (51.287, 2) = ghRoundInt (28.7) -> 29
    Result := ghRoundInt (ghFracShift (Value, -Decimals));
  End;

  Function ghFracRound (Const Value :Extended; Const Epsilon :Extended = 0)
    :Integer;
  Begin
    Result := CompareValue (Abs (ghFracDecimal (Value)), 0.5, Epsilon);

    If Result = LessThanValue Then  // -1 -> 0, GreaterThanValue = 1
      Result := 0
    Else
      If Result = EqualsValue Then  // Fraction = 0.5 +/- Epsilon
        Case GHMidpointRounding Of
          ghmrUnbiased : Result := Byte (Not ghIntEven (Value));
          ghmrPositive : Result := Byte (Value > 0);
          ghmrNegative : Result := Byte (Value < 0);
          Else
            Result := Byte (GHMidpointRounding = ghmrInfinity);
        End;

    If (Result = 1) And (Value < 0) Then
      Result := -1;
  End;

  Function ghFracShift (Const Value :Extended; Const Decimals :Integer)
    :Extended;
  Begin
    { ghFracShift (51.287, -2) = 0.287 * 100 -> 28.7
      ghFracShift (51.287, 1) = 0.287 * 0.1 -> 0.0287 }
    Result := Frac (Value) * IntPower (10, -Decimals);
  End;

  Function ghFracZero (Const Value :Extended;
    Const Decimal :Boolean = False; Const Epsilon :Extended = 0;
    Const Decimals :Integer = MaxInt) :Extended; Overload;
  Begin
    Result := ghFrac (Value, Decimal);

    If Result <> 0 Then
      If ghIsZeroOr (Abs (Result), 1, Epsilon) Then
        Result := 0  // 0.000... or 0.999... -> 0
      Else
        If Decimals < MaxInt Then
          // NOTE: We do not pass Epsilon here.
          Result := ghRound (Result, Decimals);
  End;

  Function ghFracZero (Const Value :Extended; Const Decimal :Boolean;
    Const Decimals :Integer) :Extended; Overload;
  Begin
    Result := ghFracZero (Value, Decimal, 0, Decimals);
  End;

  Function ghFracZero (Const Value :Extended; Const Epsilon :Extended;
    Const Decimals :Integer = MaxInt) :Extended; Overload;
  Begin
    Result := ghFracZero (Value, False, Epsilon, Decimals);
  End;

  Function ghFracZero (Const Value :Extended; Const Decimals :Integer)
    :Extended; Overload;
  Begin
    Result := ghFracZero (Value, False, 0, Decimals);
  End;

  Procedure ghFreeNil (Var Ptr);
  Begin
    FreeMem (Pointer (Ptr));
    Pointer (Ptr) := Nil;
  End;

  Procedure ghFreeObjPtr (Const Ptr);
  Var
    APtr :PObject Absolute Ptr;
  Begin
    If APtr = Nil Then
      Exit;

    APtr.Free;
    FreeMem (APtr);
  End;

  Procedure ghFreeObjPtrNil (Var Ptr);
  Begin
    ghFreeObjPtr (Ptr);
    Pointer (Ptr) := Nil;
  End;

  Procedure ghFreePtrPtr (Const Ptr);
  Var
    APtr :PPointer Absolute Ptr;
  Begin
    If APtr = Nil Then
      Exit;

    FreeMem (APtr^);
    FreeMem (APtr);         
  End;

  Procedure ghFreePtrPtrNil (Var Ptr);
  Begin
    ghFreePtrPtr (Ptr);
    Pointer (Ptr) := Nil;
  End;

  Function ghFrom (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer = MaxInt) :String; Overload;
  Var
    I :Integer;
  Begin
    I := ghIndex (Value, Chr);

    If I > 0 Then
      Result := ghCopy (Value, I, EndIndex)
    Else
      Result := '';
  End;

  Function ghFrom (Const Value, SubStr :String;
    Const EndIndex :Integer = MaxInt;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Var
    I :Integer;
  Begin
    I := ghPos (Value, SubStr, CompareType);

    If I > 0 Then
      Result := ghCopy (Value, I, EndIndex)
    Else
      Result := '';
  End;

  Function ghFrom (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;
  Begin
    Result := ghFrom (Value, SubStr, MaxInt, CompareType);
  End;

  Function ghGender (Const Letter :Char) :TghGender;
  Begin
    Case Letter Of
      'M', 'm' : Result := ghgnMasculine;
      'F', 'f' : Result := ghgnFeminine;
      Else
        Result := ghgnUndefined;
    End;
  End;

  Function ghGenderNumberStr (Const Values :Array Of String;
    Const Gender :TghGender; Const Plural :Boolean = False) :String;
  Begin
    { Spanish example:
      ghGenderNumberStr (['autor', 'autores', 'autora', 'autoras'],
        ghgnFeminine, False) -> 'autora' (feminine singular of "autor") }

    Result := Values [ghMinZero (((Byte (Gender) * 2) -

      { Same form for neuter and masculine, or same form for neuter,
        masculine and feminine? }
      (6 - Length (Values))) +

      Byte (Plural))];
  End;

  Function ghGetDialogCodeButton (Const Window :THandle) :Boolean;
  Begin
    Result := ghDialogCodeButton (ghDialogCode (Window));
  End;

  Function ghGreater (Const Value1, Value2 :Extended;
    Const Epsilon :Extended = 0) :Boolean;
  Begin
    Result := CompareValue (Value1, Value2, Epsilon) = GreaterThanValue;
  End;

  Function ghGUID :TGUID;
  Begin
    CreateGUID (Result);
  End;

  Function ghGUIDStr :String;
  Begin
    Result := GUIDToString (ghGUID ());
  End;

  Function ghHasBit (Const Value, Mask :Integer) :Boolean;
  Begin
    Result := (Value And Mask) <> 0;
  End;

  Function ghHasBits (Const Value, Mask :Integer) :Boolean; Overload;
  Begin
    Result := (Mask <> 0) And ((Value And Mask) = Mask);
  End;

  Function ghHasBits (Const Value :Integer;
    Const Masks :Array Of Integer) :Boolean; Overload;
  Var
    I :Integer;
  Begin
    For I := 0 To High (Masks) Do
      If ghHasBits (Value, Masks [I]) Then
      Begin
        Result := True;
        Exit;
      End;

    Result := False;
  End;

  Function ghHasButtonStyle (Const Window :THandle) :Boolean;
  Begin
    Result := ghButtonType (Window) In [bs_PushButton, bs_DefPushButton];
  End;

  Function ghHasValue (Const Value :Variant) :Boolean;
  Begin
    Result := ghMoreNull (Value) And Not VarIsClear (Value);
  End;

  Function ghHigh (Const Value :Variant; Const Dim :Integer = 1) :Integer;
  Begin
    If VarIsArray (Value) Then
      Result := VarArrayHighBound (Value, Dim)
    Else
      Result := 0;
  End;

  Function ghIconFlag (Const Mark :Char) :Integer;
  Begin
    Case Mark Of
      '!' : Result := mb_IconWarning;
      '.' : Result := mb_IconStop;
      '?' : Result := mb_IconQuestion;
      '_' : Result := mb_UserIcon;
      '*' : Result := mb_IconInformation;
      Else
        Result := 0;
    End;
  End;

  Function ghIconStd (Const Icon :PChar) :Boolean;
  Begin
    Result := ghInRange (Icon, IDI_Application, IDI_WinLogo);
  End;

  Function ghInBits (Const Value, Mask :Integer) :Boolean;
  Begin
    Result := (Value <> 0) And ((Value And Not Mask) = 0);
  End;

  Function ghInc (Var Int :Integer; Const Inc :Integer = 1) :Integer;
    Overload;
  Begin
    Int := Int + Inc;
    Result := Int;
  End;

  Function ghInc (Const Value :String; Const Inc :Integer = 1) :String;
    Overload;
  Begin
    Result := ghRightDigits (Value);

    If Result <> '' Then
      Result := ghDeleteRight (Value, Length (Result)) +
        ghStr (StrToInt (Result) + Inc, Length (Result))
    Else
      Result := Value + IntToStr (Inc);
  End;

  Function ghInc (Const Value :Variant; Const Inc :Integer = 1) :Variant;
    Overload;
  Begin
    If VarIsNull (Value) Then
      Result := Inc
    Else
      If VarIsStr (Value) Then
        Result := ghInc (String (Value), Inc)
      Else
        Result := Value + Inc;
  End;

  Function ghIncCircular (Const Value, MinValue, MaxValue :Integer;
    Const Inc :Integer = 1) :Integer;
  Var
    RangeSize :Integer;
  Begin
    RangeSize := MaxValue - Pred (MinValue);
    Result := Value + Inc;

    If Result < MinValue Then
      Result := MaxValue - (Pred (MinValue - Result) Mod RangeSize)
    Else
      If Result > MaxValue Then
        Result := MinValue + (Pred (Result - MaxValue) Mod RangeSize);
  End;
  
  Function ghIncPositive (Const Value :Integer; Const Inc :Integer = 1)
    :Integer;
  Begin
    If Value > 0 Then
      Result := Value + Inc
    Else
      Result := Value;
  End;

  Function ghIncUnits (Const Groups, Units :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9;
    Const AInc :Integer = 1) :TghTwoInts; Overload;
  Begin
    Result := ghDivMod (ghUnits (Groups, Units, MinUnit, MaxUnit) + AInc,
      MaxUnit - Pred (MinUnit));
    Inc (Result.LowValue, MinUnit);
  End;

  Function ghIncUnits (Const Value :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9;
    Const AInc :Integer = 1) :TghTwoInts; Overload;
  Begin
    Result := ghIncUnits (Value.HighValue, Value.LowValue, MinUnit,
      MaxUnit, AInc);
  End;

  Function ghIndex (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer; Overload;
  Begin
    For Result := StartIndex To Length (Value) Do
      If Value [Result] = Chr Then
        Exit;

    Result := 0;
  End;

  Function ghIndex (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1) :Integer; Overload;
  Begin
    For Result := StartIndex To Length (Value) Do
      If Value [Result] In Chrs Then
        Exit;

    Result := 0;
  End;

  Function ghIndex (Const Value, Chrs :String;
    Const StartIndex :Integer = 1) :Integer; Overload;
  Begin
    Result := ghIndex (Value, ghChrs (Chrs), StartIndex);
  End;

  Function ghIndex (Const List :TStrings; Const Str :String;
    Const StartIndex :Integer = 0;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    For Result := StartIndex To List.Count - 1 Do
      If ghEquals (List [Result], Str, CompareType) Then
        Exit;

    Result := -1;
  End;

  Function ghIndex (Const List :TStrings; Const Str :String;
    Const CompareType :TghStrCompare) :Integer; Overload;
  Begin
    Result := ghIndex (List, Str, 0, CompareType);
  End;

  Function ghIndexALast (Const Value :String; Const Chr :Char;
    Occurrence :Integer; Const StartIndex :Integer = 1) :Integer;
  Begin
    If Occurrence > 0 Then
      For Result := Length (Value) DownTo StartIndex Do
        If (Value [Result] = Chr) And (ghInc (Occurrence, -1) = 0) Then
          Exit;

    Result := 0;
  End;
  
  Function ghIndexLast (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer;
  Begin
    For Result := Length (Value) DownTo StartIndex Do
      If Value [Result] = Chr Then
        Exit;

    Result := 0;
  End;

  Function ghIndexLeft (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer;
  Begin
    Result := ghIndex (Value, Chr, StartIndex) - 1;

    If Result = -1 Then
      Result := Length (Value);
  End;

  Function ghIndexLeftLast (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer;
  Begin
    Result := ghIndexLast (Value, Chr, StartIndex) - 1;

    If Result = -1 Then
      Result := Length (Value);
  End;

  Function ghIndexMax (Const Value, Count :Integer) :Integer;
  Begin
    Result := Min (Value, Count - 1);
  End;

  Function ghIndexMin (Const Value :Integer) :Integer;
  Begin
    Result := Max (Value, 0);
  End;

  Function ghInheritsFrom (AClass :TClass; Const Name :String) :Boolean;
  Begin
    While (AClass <> Nil) And Not AClass.ClassNameIs (Name) Do
      AClass := AClass.ClassParent;

    Result := AClass <> Nil;
  End;

  Function ghInRange (Const Value, MinValue, MaxValue :Cardinal) :Boolean;
    Overload;
  Begin
    Result := (Value >= MinValue) And (Value <= MaxValue);
  End;

  Function ghInRange (Const Value, MinValue, MaxValue :Pointer) :Boolean;
    Overload;
  Begin
    Result := ghInRange (Cardinal (Value), Cardinal (MinValue),
      Cardinal (MaxValue));///change for Win64
  End;

  Function ghInt (Const Value :String) :Integer; Overload;
  Begin
    Result := StrToIntDef (Value, 0);
  End;

  Function ghInt (Const Value :String; Const StartIndex :Integer) :Integer;
    Overload;
  Begin
    Result := ghInt (ghCopy (Value, StartIndex));
  End;

  Function ghInt (Const Value :String; Const StartIndex, Chrs :Integer)
    :Integer; Overload;
  Begin
    Result := ghInt (Copy (Value, StartIndex, Chrs));
  End;

  Function ghInt (Const Value :Variant) :Integer; Overload;
  Begin
    If VarIsNull (Value) Then
      Result := 0
    Else
      If VarIsStr (Value) Then
        Result := ghInt (String (Value))
      Else
        Result := Trunc (Value);
  End;

  Function ghIntEven (Const Value :Extended) :Boolean;
  Begin
    Result := Frac (Int (Value) / 2) = 0;
  End;

  Function ghIntStrs (Const Values :Array Of Integer) :TStringDynArray;
    Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, Length (Values));

    For I := 0 To High (Values) Do
      Result [I] := IntToStr (Values [I]);
  End;

  Function ghIntStrs (Const Values :Array Of Integer;
    Const MinLength :Integer; Const PadChr :Char = '0') :TStringDynArray;
    Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, Length (Values));

    For I := 0 To High (Values) Do
      Result [I] := ghStr (Values [I], MinLength, PadChr);
  End;

  Function ghIntStrs (Const Values :Array Of Integer;
    Const AFormat :String) :TStringDynArray; Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, Length (Values));

    For I := 0 To High (Values) Do
      Result [I] := Format (AFormat, [Values [I]]);
  End;

  Function ghIsBool (Const Value :Variant; Const Bool :Boolean) :Boolean;
  Begin
    Result := VarIsType (Value, varBoolean) And (Value = Bool);
  End;

  Function ghIsChrs (Const Value :String; Const Chrs :TSysCharSet)
    :Boolean;
  Begin
    Result := (Value <> '') And ghValid (Value, Chrs);
  End;

  Function ghIsDate (Const Value :String) :Boolean;
  Begin
    Result := TryStrToDate (Value, GHDUDateTime);
  End;

  Function ghIsDiacritic (Const Value :Char) :Boolean;
  Begin
    Result := ghHasBits (ghType (Value, ct_CType3),
      c3_Diacritic Or c3_Symbol);
  End;

  Function ghIsFloat (Const Value :String) :Boolean; Overload;
  Begin
    Result := TryStrToFloat (Value, GHDUExtended);
  End;

  Function ghIsFloat (Const Value :Variant) :Boolean; Overload;
  Begin
    Result := ghIsFloat (VarToStr (Value));
  End;

  Function ghIsHex (Const Value :Char) :Boolean; Overload;
  Begin
    Result := UpCase (Value) In ghdgHex;
  End;

  Function ghIsHex (Const Value :String) :Boolean; Overload;
  Begin
    Result := ghIsChrs (UpperCase (Value), ghdgHex);
  End;

  Function ghIsIconFlag (Const Flag :Integer) :Boolean;
  Begin
    Result := (Flag = mb_UserIcon) Or
      ((Flag Shr 4) In [mb_IconStop Shr 4..mb_IconInformation Shr 4]);
  End;

  Function ghIsIndex (Const Value, Count :Integer) :Boolean; Overload;
  Begin
    Result := (Value >= 0) And (Value < Count);
  End;

  Function ghIsIndex (Const Value :Integer; Const Str :String) :Boolean;
    Overload;
  Begin
    Result := (Value > 0) And (Value <= Length (Str));
  End;

  Function ghIsIndex (Const Value :Integer; Const Str :WideString)
    :Boolean; Overload;
  Begin
    Result := (Value > 0) And (Value <= Length (Str));
  End;

  Function ghIsInt (Const Value :Extended; Const Epsilon :Extended = 0)
    :Boolean; Overload;
  Begin
    Result := ghFracZero (Value, Epsilon) = 0;
  End;

  Function ghIsInt (Const Value :String) :Boolean; Overload;
  Begin
    Result := TryStrToInt (Value, GHDUInt);
  End;

  Function ghIsLower (Const Value :String) :Boolean;
  Begin
    Result := Value = ANSILowerCase (Value);
  End;

  Function ghISODate (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (ghfmISODate, Value);
  End;

  Function ghISODate :String; Overload;
  Begin
    Result := ghISODate (Date);
  End;

  Function ghISODateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (ghfmISODateTime, Value);
  End;

  Function ghISODateTime :String; Overload;
  Begin
    Result := ghISODateTime (Now);
  End;

  Function ghIsOrNil (Const Obj :TObject; Const AClass :TClass) :Boolean;
  Begin
    Result := (Obj = Nil) Or (Obj Is AClass);
  End;

  Function ghISOTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (ghfmISOTime, Value);
  End;

  Function ghISOTime :String; Overload;
  Begin
    Result := ghISOTime (Time);
  End;

  Function ghIsPtr (Const Value :String) :Boolean;
  Begin
    Result := (Length (Value) = 8) And ghIsHex (Value);
  End;

  Function ghIsUpper (Const Value :String) :Boolean;
  Begin
    Result := Value = ANSIUpperCase (Value);
  End;

  Function ghIsValid (Const Value :Single) :Boolean; Overload;
  Begin
    Result := Not IsNAN (Value) And Not IsInfinite (Value);
  End;

  Function ghIsValid (Const Value :Double) :Boolean; Overload;
  Begin
    Result := Not IsNAN (Value) And Not IsInfinite (Value);
  End;

  Function ghIsValid (Const Value :Extended) :Boolean; Overload;
  Begin
    Result := Not IsNAN (Value) And Not IsInfinite (Value);
  End;

  Function ghIsZeroOr (Const Value, Other :Extended;
    Const Epsilon :Extended = 0) :Boolean;
  Begin
    { True if Value equals to 0, 0 +/- Epsilon, Other or Other +/- Epsilon.
      NOTE: Epsilon 0 -> Math.ExtendedResolution. }
    Result := IsZero (Value, Epsilon) Or SameValue (Value, Other, Epsilon);
  End;

  Function ghKeyChr (Const Key :Integer) :Char;
  Begin
    Result := Char (MapVirtualKey (Key, 2 { MapVK_VK_To_Char }));
  End;

  Function ghLastErrorMsg :String;
  Begin
    Result := SysErrorMessage (GetLastError);
  End;

  Function ghLastMinute (Const Value :TDateTime) :TDateTime; Overload;
  Begin
    Result := Trunc (Value) + ghmxTimeMinute;
  End;

  Function ghLastMinute :TDateTime; Overload;
  Begin
    Result := Date + ghmxTimeMinute;
  End;

  Function ghLeft (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghIndexLeft (Value, Chr, StartIndex));
  End;

  Function ghLeft (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghPosLeft (Value, SubStr, StartIndex, CompareType));
  End;

  Function ghLeft (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;
  Begin
    Result := ghLeft (Value, SubStr, 1, CompareType);
  End;

  Function ghLeftLast (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghIndexLeftLast (Value, Chr, StartIndex));
  End;

  Function ghLeftLast (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghPosLeftLast (Value, SubStr, StartIndex, CompareType));
  End;

  Function ghLeftLast (Const Value, SubStr1, SubStr2 :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghPosLeftLast (Value, SubStr1, SubStr2, StartIndex, CompareType));
  End;

  Function ghLeftOf (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghIndex (Value, Chr, StartIndex) - 1);
  End;

  Function ghLeftOf (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghPos (Value, SubStr, StartIndex, CompareType) - 1);
  End;

  Function ghLeftOf (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;
  Begin
    Result := ghLeftOf (Value, SubStr, 1, CompareType);
  End;

  Function ghLeftOfALast (Const Value :String; Const Chr :Char;
    Const Occurrence :Integer; Const StartIndex :Integer = 1) :String;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghIndexALast (Value, Chr, Occurrence, StartIndex) - 1);
  End;

  Function ghLeftOfLast (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghIndexLast (Value, Chr, StartIndex) - 1);
  End;

  Function ghLeftRight (Const Value :String; Const Chr :Char) :TghTwoStrs;
    Overload;
  Var
    I :Integer;
  Begin
    I := ghIndex (Value, Chr);

    If I = 0 Then
    Begin
      Result.Value1 := Value;
      Result.Value2 := '';
    End
    Else
    Begin
      Result.Value1 := LeftStr (Value, I - 1);
      Result.Value2 := ghCopy (Value, I + 1);
    End;
  End;

  Function ghLeftRight (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :TghTwoStrs;
    Overload;
  Var
    I :Integer;
  Begin
    I := ghPos (Value, SubStr, 1, CompareType);

    If I = 0 Then
    Begin
      Result.Value1 := Value;
      Result.Value2 := '';
    End
    Else
    Begin
      Result.Value1 := LeftStr (Value, I - 1);
      Result.Value2 := ghCopy (Value, I + Length (SubStr));
    End;
  End;

  Function ghLength (Const Value :Variant; Const Dim :Integer = 1)
    :Integer;
  Begin
    If VarIsArray (Value) Then
      Result := ghLengthDim (Value, Dim)
    Else
      Result := 1;
  End;

  Function ghLengthAll (Const Value :Variant) :Integer;
  Var
    I :Integer;
  Begin
    Result := VarArrayDimCount (Value);

    If Result = 0 Then
      Exit;

    I := Result;
    Result := 1;

    For I := 1 To I Do
      Result := Result * ghLengthDim (Value, I)
  End;

  Function ghLengthDim (Const Value :Variant; Const Dim :Integer) :Integer;
  Begin
    Result := Succ (VarArrayHighBound (Value, Dim) -
      VarArrayLowBound (Value, Dim));
  End;

  Function ghLess (Const Value1, Value2 :Extended;
    Const Epsilon :Extended = 0) :Boolean;
  Begin
    Result := CompareValue (Value1, Value2, Epsilon) = LessThanValue;
  End;

  Function ghLoadResource (Const Module :THandle; Const NameID :Integer;
    Const AType :PChar; Const Size :PCardinal = Nil) :THandle; Overload;
  Begin
    Result := ghFindResource (Module, NameID, AType, Size);

    If Result <> 0 Then
    Begin
      Result := LoadResource (Module, Result);

      If Result = 0 Then
        ghRaiseLastError;
    End;
  End;

  Function ghLoadResource (Const Module :THandle; Const Name :String;
    Const AType :PChar; Const Size :PCardinal = Nil) :THandle; Overload;
  Begin
    Result := ghLoadResource (Module, Integer (Name), AType, Size);
  End;

  Function ghLockResource (Const Module :THandle; Const NameID :Integer;
    Const AType :PChar; Const Size :PCardinal = Nil) :Pointer; Overload;
  Begin
    THandle (Result) := ghLoadResource (Module, NameID, AType, Size);

    If Result <> Nil Then
      Result := LockResource (THandle (Result));
  End;

  Function ghLockResource (Const Module :THandle; Const Name :String;
    Const AType :PChar; Const Size :PCardinal = Nil) :Pointer; Overload;
  Begin
    Result := ghLockResource (Module, Integer (Name), AType, Size);
  End;

  Function ghLow (Const Value :Variant; Const Dim :Integer = 1) :Integer;
  Begin
    If VarIsArray (Value) Then
      Result := VarArrayLowBound (Value, Dim)
    Else
      Result := 0;
  End;

  Function ghLower (Const Value :Char) :Char; Overload;
  Begin
    Result := Char (CharLower (Pointer (Value)));
  End;

  Function ghLower (Const Value :String; Const Index :Integer = 1) :Char;
    Overload;
  Begin
    Result := ghLower (ghChr (Value, Index));
  End;

  Function ghLowered (Const Value :String; Const Index :Integer = 1)
    :String;
  Begin
    Result := Value;
    ghSetLowerSafe (Result, Index);
  End;

  Function ghMax (Const Value1, Value2 :TghTwoInts) :TghTwoInts;
  Begin
    Result := TghTwoInts (Max (Value1.Int64Value, Value2.Int64Value));
  End;

  Function ghMaxSplitInt (Const Value :String;
    Const Separator :String = ' '; Const IncludeEmpties :Boolean = False)
    :Integer;
  Begin
    If Value <> '' Then
      Result := MaxIntValue (
        ghSplitInts (Value, Separator, IncludeEmpties))
    Else
      Result := 0;
  End;

  Function ghMethodInfos (AClass :TClass) :TghMethodInfoArr;
  Var
    I :Integer;
    Table :PghMethodTable;
  Begin
    Repeat
      Table := ghMethodTable (AClass);

      If Table <> Nil Then
      Begin
        I := Length (Result);
        SetLength (Result, I + Table.Count);

        For I := I To High (Result) Do
        Begin
          Result [I] := @Table.First;
          Table := ghOffset (Table, Table.First.Size);
        End;
      End;

      AClass := AClass.ClassParent;
    Until AClass = Nil;
  End;

  Function ghMethodRec (Const Obj, Address :Pointer) :TMethod;
  Begin
    Result.Data := Obj;
    Result.Code := Address;
  End;

  Function ghMethodTable (Const AClass :TClass) :PghMethodTable;
  Begin
    Result := PghMethodTable (ghOffset (AClass, vmtMethodTable)^);
  End;

  Function ghMin (Const Value1, Value2 :TghTwoInts) :TghTwoInts;
  Begin
    Result := TghTwoInts (Min (Value1.Int64Value, Value2.Int64Value));
  End;

  Function ghMinCardinal (Const Value1, Value2 :Cardinal) :Cardinal;
  Begin
    If Value1 < Value2 Then
      Result := Value1
    Else
      Result := Value2;
  End;

  Function ghMinMax (Const Value1, Value2 :Integer) :TghTwoInts;
  Begin
    If Value1 < Value2 Then
    Begin
      Result.Value1 := Value1;
      Result.Value2 := Value2;
    End
    Else
    Begin
      Result.Value1 := Value2;
      Result.Value2 := Value1;
    End
  End;

  Function ghMinZero (Const Value :Integer) :Integer;
  Begin
    If Value > 0 Then
      Result := Value
    Else
      Result := 0;
  End;

  Function ghModulePath (Const Module :THandle) :String; Overload;
  Var
    Buffer :Array [0..Max_Path] Of Char;
  Begin
    SetString (Result, Buffer,
      GetModuleFileName (Module, Buffer, Length (Buffer)));
  End;

  Function ghModulePath :String; Overload;
  Begin
    Result := ghModulePath (HInstance);
  End;

  Function ghMoreNull (Const Value :Variant) :Boolean;
  Begin
    // Not varEmpty or varNull
    Result := FindVarData (Value).VType > varNull;
  End;

  Function ghMsgIconFlag (Const Value :TSysCharSet) :Integer;
  Begin
    Result := ghIconFlag (ghChr ('!?', Value));
  End;

  Function ghMsgIconFlag (Const Value :String) :Integer; Overload;
  Begin
    Result := ghMsgIconFlag (ghChrs (Value));
  End;

  Function ghMsgIconFlagStrs (Const Values :Array Of String) :Integer;
    Overload;
  Begin
    Result := ghMsgIconFlag (ghChrs (Values));
  End;

  Function ghMsgPending (Const Window :THandle;
    Const MinID, MaxID :Integer) :Boolean; Overload;
  Var
    Msg :TMsg;
  Begin
    Result := ghPeekMsg (Msg, Window, MinID, MaxID);
  End;

  Function ghMsgPending (Const Window :THandle = 0; Const ID :Integer = 0)
    :Boolean; Overload;
  Begin
    Result := ghMsgPending (Window, ID, ID);
  End;

  Function ghNotEmpty (Const Value :Variant) :Variant;
  Begin
    If VarIsEmpty (Value) Then
      Result := Null
    Else
      Result := Value;
  End;

  Function ghNotPrefixed (Const Value :String; Const Prefix :Char) :String;
    Overload;
  Begin
    If ghChr (Value) = Prefix Then
      Result := ghCopy (Value, 2)
    Else
      Result := Value;
  End;

  Function ghNotPrefixed (Const Value, Prefix :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    If ghStarts (Value, Prefix, CompareType) Then
      Result := ghCopy (Value, Length (Prefix) + 1)
    Else
      Result := Value;
  End;

  Function ghNotSuffixed (Const Value :String; Const Suffix :Char) :String;
    Overload;
  Begin
    If ghChrEnd (Value) = Suffix Then
      Result := ghDeleteRight (Value)
    Else
      Result := Value;
  End;

  Function ghNotSuffixed (Const Value, Suffix :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    If ghEnds (Value, Suffix, CompareType) Then
      Result := ghDeleteRight (Value, Length (Suffix))
    Else
      Result := Value;
  End;

  Function ghOffset (Const Ptr :Pointer; Const Increment :Integer)
    :Pointer;
  Begin
    If Ptr <> Nil Then
      Result := Pointer (Integer (Ptr) + Increment)
    Else
      Result := Nil;
  End;

  Function ghOLEVar (Const Value :String) :OLEVariant; Overload;
  Begin
    Result := OLEVariant (ghVar (Value));
  End;

  Function ghOLEVar (Const Value :TVarRec) :OLEVariant; Overload;
  Begin
    Result := OLEVariant (ghVar (Value));
  End;

  Function ghOne (Const Value :Boolean) :Integer;
  Begin
    Result := (Byte (Value) * 2) - 1;
  End;

  Function ghOwns (Const Owner :TComponent; Owned :TComponent) :Boolean;
  Begin
    While (Owned <> Nil) And (Owned <> Owner) Do
      Owned := Owned.Owner;

    Result := Owned <> Nil;
  End;

  {$Warn Symbol_Platform Off}
  Function ghParams (Const ATrim :Boolean = False) :String;
  Begin
    If CmdLine [0] = '"' Then
      Result := ghCopy (CmdLine, ghIndex (CmdLine, '"', 2) + 2)
    Else
      Result := ghRightOf (CmdLine, ' ');

    If ATrim Then
      Result := Trim (Result);
  End;
  {$Warn Symbol_Platform On}

  Function ghPathName (Const Path :String) :String;
  Begin
    Result := ghExtPath (ExtractFileName (Path));
  End;

  Function ghPeekMsg (Out Msg :TMsg; Const Window :THandle;
    Const MinID, MaxID :Integer; Const Remove :Boolean = False) :Boolean;
    Overload;
  Begin
    Result := PeekMessage (Msg, Window, MinID, MaxID, Byte (Remove));
  End;

  Function ghPeekMsg (Out Msg :TMsg; Const Window :THandle = 0;
    Const ID :Integer = 0; Const Remove :Boolean = False) :Boolean;
    Overload;
  Begin
    Result := ghPeekMsg (Msg, Window, ID, ID, Remove);
  End;

  Function ghPercent (Const Value, Percentage :Extended) :Extended;
    Overload;
  Begin
    Result := Value * (Percentage / 100);
  End;

  Function ghPercent (Const Value, Percentage :Extended;
    Const Decimals :Integer) :Extended; Overload;
  Begin
    Result := ghRound (ghPercent (Value, Percentage), Decimals);
  End;

  Function ghPercent100 (Const Value, Percentage :Extended) :Extended;
    Overload;
  Begin
    Result := Value * (100 / Percentage);
  End;

  Function ghPercent100 (Const Value, Percentage :Extended;
    Const Decimals :Integer) :Extended; Overload;
  Begin
    Result := ghRound (ghPercent100 (Value, Percentage), Decimals);
  End;

  Function ghPercentage (Const Value, Portion :Extended) :Extended;
    Overload;
  Begin
    If IsZero (Portion) Then
      Result := 0
    Else
      Result := (100 / Value) * Portion;
  End;

  Function ghPercentage (Const Value, Portion :Extended;
    Const Decimals :Integer) :Extended; Overload;
  Begin
    Result := ghRound (ghPercentage (Value, Portion), Decimals);
  End;

  Function ghPlatStr (Const Value :String) :TghPlatStr;
  Begin
    If Win32Platform = ver_Platform_Win32_NT Then
    Begin
      Result.Dummy := WideString (Value);
      Result.Ptr := Pointer (Result.Dummy);
    End
    Else
      Result.Ptr := Pointer (Value);
  End;

  Function ghPos (Value, SubStr :String; Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    If ghPreparePos (Value, SubStr, CompareType) Then
      Result := PosEx (SubStr, Value, StartIndex)
    Else
      Result := 0;
  End;

  Function ghPos (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :Integer; Overload;
  Begin
    Result := ghPos (Value, SubStr, 1, CompareType);
  End;

  Function ghPosInt (Const Values :Array Of Integer; Const Int :Integer;
    Const StartPos :Integer = 0) :Integer;
  Begin
    For Result := StartPos To High (Values) Do
      If Values [Result] = Int Then
        Exit;

    Result := -1;
  End;

  Function ghPosLast (Value, SubStr :String; Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Var
    SubStrLength :Integer;
  Begin
    If ghPreparePos (Value, SubStr, CompareType) Then
    Begin
      SubStrLength := Length (SubStr);

      For Result := (Length (Value) - SubStrLength) + 1 DownTo
      StartIndex Do
        If StrLComp (@Value [Result], Pointer (SubStr), SubStrLength) =
        0 Then
          Exit;
    End;

    Result := 0
  End;

  Function ghPosLast (Value, SubStr1, SubStr2 :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    Result := ghPosLast (Value, SubStr2, Max (StartIndex, ghPosLast (
      Value, SubStr1, StartIndex, CompareType) + 1), CompareType);
  End;

  Function ghPosLeft (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
  Begin
    Result := ghPos (Value, SubStr, StartIndex, CompareType) - 1;

    If Result = -1 Then
      Result := Length (Value);
  End;

  Function ghPosLeftLast (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    Result := ghPosLast (Value, SubStr, StartIndex, CompareType) - 1;

    If Result = -1 Then
      Result := Length (Value);
  End;

  Function ghPosLeftLast (Const Value, SubStr1, SubStr2 :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    Result := ghPosLast (Value, SubStr1, SubStr2, StartIndex,
      CompareType) - 1;

    If Result = -1 Then
      Result := Length (Value);
  End;
  
  Function ghPosPtr (Const Values :Array Of Pointer; Const Ptr :Pointer;
    Const StartPos :Integer = 0) :Integer;
  Begin
    For Result := StartPos To High (Values) Do
      If Values [Result] = Ptr Then
        Exit;

    Result := -1;
  End;

  Function ghPosRight (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    Result := ghPos (Value, SubStr, StartIndex, CompareType);

    If Result > 0 Then
      Result := Result + Length (SubStr)
    Else
      Result := 1;
  End;

  Function ghPosRight (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :Integer; Overload;
  Begin
    Result := ghPosRight (Value, SubStr, 1, CompareType);
  End;

  Function ghPosStr (Const Values :Array Of String; Const Str :String;
    Const StartPos :Integer = 0;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    For Result := StartPos To High (Values) Do
      If ghEquals (Values [Result], Str, CompareType) Then
        Exit;

    Result := -1;
  End;

  Function ghPosStr (Const Values :Array Of String; Const Str :String;
    Const CompareType :TghStrCompare) :Integer; Overload;
  Begin
    Result := ghPosStr (Values, Str, 0, CompareType);
  End;

  Function ghPrefixed (Const Value :String; Const Prefix :Char;
    Const AllowEmpty :Boolean = False) :String; Overload;
  Begin
    If Value = '' Then
      If AllowEmpty Then
        Result := Prefix
      Else
        Result := ''
    Else
      If Value [1] = Prefix Then
        Result := Value
      Else
        Result := Prefix + Value;
  End;

  Function ghPrefixed (Const Value, Prefix :String;
    Const AllowEmpty :Boolean = False;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    If Value = '' Then
      If AllowEmpty Then
        Result := Prefix
      Else
        Result := ''
    Else
      If ghStarts (Value, Prefix, CompareType) Then
        Result := Value
      Else
        Result := Prefix + Value;
  End;

  Function ghPreparePos (Var Str1, Str2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
  Begin
    If (Str1 = '') Or (Str2 = '') Then
      Result := False
    Else
    Begin
      Case CompareType Of
        ghscOrdinalAnyCase :
        Begin
          Str1 := UpperCase (Str1);
          Str2 := UpperCase (Str2);
        End;

        ghscLocaleAnyCase :
        Begin
          Str1 := ANSIUpperCase (Str1);
          Str2 := ANSIUpperCase (Str2);
        End;

        ghscSimple :
        Begin
          Str1 := ghSimpleStr (Str1);
          Str2 := ghSimpleStr (Str2);
        End;

        ghscSimpleAnyCase :
        Begin
          Str1 := ghSimpleStrUpper (Str1);
          Str2 := ghSimpleStrUpper (Str2);
        End;
      End;

      Result := True;
    End;
  End;

  Function ghPropBool (Const Obj :TObject; Const Name :String) :Boolean;
  Begin
    Result := ghBool (ghPropValue (Obj, Name));
  End;

  Function ghPropObj (Const Obj :TObject; Const Name :String;
    Const AClass :TClass = Nil) :TObject; Overload;
  Begin
    If IsPublishedProp (Obj, Name) Then
      Result := GetObjectProp (Obj, Name, AClass)
    Else
      Result := Nil;
  End;

  Function ghPropObj (Const Obj :TObject; Const AClass :TClass = Nil)
    :TObject; Overload;

    // Call back routine
    Procedure GetProp (Prop :PPropInfo; Const Params :Variant;
      Var Continue :Boolean);
    Begin
      PObject (ghPtr (Params [1]))^ := GetObjectProp (
        TObject (ghPtr (Params [0])), Prop);
      Continue := False;
    End;
  Begin
    Result := Nil;
    ghEnumPropsObj (Obj, AClass, @GetProp,
      VarArrayOf ([ghVar (Obj), ghVar (@Result)]));
  End;

  Function ghProps (Const AClass :TClass) :TghPPropInfoArr; Overload;
  Begin
    SetLength (Result, GetTypeData (AClass.ClassInfo).PropCount);

    If Length (Result) > 0 Then
      GetPropInfos (AClass.ClassInfo, Pointer (Result));
  End;

  Function ghProps (Const Obj :TObject) :TghPPropInfoArr; Overload;
  Begin
    Result := ghProps (Obj.ClassType);
  End;

  Function ghPropStr (Const Obj :TObject; Const Name :String) :String;
  Begin
    Result := VarToStr (ghPropValue (Obj, Name));
  End;

  Function ghPropValue (Const Obj :TObject; Const Name :String) :Variant;
  Begin
    If IsPublishedProp (Obj, Name) Then
      Result := GetPropValue (Obj, Name, False)
    Else
      Result := Null;
  End;

  Function ghPtr (Const Value :String) :Pointer; Overload;
  Begin
    If ghIsPtr (Value) Then
      Result := Pointer (StrToInt ('$' + Value))
    Else
      Result := Nil;
  End;

  Function ghPtr (Const Value :Variant) :Pointer; Overload;
  Begin
    If ghMoreNull (Value) Then
      If VarIsStr (Value) Then
        Result := ghPtr (String (Value))
      Else
        Result := Pointer (Cardinal (Value))
    Else
      Result := Nil;
  End;

  Function ghQuotedISODate (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghISODate (Value));
  End;

  Function ghQuotedISODate :String; Overload;
  Begin
    Result := ghQuotedISODate (Date);
  End;

  Function ghQuotedISODateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghISODateTime (Value));
  End;

  Function ghQuotedISODateTime :String; Overload;
  Begin
    Result := ghQuotedISODateTime (Now);
  End;

  Function ghQuotedISOTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghISOTime (Value));
  End;

  Function ghQuotedISOTime :String; Overload;
  Begin
    Result := ghQuotedISOTime (Time);
  End;

  Function ghQuotedSQLDate (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghSQLDate (Value));
  End;

  Function ghQuotedSQLDate :String; Overload;
  Begin
    Result := ghQuotedSQLDate (Date);
  End;

  Function ghQuotedSQLDateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghSQLDateTime (Value));
  End;

  Function ghQuotedSQLDateTime :String; Overload;
  Begin
    Result := ghQuotedSQLDateTime (Now);
  End;

  Function ghQuotedSQLTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghSQLTime (Value));
  End;

  Function ghQuotedSQLTime :String; Overload;
  Begin
    Result := ghQuotedSQLTime (Time);
  End;

  {$StackFrames On}
  Procedure ghRaise (Const Error :String; Const Callers :Integer = 1);
    Overload;
  Begin
    Raise Exception.Create (Error) At ghReturnAddress (Callers);
  End;
  {$StackFrames Off}

  Procedure ghRaise (Const Error :String; Const Params :Array Of Const;
    Const Callers :Integer = 1); Overload;
  Begin
    ghRaise (Format (Error, Params), Callers + 1);
  End;

  Procedure ghRaiseIndex (Const Value :Integer;
    Const Callers :Integer = 1);
  Begin
    ghRaise (SListIndexError, [Value], Callers + 1);
  End;

  Procedure ghRaiseLastError (
    Const AFormat :String = '%s (GetLastError %d).';
    Const Callers :Integer = 1);
  Var
    Code :Integer;
  Begin
    { NOTE: We must call the GetLastError method before doing anything
      else. }
    Code := GetLastError;
    ghRaise (AFormat, [ghLastErrorMsg, Code], Callers + 1);
  End;

  Function ghReferIf (Var Str :String; Const Value :String) :PString;
  Begin
    If Str = Value Then
      Result := @Str
    Else
      Result := Nil;
  End;

  Function ghReplace (Const Value :String; Const Chr, NewChr :Char)
    :String; Overload;
  Var
    I :Integer;
  Begin
    Result := Value;

    For I := 1 To Length (Result) Do
      If Result [I] = Chr Then
        Result [I] := NewChr;
  End;

  Function ghReplace (Const Value :String; Const Chrs :TSysCharSet;
    Const NewChr :Char) :String; Overload;
  Var
    I :Integer;
  Begin
    Result := Value;

    For I := 1 To Length (Result) Do
      If Result [I] In Chrs Then
        Result [I] := NewChr;
  End;

  Procedure ghRereadShortStr (Const Reader :TReader;
    Var ARightStr :String);
  Var
    Buffer :Array [0..255] Of Byte;
    Count, I :Integer;
  Begin
    { ghRereadShortStr allows the application to get the short string that
      has just been read with a TReader object.  NOTE: The application must
      call this function again with the returned string if it is considered
      incomplete. }

    Count := Reader.Position;
    Reader.Position := ghMinZero (Reader.Position - SizeOf (Buffer));
    Count := Count - Reader.Position;
    Reader.Read (Buffer, Count);

    // We look for the tentative length byte from right to left
    For I := Count - (Length (ARightStr) + 2) DownTo 0 Do
      If Buffer [I] = Count - Succ (I) Then
      Begin
        ARightStr := PShortString (@Buffer [I])^;
        Exit;
      End;

    ghRaise ('Short string not found in reader.');
  End;

  { NOTE: This function takes into acount only the Callers most direct
    caller routines that have an EBP stack frame.  The compiler does not
    generate stack frame for this function. }
  Function ghReturnAddress (Callers :Integer = 1) :Pointer;
  Asm
    Mov ECX, EBP  // Stack frame of the most direct caller

    @Loop:            // Repeat
      Dec EAX         // Dec (Callers)
      JZ  @Exit       // If Callers = 0 Then Break
      Mov ECX, [ECX]  // Stack frame (EBP) of previous caller
      Jmp @Loop       // Until False

    @Exit:
      Mov EAX, [ECX + 4]  // Result := Return address of "that" caller
  End;

  Function ghReverse (Const List :TList) :TList;
  Var
    I :Integer;
  Begin
    If List.Count > 1 Then
      For I := 0 To (List.Count Div 2) - 1 Do
        List.Exchange (I, Pred (List.Count) - I);

    Result := List;
  End;

  Function ghRight (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer = MaxInt) :String; Overload;
  Begin
    Result := ghCopy (Value, ghIndex (Value, Chr) + 1, EndIndex);
  End;

  Function ghRight (Const Value, SubStr :String;
    Const EndIndex :Integer = MaxInt;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    Result := ghCopy (Value, ghPosRight (Value, SubStr, CompareType),
      EndIndex);
  End;

  Function ghRight (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;
  Begin
    Result := ghRight (Value, SubStr, MaxInt, CompareType);
  End;

  Function ghRightChrs (Const Value :String; Const Chrs :TSysCharSet;
    Const MaxLength :Integer = MaxInt) :String;
  Var
    I :Integer;
  Begin
    Result := '';

    For I := Length (Value) DownTo 1 Do
      If (Length (Result) < MaxLength) And (Value [I] In Chrs) Then
        Result := Value [I] + Result
      Else
        Exit;
  End;

  Function ghRightDigits (Const Value :String;
    Const MaxLength :Integer = MaxInt) :String;
  Begin
    Result := ghRightChrs (Value, ghdgDecimal, MaxLength);
  End;

  Function ghRightInt (Const Value :String;
    Const MaxLength :Integer = MaxInt) :Integer;
  Begin
    Result := ghInt (ghRightDigits (Value, MaxLength));
  End;

  Function ghRightLeft (Const Value :String; Const Chr1, Chr2 :Char)
    :String; Overload;
  Begin
    Result := ghLeft (ghRight (Value, Chr1), Chr2);
  End;

  Function ghRightLeft (Const Value, SubStr1, SubStr2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    Result := ghLeft (ghRight (Value, SubStr1, CompareType), SubStr2,
      CompareType);
  End;

  Function ghRightLeftOf (Const Value :String; Const Chr1, Chr2 :Char)
    :String; Overload;
  Begin
    Result := ghRightOf (Value, Chr1);

    If Result <> '' Then
      Result := ghLeftOf (Result, Chr2);
  End;

  Function ghRightLeftOf (Const Value, SubStr1, SubStr2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    Result := ghRightOf (Value, SubStr1, CompareType);

    If Result <> '' Then
      Result := ghLeftOf (Result, SubStr2, CompareType);
  End;

  Function ghRightOf (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer = MaxInt) :String; Overload;
  Var
    I :Integer;
  Begin
    I := ghIndex (Value, Chr);

    If I > 0 Then
      Result := ghCopy (Value, I + 1, EndIndex)
    Else
      Result := '';
  End;

  Function ghRightOf (Const Value, SubStr :String;
    Const EndIndex :Integer = MaxInt;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Var
    I :Integer;
  Begin
    I := ghPos (Value, SubStr, CompareType);

    If I > 0 Then
      Result := ghCopy (Value, I + Length (SubStr), EndIndex)
    Else
      Result := '';
  End;

  Function ghRightOf (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;
  Begin
    Result := ghRightOf (Value, SubStr, MaxInt, CompareType);
  End;

  Function ghRound (Const Value :Extended; Const Decimals :Integer = 0;
    Const Epsilon :Extended = 0) :Extended;
  Begin
    If (Decimals = MaxInt) Or (Value = 0) Then
      Result := Value
    Else
      If Decimals = 0 Then
        Result := Int (Value) + ghFracRound (Value, Epsilon)
      Else
        Result := ghRoundMultiple (Value, IntPower (10, -Decimals),
          Epsilon);
  End;

  Function ghRoundInt (Const Value :Extended; Const Epsilon :Extended = 0)
    :Int64;
  Begin
    Result := Trunc (ghRound (Value, 0, Epsilon));
  End;

  Function ghRoundMultiple (Const Value, Divisor :Extended;
    Const Epsilon :Extended = 0) :Extended;
  Begin
    Result := Value / Divisor;  // First we get the quotient

    // If Value is already multiple of Divisor
    If ghIsInt (Result, Epsilon) Then
      Result := Value
    Else
      // We round the quotient to integer and multiply it by Divisor
      Result := ghRound (Result, 0, Epsilon) * Divisor;
  End;

  Function ghSearchPath (Const Path :String;
    Const RaiseError :Boolean = False) :String;
  Var
    FullPath :Array [0..Max_Path] Of Char;
  Begin
    If SearchPath (Nil, PChar (Path), Nil, Length (FullPath), FullPath,
    GHDUPChar) > 0 Then
      Result := FullPath
    Else
      If RaiseError Then
        ghRaiseLastError ('%s: "' + Path + '".')
      Else
        Result := '';
  End;

  Function ghSendMsg (Const Window :THandle; Const ID :Integer) :Integer;
  Begin
    Result := SendMessage (Window, ID, 0, 0);
  End;

  Function ghSetBit (Const Buffer :Pointer; Const Bit :Integer;
    Const Value :Boolean = True) :Boolean; Overload;
  Var
    Bytes :PByteArray Absolute Buffer;
  Begin
    With ghDivMod (Bit, 8) Do
    Begin
      Result := ghBitOn (Bytes [HighValue], LowValue);

      If Value Then
        Bytes [HighValue] := Bytes [HighValue] Or (1 Shl LowValue)
      Else
        Bytes [HighValue] := Bytes [HighValue] And Not (1 Shl LowValue);
    End;
  End;

  Function ghSetChr (Var Str :String; Const Index :Integer;
    Const Value :Char) :Boolean;
  Begin
    { NOTE: A direct character assignment in a string always calls to
      UniqueString, ghSetChr first checks if specified character matches
      the character in the given position (but it does not check the
      validity of Index). }

    If Value <> Str [Index] Then
    Begin
      Str [Index] := Value;
      Result := True;
    End
    Else
      Result := False;
  End;

  Function ghSetChrSafe (Var Str :String; Const Index :Integer;
    Const Value :Char) :Boolean;
  Begin
    Result := ghIsIndex (Index, Str) And ghSetChr (Str, Index, Value);
  End;

  Procedure ghSetIcon (Var Params :TMsgBoxParams; Const Icon :PChar;
    Const Module :THandle = 0);
  Begin
    If Module <> 0 Then
      Params.hInstance := Module
    Else
      Params.hInstance := ghDefaultIconModule (Icon);

    Params.dwStyle := ghFlagIcon (Params.dwStyle, mb_UserIcon);
    Params.lpszIcon := Icon;
  End;

  Function ghSetInt (Const Ref :Pointer; Const Value :Integer = 0)
    :Boolean;
  Begin
    Result := Ref <> Nil;

    If Result Then
      PInteger (Ref)^ := Value;
  End;

  Function ghSetLower (Var Str :String; Const Index :Integer = 1)
    :Boolean;
  Begin
    Result := ghSetChr (Str, Index, ghLower (Str [Index]));
  End;

  Function ghSetLowerSafe (Var Str :String; Const Index :Integer = 1)
    :Boolean;
  Begin
    Result := ghIsIndex (Index, Str) And ghSetLower (Str, Index);
  End;

  Function ghSetPtr (Const Ref :Pointer; Const Value :Pointer = Nil)
    :Boolean;
  Begin
    Result := ghSetInt (Ref, Integer (Value));///change for Win64
  End;

  Procedure ghSetSubcomponent (Const Component :TComponent;
    Const Name :String);
  Begin
    Component.SetSubComponent (True);
    Component.Name := Name;
  End;

  Function ghSetUpper (Var Str :String; Const Index :Integer = 1)
    :Boolean;
  Begin
    Result := ghSetChr (Str, Index, ghUpper (Str [Index]));
  End;

  Function ghSetUpperSafe (Var Str :String; Const Index :Integer = 1)
    :Boolean;
  Begin
    Result := ghIsIndex (Index, Str) And ghSetUpper (Str, Index);
  End;

  Function ghSimple (Const Value :Char) :Char;
  Begin
    Result := ghComposition (Value) [1];
  End;

  Function ghSimpleStr (Const Value :String;
    Const DiacriticIndex :Integer) :String; Overload;
  Begin
    Result := Value;
    ghSimplifySafe (Result, DiacriticIndex);
  End;

  Function ghSimpleStr (Const Value :String) :String; Overload;
  Var
    I :Integer;
  Begin
    Result := Value;

    For I := 1 To Length (Result) Do
      ghSetChr (Result, I, ghSimple (Result [I]));
  End;

  Function ghSimpleStrUpper (Const Value :String) :String;
  Begin
    Result := ANSIUpperCase (ghSimpleStr (Value));
  End;

  Function ghSimpleUpper (Const Value :Char) :Char;
  Begin
    Result := ghUpper (ghSimple (Value));
  End;

  Function ghSimplify (Var Str :String; Const Index :Integer) :Boolean;
  Begin
    Result := ghSetChr (Str, Index, ghSimple (Str [Index]));
  End;

  Function ghSimplifySafe (Var Str :String; Const Index :Integer) :Boolean;
  Begin
    Result := ghIsIndex (Index, Str) And ghSimplify (Str, Index);
  End;

  Function ghSplit (Const Value :String; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False) :TStringDynArray;
  Var
    Count, Index :Integer;
    SubStr :String;
  Begin
    Result := Nil;

    If Value = '' Then
      Exit;

    If Separator = '' Then
    Begin
      SetLength (Result, 1);
      Result [0] := Value;
    End
    Else
    Begin
      SetLength (Result, ghCountStr (Value, Separator, ghscOrdinal) +
        1);

      If Length (Result) = 1 Then
        Result [0] := Value
      Else
      Begin
        Index := 1;
        Count := 0;

        Repeat
          SubStr := ghLeft (Value, Separator, Index, ghscOrdinal);

          If (SubStr <> '') Or IncludeEmpties Then
          Begin
            Result [Count] := SubStr;
            Inc (Count);
          End;

          Inc (Index, Length (SubStr) + Length (Separator));
        Until Index > Length (Value) + 1;

        If Count < Length (Result) Then
          SetLength (Result, Count);
      End;
    End;
  End;

  Function ghSplitInts (Const Value :String; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False) :TIntegerDynArray;
  Var
    I :Integer;
    Strs :TStringDynArray;
  Begin
    Strs := ghSplit (Value, Separator, IncludeEmpties);
    SetLength (Result, Length (Strs));

    For I := 0 To High (Strs) Do
      Result [I] := ghInt (Strs [I]);
  End;

  Function ghSplitLast (Const Value :String; Const Separator :Char = ' ';
    Const IncludeEmpties :Boolean = False) :String;
  Var
    I, EndIndex :Integer;
  Begin
    EndIndex := Length (Value);

    For I := EndIndex DownTo 1 Do
      If Value [I] = Separator Then
        If I < EndIndex Then
        Begin
          Result := ghCopy (Value, I + 1, EndIndex);
          Exit;
        End
        Else  // I = EndIndex
          If IncludeEmpties Then
          Begin
            Result := '';
            Exit;
          End
          Else
            Dec (EndIndex);

    Result := LeftStr (Value, EndIndex);
  End;

  Function ghSQLDate (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (GHFMSQLDate, Value);
  End;

  Function ghSQLDate :String; Overload;
  Begin
    Result := ghSQLDate (Date);
  End;

  Function ghSQLDateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (GHFMSQLDateTime, Value);
  End;

  Function ghSQLDateTime :String; Overload;
  Begin
    Result := ghSQLDateTime (Now);
  End;

  Function ghSQLTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (GHFMSQLTime, Value);
  End;

  Function ghSQLTime :String; Overload;
  Begin
    Result := ghSQLTime (Time);
  End;

  Function ghSQLValue (Const Value :Variant) :String;
  Begin
    If ghMoreNull (Value) Then
      If VarIsStr (Value) Then
        Result := QuotedStr (Value)
      Else
        If VarIsType (Value, varDate) Then
          If Frac (Value) = 0 Then
            Result := ghQuotedSQLDate (Value)
          Else
            Result := ghQuotedSQLDateTime (Value)
        Else
          Result := Value
    Else
      Result := 'Null'
  End;

  Function ghSQLValues (Const Values :Variant) :TStringDynArray;
  Var
    I :Integer;
  Begin
    If VarIsArray (Values) Then
    Begin
      I := ghHigh (Values);
      SetLength (Result, I + 1);

      For I := 0 To I Do
        Result [I] := ghSQLValue (Values [I]);
    End
    Else
    Begin             
      SetLength (Result, 1);
      Result [0] := ghSQLValue (Values);
    End;
  End;

  Function ghStarts (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
  Begin
    Result := (Value <> '') And (SubStr <> '') And
      ghEquals (LeftStr (Value, Length (SubStr)), SubStr, CompareType);
  End;

  Function ghStartsWith (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
  Begin
    Result := (Length (Value) > Length (SubStr)) And
      ghStarts (Value, SubStr, CompareType);
  End;

  Function ghStdDateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (ghfmStdDateTime, Value);
  End;

  Function ghStdDateTime :String; Overload;
  Begin
    Result := ghStdDateTime (Now);
  End;

  Function ghStr (Const Value, MinLength :Integer;
    Const PadChr :Char = '0') :String; Overload;
  Var
    AResult :PChar Absolute Result;
    I :Integer;
  Begin
    Str (Value:MinLength, Result);

    If PadChr = ' ' Then
      Exit;

    I := 0;

    While AResult [I] = ' ' Do
    Begin
      AResult [I] := PadChr;  // Direct writing on the unique string Result
      Inc (I);
    End;
  End;

  Function ghStr (Const Value :Pointer) :String; Overload;
  Begin
    Result := Format ('%p', [Value]);
  End;

  Function ghStrs (Const Values :Array Of String) :TStringDynArray;
    Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, Length (Values));

    For I := 0 To High (Values) Do
      Result [I] := Values [I];
  End;

  Function ghStrs (Const List :TStrings) :TStringDynArray; Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, List.Count);

    For I := 0 To List.Count - 1 Do
      Result [I] := List [I];
  End;

  Function ghStyle (Const Window :THandle) :Integer;
  Begin
    Result := GetWindowLong (Window, gwl_Style);
  End;

  Function ghSuffixed (Const Value :String; Const Suffix :Char;
    Const AllowEmpty :Boolean = False) :String; Overload;
  Begin
    If Value = '' Then
      If AllowEmpty Then
        Result := Suffix
      Else
        Result := ''
    Else
      If Value [Length (Value)] = Suffix Then
        Result := Value
      Else
        Result := Value + Suffix;
  End;

  Function ghSuffixed (Const Value, Suffix :String;
    Const AllowEmpty :Boolean = False;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    If Value = '' Then
      If AllowEmpty Then
        Result := Suffix
      Else
        Result := ''
    Else
      If ghEnds (Value, Suffix, CompareType) Then
        Result := Value
      Else
        Result := Value + Suffix;
  End;

  Function ghSwapChrs (Const Value :TghTwoChrs) :TghTwoChrs;
  Begin
    Result.Value1 := Value.Value2;
    Result.Value2 := Value.Value1;
  End;

  Function ghSwapWords (Value :Integer) :Integer;
  Asm
    Mov ECX, EAX  // ECX := Value
    ShL ECX, 16   // ECX := ECX ShL 16
    ShR EAX, 16   // Value := Value ShR 16
    Or  EAX, ECX  // Result := Value Or ECX
  End;

  Function ghSwitch (Const Value, OffMask, OnMask :Integer) :Integer;
  Begin
    Result := ghTurnOff (Value, OffMask) Or OnMask;
  End;

  Function ghTempDir :String;
  Var
    Buffer :Array [0..Max_Path] Of Char;
  Begin
    SetString (Result, Buffer, GetTempPath (SizeOf (Buffer), Buffer));

    // C:\XXX\ -> C:\XXX, T:\ -> T:\
    If Length (Result) > 3 Then
      Result := ghDeleteRight (Result);
  End;

  Function ghTempPath (Const NamePrefix :String) :String;
  Var
    Buffer :Array [0..Max_Path] Of Char;
  Begin
    If GetTempFileName (Pointer (ghTempDir), PChar (NamePrefix), 0,
    Buffer) = 0 Then
      Result := ''
    Else
      Result := Buffer;
  End;

  Function ghTernary (Const Value :Boolean) :TghTernary;
  Begin
    Result := Succ (TghTernary (Value));
  End;

  Function ghTheExe (Const Path :String) :Boolean;
  Begin
    Result := ANSISameText (Path, ghExe);
  End;

  Function ghTheModule (Const Path :String) :Boolean;
  Begin
    Result := ANSISameText (Path, ghModulePath);
  End;

  Function ghTo (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;
  Var
    I :Integer;
  Begin
    I := ghIndex (Value, Chr, StartIndex);

    If I > 0 Then
      Result := ghCopy (Value, StartIndex, I)
    Else
      Result := '';
  End;

  Function ghTo (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Var
    I :Integer;
  Begin
    I := ghPosRight (Value, SubStr, StartIndex, CompareType);

    If I > 1 Then
      Result := ghCopy (Value, StartIndex, I - 1)
    Else
      Result := '';
  End;

  Function ghTo (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;
  Begin
    Result := ghTo (Value, SubStr, 1, CompareType);
  End;

  Function ghTryPathSubDir (Const Path, SubDir :String) :String;
  Begin
    Result := ghTrySubDir (ghDir (Path), SubDir);
  End;

  Function ghTrySubDir (Const Dir, SubDir :String) :String;
  Begin
    Result := ghConcatPaths (Dir, SubDir);

    If Not DirectoryExists (Result) Then
      Result := Dir;
  End;

  Function ghTurnOff (Const Value, Mask :Integer) :Integer;
  Begin
    Result := Value And Not Mask;
  End;

  Function ghTwoInts (Const Value1, Value2 :Integer) :TghTwoInts; Overload;
  Begin
    Result.Value1 := Value1;
    Result.Value2 := Value2;
  End;

  Function ghTwoInts (Const Values :Variant; Const StartIndex :Integer = 0)
    :TghTwoInts; Overload;
  Begin
    Result.Value1 := ghInt (Values [StartIndex]);
    Result.Value2 := ghInt (Values [StartIndex + 1]);
  End;

  Function ghType (Const Value :Char; Const Info :Integer = ct_CType1)
    :Word;
  Begin
    If Not GetStringTypeEx (SysLocale.DefaultLCID, Info, @Value, 1,
    Result) Then
      Result := 0;
  End;

  Function ghUnitDiff (Const Groups1, Units1, Groups2, Units2 :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;
  Begin
    Result := ghUnits (Groups1, Units1, MinUnit, MaxUnit) -
      ghUnits (Groups2, Units2, MinUnit, MaxUnit);
  End;

  Function ghUnitDiff (Const Value1, Value2 :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;
  Begin
    Result := ghUnits (Value1, MinUnit, MaxUnit) -
      ghUnits (Value2, MinUnit, MaxUnit);
  End;

  Function ghUnits (Const Groups, Units :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;
  Begin
    Result := (Groups * (MaxUnit - Pred (MinUnit))) + (Units - MinUnit);
  End;

  Function ghUnits (Const Value :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;
  Begin
    Result := ghUnits (Value.HighValue, Value.LowValue, MinUnit, MaxUnit);
  End;

  Function ghUpper (Const Value :Char) :Char; Overload;
  Begin
    Result := Char (CharUpper (Pointer (Value)));
  End;

  Function ghUpper (Const Value :String; Const Index :Integer = 1) :Char;
    Overload;
  Begin
    Result := ghUpper (ghChr (Value, Index));
  End;

  Function ghUppered (Const Value :String; Const Index :Integer = 1)
    :String;
  Begin
    Result := Value;
    ghSetUpperSafe (Result, Index);
  End;

  Function ghValid (Const Value :String; Const Chrs :TSysCharSet) :Boolean;
  Var
    I :Integer;
  Begin
    Result := True;

    For I := 1 To Length (Value) Do
      If Not (Value [I] In Chrs) Then
      Begin
        Result := False;
        Exit;
      End;
  End;

  Function ghValue (Const AVar :Variant; Const Pos :Integer) :Variant;
    Overload;
  Begin
    If VarIsArray (AVar) Then
      Result := AVar [ghLow (AVar) + Pos]
    Else
      Result := AVar;
  End;

  Function ghVar (Const Value :String) :Variant; Overload;
  Begin
    If Value = '' Then
      Result := Null
    Else
      Result := Value;
  End;

  Function ghVar (Const Value :Pointer) :Variant; Overload;
  Begin
    If Value = Nil Then
      Result := Null
    Else
      Result := Cardinal (Value);
  End;

  Function ghVar (Const Value :TVarRec) :Variant; Overload;
  Begin
    With Value Do
      Case VType Of
        vtInteger    : Result := VInteger;
        vtBoolean    : Result := VBoolean;
        vtChar       : Result := VChar;
        vtExtended   : Result := VExtended^;
        vtString     : Result := VString^;
        vtPointer    : Result := ghVar (VPointer);
        vtPChar      : Result := String (VPChar);
        vtObject     : Result := ghVar (VObject);
        vtClass      : Result := ghVar (VClass);
        vtWideChar   : Result := VWideChar;
        vtPWideChar  : Result := WideString (VPWideChar);
        vtANSIString : Result := String (VANSIString);
        vtCurrency   : Result := VCurrency^;
        vtVariant    : Result := VVariant^;
        vtInterface  : Result := IInterface (VInterface);
        vtWideString : Result := WideString (VWideString);
        vtInt64      : Result := VInt64^;
        Else
          Result := Null;
      End;
  End;

  Function ghVar (Const Values :Array Of Const) :Variant; Overload;
  Var
    I :Integer;
  Begin
    Result := ghVarArr (Length (Values), False);

    For I := 0 To High (Values) Do
      ghAssign (Result, I, ghVar (Values [I]));
  End;

  Function ghVarArr (Const Size :Integer;
    Const AType :TVarType = varVariant;
    Const AllowLessThanTwo :Boolean = True) :Variant; Overload;
  Begin
    If (Size > 1) Or AllowLessThanTwo Then
      Result := VarArrayCreate ([0, Size - 1], AType)
    Else
      Result := Unassigned;
  End;

  Function ghVarArr (Const Size :Integer;
    Const AllowLessThanTwo :Boolean) :Variant; Overload;
  Begin
    Result := ghVarArr (Size, varVariant, AllowLessThanTwo);
  End;

  Function ghWantAllKeys (Const Window :THandle) :Boolean;
  Begin
    Result := ghHasBit (ghDialogCode (Window), dlgc_WantAllKeys);
  End;

  Function ghWantTab (Const Window :THandle) :Boolean;
  Begin
    Result := ghDialogCodeTab (ghDialogCode (Window));
  End;

  Function ghZero (Const Buffer :Pointer; Const Bytes :Integer) :Integer;
  Begin
    ZeroMemory (Buffer, Bytes);
    Result := Bytes;
  End;

End.

