{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFSavePoint.pas - TghSavePoint class unit.                             }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFSavePoint;  { Save Point }

Interface

  Uses
    GHFCustomSavePoint;

  Type
    TghSavePoint = Class;
    
    { Save Point Event type }
    TghSavePointEvent = Procedure (Point :TghSavePoint) Of Object;

    { Save Point class }
    TghSavePoint = Class (TghCustomSavePoint)
      Protected
        { Overridden methods }
        Procedure InternalConfirm; Override;
        Procedure InternalDestroy; Override;
        Procedure InternalRestore; Override;
        Procedure InternalSave; Override;
      Public
        { Fields }
        Data :Pointer;
    End;

    { Save Points class }
    TghSavePoints = Class (TghCustomSavePoints)
      Protected
        { Fields }
        FOnConfirm :TghSavePointEvent;
        FOnDestroy :TghSavePointEvent;
        FOnRestore :TghSavePointEvent;
        FOnSave :TghSavePointEvent;

        { Overridden methods }
        Function NewPointClass :TghCustomSavePointClass; Override;
      Published
        { Properties }
        Property OnConfirm :TghSavePointEvent Read FOnConfirm
          Write FOnConfirm;
        Property OnDestroy :TghSavePointEvent Read FOnDestroy
          Write FOnDestroy;
        Property OnRestore :TghSavePointEvent Read FOnRestore
          Write FOnRestore;
        Property OnSave :TghSavePointEvent Read FOnSave Write FOnSave;
    End;

Implementation

  { TghSavePoint }

  { Overridden protected methods }

  Procedure TghSavePoint.InternalConfirm;
  Begin
    If Assigned (TghSavePoints (Owner).OnConfirm) Then
      TghSavePoints (Owner).OnConfirm (Self);
  End;

  Procedure TghSavePoint.InternalDestroy;
  Begin
    If Assigned (TghSavePoints (Owner).OnDestroy) Then
      TghSavePoints (Owner).OnDestroy (Self);
  End;

  Procedure TghSavePoint.InternalRestore;
  Begin
    If Assigned (TghSavePoints (Owner).OnRestore) Then
      TghSavePoints (Owner).OnRestore (Self);
  End;

  Procedure TghSavePoint.InternalSave;
  Begin
    If Assigned (TghSavePoints (Owner).OnSave) Then
      TghSavePoints (Owner).OnSave (Self);
  End;

  { TghSavePoints }

  { Protected overridden methods }

  Function TghSavePoints.NewPointClass :TghCustomSavePointClass;
  Begin
    Result := TghSavePoint;
  End;
    
End.
