{}
{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFWebBrowser.pas - TghWebBrowser class unit.                           }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFWebBrowser;  { Web Browser }

Interface

  Uses
    ShDocVw, ActiveX;

  Type
    TghWebBrowser = Class (TWebBrowser)
      Protected
        { Fields }
        WaitingForDocComplete :Boolean;

        { Static methods }
        Function GetDoc :OLEVariant;
        Function GetIdle :Boolean;

        { Virtual methods }
        Function InternalElement (Const AElement :String) :OLEVariant;
          Virtual;

        { Overridden methods }
        Procedure InvokeEvent (DispID :TDispID; Var Params :TDispParams);
          Override;
      Public
        { Static methods }
        Function Cell (Const ATable :String; Const ARow, Column :Integer)
          :OLEVariant;
        Function CellText (Const ATable :String;
          Const ARow, Column :Integer) :OLEVariant;
        Procedure Click (Const AElement :String);
        Function ClassElement (Const ADoc :OLEVariant;
          Const AClass :String) :OLEVariant; Overload;
        Function ClassElement (Const AClass :String) :OLEVariant; Overload;
        Function Element (Const AElement :String) :OLEVariant;
        Procedure EnterData (Const Elements :String; Const Values :Variant;
          Const SubmitElement :String;
          Const AWaitForIdle :Boolean = False); Overload;
        Procedure EnterData (Const Elements :String; Const Values :Variant;
          Const BlurElements :Boolean; Const AWaitForIdle :Boolean = False);
          Overload;
        Procedure ForceBlur (Const AElement :OLEVariant); Overload;
        Procedure ForceBlur (Const AElement :String); Overload;
        Function IDElement (Const ADoc :OLEVariant; Const ID :String)
          :OLEVariant; Overload;
        Function IDElement (Const ID :String) :OLEVariant; Overload;
        Procedure NavigateWait (Const URL :String);
        Function Row (Const ATable :String; Const Index :Integer)
          :OLEVariant;
        Function TagNameElement (Const ADoc :OLEVariant;
          Const TagName :String) :OLEVariant; Overload;
        Function TagNameElement (Const TagName :String) :OLEVariant;
          Overload;

        { Virtual methods }
        Procedure EnterData (Const Elements :String;
          Const AValues :Variant; Const BlurElements :Boolean = False;
          Const SubmitElement :String = '';
          Const AWaitForIdle :Boolean = False); Overload; Virtual;
        Procedure WaitForIdle; Virtual;

        { Properties }
        Property Doc :OLEVariant Read GetDoc;
        Property Idle :Boolean Read GetIdle;
    End;

Implementation

  Uses
    Variants, GHFUtils, GHFRTL, Forms;

  { TghWebBrowser }

  { Protected static methods }

  Function TghWebBrowser.GetDoc :OLEVariant;
  Begin
    Result := Document;
  End;

  Function TghWebBrowser.GetIdle :Boolean;
  Begin
    Result := Not WaitingForDocComplete And Not Busy And
      (ReadyState = ReadyState_Complete);
  End;

  { Protected virtual methods }

  Function TghWebBrowser.InternalElement (Const AElement :String)
    :OLEVariant;
  Begin
    Result := IDElement (AElement);

    If VarIsClear (Result) Then
      Result := ClassElement (AElement);
  End;

  { Protected overridden methods }

  Procedure TghWebBrowser.InvokeEvent (DispID :TDispID;
    Var Params :TDispParams);
  Begin
    If DispID = 250 Then  // OnBeforeNavigate2
      WaitingForDocComplete := True
    Else
      If (DispID = 259 { OnDocumentComplete }) And
      (ReadyState = ReadyState_Complete) Then
        WaitingForDocComplete := False;

    Try
      Inherited InvokeEvent (DispID, Params);
    Except
      WaitingForDocComplete := False;
      Raise;
    End;
  End;

  { Public static methods }

  Function TghWebBrowser.Cell (Const ATable :String;
    Const ARow, Column :Integer) :OLEVariant;
  Begin
    Result := Row (ATable, ARow).Cells.Item (Column);
  End;

  Function TghWebBrowser.CellText (Const ATable :String;
    Const ARow, Column :Integer) :OLEVariant;
  Begin
    {Result := Trim (Cell (ATable, ARow, Column).InnerText);}{}
  End;

  Procedure TghWebBrowser.Click (Const AElement :String);
  Begin
    Element (AElement).Click;
    WaitForIdle;
  End;
  
  Function TghWebBrowser.ClassElement (Const ADoc :OLEVariant;
    Const AClass :String) :OLEVariant;
  Var
    I :Integer;
  Begin
    Result := ADoc.GetElementsByTagName ('*');

    If Result.Length > 0 Then
    Begin
      For I := 0 To Result.Length - 1 Do
        If Result.Item (I).ClassName = AClass Then
        Begin
          Result := Result.Item (I);
          Exit;
        End;

      If ADoc.Frames.Length > 0 Then
        For I := 0 To ADoc.Frames.Length - 1 Do
        Begin
          Result := ClassElement (ADoc.Frames.Item (I).Document, AClass);

          If Not VarIsClear (Result) Then
            Exit;
        End;
    End;

    Result := Unassigned;
  End;

  Function TghWebBrowser.ClassElement (Const AClass :String) :OLEVariant;
  Begin
    Result := ClassElement (Doc, AClass);
  End;

  Function TghWebBrowser.Element (Const AElement :String) :OLEVariant;
  Begin
    Result := InternalElement (AElement);

    If VarIsClear (Result) Then
      ghRaise ('Element not found in HTML document: "%s".', [AElement]);
  End;

  Procedure TghWebBrowser.EnterData (Const Elements :String;
    Const Values :Variant; Const SubmitElement :String;
    Const AWaitForIdle :Boolean = False);
  Begin
    EnterData (Elements, Values, False, SubmitElement, AWaitForIdle);
  End;

  Procedure TghWebBrowser.EnterData (Const Elements :String;
    Const Values :Variant; Const BlurElements :Boolean;
    Const AWaitForIdle :Boolean = False);
  Begin
    EnterData (Elements, Values, BlurElements, '', AWaitForIdle);
  End;

  Procedure TghWebBrowser.ForceBlur (Const AElement :OLEVariant);
  Begin
    AElement.Focus;
    AElement.Blur;
  End;

  Procedure TghWebBrowser.ForceBlur (Const AElement :String);
  Begin
    ForceBlur (Element (AElement));
  End;

  Function TghWebBrowser.IDElement (Const ADoc :OLEVariant;
    Const ID :String) :OLEVariant;
  Var
    I :Integer;
  Begin
    Result := ADoc.GetElementByID (ID);

    If VarIsClear (Result) Then
      For I := 0 To ADoc.Frames.Length - 1 Do
      Begin
        Result := IDElement (ADoc.Frames.Item (I).Document, ID);

        If Not VarIsClear (Result) Then
          Exit;
      End;
  End;

  Function TghWebBrowser.IDElement (Const ID :String) :OLEVariant;
  Begin
    Result := IDElement (Doc, ID);
  End;

  Procedure TghWebBrowser.NavigateWait (Const URL :String);
  Begin
    Navigate (URL);
    WaitForIdle;
  End;

  Function TghWebBrowser.Row (Const ATable :String; Const Index :Integer)
    :OLEVariant;
  Begin
    Result := Element (ATable).Rows.Item (Index);
  End;

  Function TghWebBrowser.TagNameElement (Const ADoc :OLEVariant;
    Const TagName :String) :OLEVariant;
  Var
    I :Integer;
  Begin
    Result := ADoc.GetElementsByTagName (TagName);

    If Result.Length > 0 Then
      Result := Result.Item (0)
    Else
      If ADoc.Frames.Length > 0 Then
        For I := 0 To ADoc.Frames.Length - 1 Do
        Begin
          Result := TagNameElement (ADoc.Frames.Item (I).Document,
            TagName);

          If Not VarIsClear (Result) Then
            Exit;
        End
      Else
        Result := Unassigned;
  End;

  Function TghWebBrowser.TagNameElement (Const TagName :String)
    :OLEVariant;
  Begin
    Result := TagNameElement (Doc, TagName);
  End;

  { Public virtual methods }

  Procedure TghWebBrowser.EnterData (Const Elements :String;
    Const AValues :Variant; Const BlurElements :Boolean = False;
    Const SubmitElement :String = ''; Const AWaitForIdle :Boolean = False);
  Var
    AElement :OLEVariant;
    I :Integer;
  Begin
    With ghSplitList (Elements, ';') Do
      Try
        For I := 0 To Count - 1 Do
        Begin
          AElement := Element (Strings [I]);

          If ghPosStr (['checkbox', 'radio'], AElement.Type) > -1 Then
            AElement.Checked := ghBool (ghValue (AValues, I))
          Else
            AElement.Value := ghValue (AValues, I);

          If BlurElements Then
            ForceBlur (AElement);
        End;
      Finally
        Free;
      End;

    If SubmitElement <> '' Then
      Element (SubmitElement).Click;

    If AWaitForIdle Then
      WaitForIdle;
  End;

  Procedure TghWebBrowser.WaitForIdle;
  Begin
    Repeat
      Forms.Application.ProcessMessages;
    Until Idle;
  End;

End.

